/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.actionHandler = exports.assertiveId = exports.politeId = void 0;

var _constants = require("../constants");

var _uiConfig = require("@servicenow/ui-config");

var _log = _interopRequireDefault(require("../log"));

var _uiInternal = require("@servicenow/ui-internal");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var politeId = 'SEISMIC_ARIA_LIVE_REGION_POLITE';
exports.politeId = politeId;
var assertiveId = 'SEISMIC_ARIA_LIVE_REGION_ASSERTIVE';
exports.assertiveId = assertiveId;

var applyAriaLiveRegionAttributes = function applyAriaLiveRegionAttributes(el, type, id) {
  var live = type === _constants.ARIA_LIVE_ASSERTIVE || type === _constants.ARIA_LIVE_POLITE ? type : _constants.ARIA_LIVE_POLITE;
  el.setAttribute('id', id || (type === _constants.ARIA_LIVE_ASSERTIVE ? assertiveId : politeId));
  el.setAttribute('aria-live', live);
  el.setAttribute('aria-atomic', true);
  applyStyles(el);
};

var applyStyles = function applyStyles(el) {
  el.style.position = 'absolute';
  el.style.overflow = 'hidden';
  el.style.width = '1px';
  el.style.height = 'auto';
  el.style.margin = 0;
  el.style.border = 'none';
  el.style.padding = 0;
  el.style.whiteSpace = 'nowrap';
  el.style.clip = 'rect(0 0 0 0)';
  el.style.clipPath = 'inset(50%)';
};

var politeTimeout = (0, _uiConfig.getConfigItem)(_uiConfig.configConstants.ARIA_LIVE_POLITE_TIMEOUT);
var assertiveTimeout = (0, _uiConfig.getConfigItem)(_uiConfig.configConstants.ARIA_LIVE_ASSERTIVE_TIMEOUT);

var createAriaLiveRegionEl = function createAriaLiveRegionEl(id, type) {
  var el = document.createElement('SPAN');
  applyAriaLiveRegionAttributes(el, type, id);
  document.body.appendChild(el);
  return el;
};

var getAriaLiveRegionEl = function getAriaLiveRegionEl(type) {
  var id = type === _constants.ARIA_LIVE_ASSERTIVE ? assertiveId : politeId;
  return document.querySelector("#".concat(id)) || createAriaLiveRegionEl(id, type);
};

var actionHandler = {
  stopPropagation: true,
  effect: function effect(_ref) {
    var host = _ref.host,
        _ref$action$payload = _ref.action.payload,
        message = _ref$action$payload.message,
        _ref$action$payload$t = _ref$action$payload.type,
        type = _ref$action$payload$t === void 0 ? _constants.ARIA_LIVE_POLITE : _ref$action$payload$t;

    if (type !== _constants.ARIA_LIVE_ASSERTIVE && type !== _constants.ARIA_LIVE_POLITE) {
      process.env.NODE_ENV !== 'production' && (0, _log["default"])("\"ARIA_LIVE_REGION_UPDATE_REQUESTED\" was dispatched with action.type: ".concat(type, ". Only action.type of \"assertive\" or \"polite\" is allowed."), {
        level: _uiInternal.allLogTypes.WARN,
        host: host,
        origin: 'dispatch'
      });
      return;
    }

    var ariaLiveRegionEl = getAriaLiveRegionEl(type);
    var textNode = document.createTextNode(message);
    ariaLiveRegionEl.appendChild(textNode);
    setTimeout(function () {
      if (textNode.parentNode === ariaLiveRegionEl) ariaLiveRegionEl.removeChild(textNode);
    }, type === _constants.ARIA_LIVE_ASSERTIVE ? assertiveTimeout : politeTimeout);
  }
};
exports.actionHandler = actionHandler;
//# sourceMappingURL=ariaLiveRegion.js.map