/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addActionHandlers = addActionHandlers;

var _utils = require("../utils");

var _uiInternal = require("@servicenow/ui-internal");

var _onAction = require("./onAction");

var _onActionHoisted = require("./onActionHoisted");

var _customEventActionHandler = require("./customEventActionHandler");

var _utils2 = require("./utils");

var _symbols = require("../symbols");

var _constants = require("../constants");

var _log = _interopRequireDefault(require("../log"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function resolveCaptureHandlerMetaData(interceptors) {
  var interceptorCount = interceptors.length;

  for (var i = 0; i < interceptorCount; i++) {
    var _interceptors$i = interceptors[i],
        capture = _interceptors$i.capture,
        eventHandler = _interceptors$i.eventHandler;
    if (capture !== undefined && eventHandler !== undefined) return {
      capture: capture,
      eventHandler: eventHandler
    };
  }

  return {
    capture: false,
    eventHandler: false
  };
}

function addActionHandlers(host, actions) {
  var handlers = [];
  var hoisted = (0, _utils2.isHoisted)(host);
  var hostVariables = (0, _utils.getHostVariables)(host);

  if (actions[undefined]) {
    (0, _log["default"])('You are registering an action handler for a type that is: undefined.  We will send undefined actions to the upside down place in the next release.', {
      level: _uiInternal.allLogTypes.WARN,
      host: host,
      origin: 'addActionHandlers'
    });
  }

  var _loop = function _loop(k) {
    actions[k].forEach(function forEachTarget(interceptors, targetSelector) {
      var _resolveCaptureHandle = resolveCaptureHandlerMetaData(interceptors),
          capture = _resolveCaptureHandle.capture,
          eventHandler = _resolveCaptureHandle.eventHandler;

      var resolvedTarget = targetSelector === 'host' ? host : typeof targetSelector === 'string' ? document.querySelector(targetSelector) : targetSelector;

      if (hoisted) {
        var hoistElem = (0, _utils2.getHoistTarget)(hostVariables);

        if (hoistElem.tagName === _constants.SEISMIC_HOIST_TAG_NAME) {
          var hoistElemVariables = (0, _utils.getHostVariables)(hoistElem);
          handlers.push((0, _customEventActionHandler.addActionHandler)(hoistElem, targetSelector, host, k, capture, eventHandler, _onActionHoisted.onActionHoisted));
          hoistElemVariables[_symbols.RETARGETED_ACTION_TYPES] = [].concat(_toConsumableArray(hoistElemVariables[_symbols.RETARGETED_ACTION_TYPES]), [k]);
        }
      }

      handlers.push((0, _customEventActionHandler.addActionHandler)(resolvedTarget, targetSelector, host, k, capture, eventHandler, _onAction.onAction));
      hostVariables[_symbols.TARGETED_ACTION_TYPES] = [].concat(_toConsumableArray(hostVariables[_symbols.TARGETED_ACTION_TYPES]), [k]); // eslint-disable-line no-param-reassign
    });
  };

  for (var k in actions) {
    _loop(k);
  }

  return handlers;
}
//# sourceMappingURL=addActionHandlers.js.map