/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDispatcher = createDispatcher;

var _uiMetrics = require("@servicenow/ui-metrics");

var _uiInternal = require("@servicenow/ui-internal");

var _customEventDispatcher = require("./customEventDispatcher");

var _utils = require("./utils");

var _symbols = require("../symbols");

var _utils2 = require("../utils");

var _log = _interopRequireDefault(require("../log"));

var _constants = require("../constants");

var _addActionHandlerForUnknownType = require("./addActionHandlerForUnknownType");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function createDispatcher(host) {
  var dispatcher = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _customEventDispatcher.customEventDispatcher;
  var hostVariables = (0, _utils2.getHostVariables)(host);
  return function dispatch(type) {
    var payload = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var error = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

    if (type === undefined) {
      (0, _log["default"])("You are dispatching an action that is type: ".concat(type, ".  We will send undefined actions to the upside down place in the next release."), {
        level: _uiInternal.allLogTypes.WARN,
        host: host,
        origin: 'dispatch'
      });
    }

    if ((0, _utils.skipForDetachedComponent)(host, type, payload)) return;

    if ((0, _utils.isStoreUpdate)(type)) {
      hostVariables[_symbols.HELPERS_SCHEDULED].updateState(payload, _defineProperty({}, _constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED, true), (0, _uiMetrics.getInteractionId)(meta));

      return (0, _log["default"])("".concat(type, " is a private action type and has been deprecated. Please use updateState."), {
        host: host,
        origin: 'dispatch'
      });
    }

    if ((0, _utils.isInternalActionType)(type, meta.internal)) {
      var _getHostInfo = (0, _uiInternal.getHostInfo)(host),
          tagName = _getHostInfo.tagName,
          componentId = _getHostInfo.componentId;

      (0, _log["default"])("".concat((0, _utils.getInternalActionName)(type), " cannot be dispatched as it is an internal action type"), {
        tagName: tagName,
        componentId: componentId,
        host: host,
        origin: 'dispatch'
      }); // Only return if the type is not one of the following, in order to maintain backwards compatibility

      if (![_constants.PROPERTIES_SET, _constants.DISPATCH_EVENT].includes(type)) return;
    }

    if ((0, _utils.isPropertyUpdate)(type)) return hostVariables[_symbols.HELPERS_IMMEDIATE].updateProperties(payload, (0, _uiMetrics.getInteractionId)(meta));
    (0, _utils.validateActionPayload)(host, type, payload, hostVariables); // TODO: fix
    // eslint-disable-next-line no-param-reassign

    meta = (0, _utils.normalizeActionMeta)(meta, {
      host: host
    });
    var action = {
      type: type,
      payload: meta.retargeted ? payload : (0, _utils.appendToPayload)(host, payload),
      meta: (0, _utils.createMetaData)(host, meta, hostVariables),
      error: error
    };
    (0, _utils2.emitNowUiEvent)(_constants.ACTION_DISPATCHED, {
      host: host,
      action: action
    });
    if (!hostVariables[_symbols.PRESENTATIONAL] && !meta.internal && !(0, _utils.hasActionHandlerForType)(hostVariables, type)) (0, _addActionHandlerForUnknownType.addActionHandlerForUnknownType)(host, type, hostVariables);
    (0, _uiMetrics.mark)(host, (0, _uiMetrics.getInteractionId)(action.meta), _uiMetrics.types.DISPATCH, {
      type: type
    });
    return dispatcher(host, action);
  };
}
//# sourceMappingURL=createDispatcher.js.map