/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addActionHandler = addActionHandler;

var _utils = require("./utils");

var _symbols = require("../symbols");

var _constants = require("../constants");

var _utils2 = require("../utils");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var allowedPropertyUpdates = ['payload', 'error', 'meta'];

function getPath(meta, host) {
  if (meta.event.path) return meta.event.path;
  var path = [];
  var el = host;
  var parentNode = el.parentNode || el.host;

  while (parentNode) {
    path.push(el);
    el = parentNode;
    parentNode = el.parentNode || el.host;
  }

  return [].concat(path, [document, window]);
}

function getComposedPath(meta, host) {
  var hoistComposedPath = meta[_symbols.HOIST_COMPOSED_PATH];
  if (!hoistComposedPath) return meta.event.composedPath();
  var composedPath = [];

  for (var i = 0; i < hoistComposedPath.length; i++) {
    var el = hoistComposedPath[i];
    if (el.tagName === _constants.SEISMIC_HOIST_TAG_NAME) break;
    composedPath.push(el);
  }

  return [].concat(composedPath, _toConsumableArray(getPath(meta, host)));
}

function addActionHandler(targetEl, targetSelector, host, type, capture, eventHandler, handler) {
  var hostVariables = (0, _utils2.getHostVariables)(host);

  var listener = function listener(event) {
    var config = hostVariables[_symbols.CONFIG];
    var detail = (0, _utils.createEventActionDetails)(type, host, event, capture, eventHandler);
    handler(targetEl, targetSelector, host, type, new Proxy(_objectSpread(_objectSpread({}, detail), {}, {
      stopPropagation: function stopPropagation() {
        return event.stopPropagation();
      }
    }), {
      get: function get(action, key) {
        var payload = action.payload,
            meta = action.meta;

        if (key === 'defaultPrevented') {
          return payload && payload.event ? payload.event.defaultPrevented : event.defaultPrevented;
        }

        if (key === 'preventDefault') {
          return function () {
            return payload && payload.event ? payload.event.preventDefault() : event.preventDefault();
          };
        }

        if (key === 'composedPath') return function () {
          return getComposedPath(meta, host);
        };
        return action[key];
      },
      set: function set(action, key, value) {
        if (allowedPropertyUpdates.includes(key)) {
          action[key] = value; // eslint-disable-line no-param-reassign

          return true;
        }

        return false;
      }
    }));
    if (config && config.shouldStopPropagation[type]) event.stopPropagation();
  };

  targetEl.addEventListener(type, listener, capture);
  return function removeActionHandler() {
    targetEl.removeEventListener(type, listener, capture);
  };
}
//# sourceMappingURL=customEventActionHandler.js.map