/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.customEventDispatcher = customEventDispatcher;

var _uiConfig = require("@servicenow/ui-config");

var _uiInternal = require("@servicenow/ui-internal");

var _uiMetrics = require("@servicenow/ui-metrics");

var _constants = require("../constants");

var _symbols = require("../symbols");

var _utils = require("../utils");

var _scheduler = require("../scheduler");

var _utils2 = require("./utils");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var ENABLE_ACTION_DISPATCHED_HANDLING = _uiConfig.configConstants.ENABLE_ACTION_DISPATCHED_HANDLING;
var shouldApplyRuntimeActionHandlers = (0, _uiInternal.memoize)(function () {
  return (0, _uiConfig.getConfigItem)(ENABLE_ACTION_DISPATCHED_HANDLING) === true;
}, {
  profileName: 'shouldApplyRuntimeActionHandlers'
});
var DISPATCH = _scheduler.types.DISPATCH,
    IMMEDIATE = _scheduler.types.IMMEDIATE;

function addEventGetters(event) {
  // FIXME: is there a better, faster way than this?
  Object.defineProperties(event.detail.meta, {
    event: {
      value: event,
      enumerable: true
    },
    path: {
      get: function get() {
        return event.composedPath().map(function (el) {
          var variables = (0, _utils.getHostVariables)(el);
          return variables && variables[_symbols.COMPONENT_ID];
        }).filter(function (componentId) {
          return componentId;
        });
      }
    }
  });
  return event;
}

function dispatchLegacyEvent(host, detail) {
  var event = new CustomEvent(_constants.DISPATCH_EVENT, {
    composed: true,
    bubbles: true,
    detail: detail
  });
  host.dispatchEvent(addEventGetters(event));
}

function customEventDispatcher(host, _ref) {
  var type = _ref.type,
      payload = _ref.payload,
      meta = _ref.meta,
      error = _ref.error;
  var hostVariables = (0, _utils.getHostVariables)(host);
  var presentational = hostVariables[_symbols.PRESENTATIONAL];
  var shouldBubble = meta.internal && type !== _constants.COMPONENT_PROPERTY_UPDATE_REQUESTED ? false : true;
  var event = new CustomEvent(type, {
    composed: shouldBubble,
    bubbles: shouldBubble,
    cancelable: true,
    detail: {
      type: type,
      payload: payload,
      error: error,
      meta: meta
    }
  });
  (0, _scheduler.schedule)({
    fn: function fn() {
      if (process.env.UNSAFE_SEISMIC_EMIT_CUSTOM_EVENT) {
        dispatchLegacyEvent(host, {
          type: type,
          payload: payload,
          error: error,
          // spread or Object.defineProperties fails because the meta reference is the same
          meta: _objectSpread({}, meta)
        });
      }

      if (!presentational && shouldApplyRuntimeActionHandlers()) (0, _utils2.applyRuntimeActionHandlers)(host, type);
      return host.dispatchEvent(addEventGetters(event));
    },
    meta: {
      el: host,
      interactionId: (0, _uiMetrics.getInteractionId)(meta)
    },
    type: meta[_constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED] ? IMMEDIATE : DISPATCH
  });
  return event;
}
//# sourceMappingURL=customEventDispatcher.js.map