/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.onAction = onAction;

var _uiMetrics = require("@servicenow/ui-metrics");

var _utils = require("../utils");

var _createContext = require("./createContext");

var _symbols = require("../symbols");

var _sandboxEffect = require("./sandboxEffect");

var _constants = require("../constants");

var _normalizeActionHandler = require("../utils/config/normalizeActionHandler");

var _getHostVariables = _interopRequireDefault(require("../utils/getHostVariables"));

var _customEventActionHandler = require("./customEventActionHandler");

var _onActionHoisted = require("./onActionHoisted");

var _utils2 = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

// defer the execution of the origin handler to support preventDefault
function runEffects(fn, isOrigin) {
  return isOrigin ? (0, _utils.beforeNextTick)(function () {
    return fn();
  }) : fn();
}

function onAction(targetEl, targetSelector, host, actionType) {
  var action = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
  var hostVariables = (0, _getHostVariables["default"])(host);
  var _action$type = action.type,
      type = _action$type === void 0 ? actionType : _action$type,
      _action$meta = action.meta,
      meta = _action$meta === void 0 ? {
    path: [],
    eventHandler: false
  } : _action$meta;
  var path = meta.path,
      eventHandler = meta.eventHandler,
      event = meta.event,
      capture = meta.capture;

  var _path = _slicedToArray(path, 1),
      origin = _path[0];

  var isOrigin = hostVariables[_symbols.COMPONENT_ID] === origin;
  if (eventHandler) // interaction ids aren't applied to native events dispacthed by the browser
    (0, _uiMetrics.applyInteractionId)(meta);
  var isImmediate = [_constants.COMPONENT_DISCONNECTED, _constants.COMPONENT_DOM_READY, _constants.COMPONENT_DOM_TREE_READY, _constants.COMPONENT_TREE_RENDERED, _constants.COMPONENT_RENDERED, _constants.COMPONENT_RENDER_REQUESTED, _constants.COMPONENT_PROPERTY_UPDATE_REQUESTED].includes(actionType);

  if (event && !eventHandler) {
    var hoistElm = event.composedPath().find(function (elem) {
      return elem.tagName === _constants.SEISMIC_HOIST_TAG_NAME;
    });
    var hoistElmVariables = (0, _getHostVariables["default"])(hoistElm);

    if (hoistElm && hoistElmVariables && !hoistElmVariables[_symbols.RETARGETED_ACTION_TYPES].some(function (actionType) {
      return actionType === type;
    })) {
      (0, _customEventActionHandler.addActionHandler)(hoistElm, targetSelector, hoistElm.sourceElement, type, capture, eventHandler, _onActionHoisted.onActionHoisted);
      hoistElmVariables[_symbols.RETARGETED_ACTION_TYPES] = [].concat(_toConsumableArray(hoistElmVariables[_symbols.RETARGETED_ACTION_TYPES]), [type]);
    }
  }

  runEffects(function () {
    var interceptors = hostVariables[_symbols.CONFIG].interceptors[type] && hostVariables[_symbols.CONFIG].interceptors[type].has(targetSelector) ? hostVariables[_symbols.CONFIG].interceptors[type].get(targetSelector) : null;
    if (!interceptors || !interceptors.length) return;
    var interceptorCount = interceptors.length;
    if (isOrigin && action.defaultPrevented || !isOrigin && action.propagationStopped) return;
    (0, _utils.emitNowUiEvent)(_constants.ACTION_HANDLED, {
      host: host,
      action: action,
      targetSelector: targetSelector,
      interceptors: interceptors
    });
    if (actionType === _constants.COMPONENT_BOOTSTRAPPED) hostVariables[_symbols.BOOTSTRAPPED_HANDLED] = true; // eslint-disable-line no-param-reassign

    if (actionType === _constants.COMPONENT_CONNECTED) hostVariables[_symbols.CONNECTED_HANDLED] = true; // eslint-disable-line no-param-reassign

    var context = (0, _createContext.createContext)(host, action);
    var ctx = context;

    for (var i = 0; i < interceptorCount; i++) {
      var interceptor = interceptors[i];
      ctx = interceptor.fn(ctx) || ctx;
    }

    var _ctx = ctx,
        effects = _ctx.effects;
    var effectCount = effects.length;

    for (var _i2 = 0; _i2 < effectCount; _i2++) {
      var effect = effects[_i2];

      if (typeof effect === 'function') {
        var normalizedEffect = (0, _normalizeActionHandler.normalizeActionHandler)(effect, host.tagName.toLowerCase());
        (0, _sandboxEffect.sandboxEffect)(normalizedEffect, hostVariables[_symbols.DISPATCH], action, host).apply(void 0, _toConsumableArray(normalizedEffect.args));
        continue;
      }

      if (!effect.descendantOnly || (0, _utils2.wasActionDispatchedByDescendant)(action.composedPath(), host)) (0, _sandboxEffect.sandboxEffect)(effect, hostVariables[_symbols.DISPATCH], action, host).apply(void 0, _toConsumableArray(effect.args));
    }
  }, isOrigin && !eventHandler && !isImmediate);
}
//# sourceMappingURL=onAction.js.map