/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sandboxEffect = sandboxEffect;

var _uiMetrics = require("@servicenow/ui-metrics");

var _uiInternal = require("@servicenow/ui-internal");

var _log = _interopRequireDefault(require("../log"));

var _effects = require("../effects");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function sandboxEffect(effect, dispatch, action, host) {
  var actionType = action.type,
      _action$meta$effectTy = action.meta.effectType,
      effectType = _action$meta$effectTy === void 0 ? '' : _action$meta$effectTy,
      _action$payload = action.payload,
      payload = _action$payload === void 0 ? {} : _action$payload;
  var location = '';
  var details = {};

  if (effectType === _uiInternal.locations.EVENT) {
    var event = payload.event;
    var type = event.type;
    location = _uiInternal.locations.EVENT;
    details = {
      type: type,
      event: event
    };
  } else {
    location = _uiInternal.locations.EFFECT;
    details = {
      actionType: actionType
    };
  }

  return function sandboxedEffect() {
    try {
      // there can be multiple handlers for a given action type defined
      // by multiple behaviors; we only want to apply the options specific
      // to the effects for a single behavior, type, and effect combination
      var options = effect.options;

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      if (options) {
        var _args$action = args[args.length - 1].action,
            _payload = _args$action.payload,
            meta = _args$action.meta;
        if (meta.options === null || meta.options === undefined) meta.options = options;

        if (_payload === null || _payload === undefined) {
          args[args.length - 1].action.payload = {
            options: options
          }; // eslint-disable-line no-param-reassign
        } else if (_typeof(_payload) === 'object') {
          // TODO: payload.options could have be applied from a previous behavior
          // or it could be part of the payload dispatched; not sure what to log
          // or how to properly handle this scenario. if options were part of the metadata
          // instead it would solved the problem.
          // if (typeof payload.options === 'object')
          _payload.options = options;
        }
      }

      var transactionCtx = {
        interactionId: (0, _uiMetrics.getInteractionId)(action.meta),
        host: host,
        type: action.type
      };

      _effects.effectRunner.apply(void 0, [effect, transactionCtx].concat(args));
    } catch (error) {
      (0, _uiInternal.dispatchError)(error, {
        dispatch: dispatch,
        host: host,
        location: location,
        details: details,
        metadata: {
          internal: true
        },
        log: _log["default"]
      });
    }
  };
}
//# sourceMappingURL=sandboxEffect.js.map