/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.applyRuntimeActionHandlers = applyRuntimeActionHandlers;

var _utils = require("../../utils");

var _customEventActionHandler = require("../customEventActionHandler");

var _onAction = require("../onAction");

var _constants = require("../../constants");

var _symbols = require("../../symbols");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var shouldApplyActionHandler = function shouldApplyActionHandler(hostVariables, type) {
  return !hostVariables[_symbols.PRESENTATIONAL] && !hostVariables[_symbols.RUNTIME_ACTION_HANDLERS].includes(type) && hostVariables[_symbols.TARGETED_ACTION_TYPES].includes(_constants.COMPONENT_ACTION_DISPATCHED);
};

var createActionHandler = function createActionHandler(host, hostVariables, type) {
  hostVariables[_symbols.RUNTIME_ACTION_HANDLERS].push(type);

  if (hostVariables[_symbols.CONFIG].interceptors[type] && hostVariables[_symbols.CONFIG].interceptors[type].has('host')) {
    var interceptors = [].concat(_toConsumableArray(hostVariables[_symbols.CONFIG].interceptors[type].get('host')), _toConsumableArray(hostVariables[_symbols.CONFIG].interceptors[_constants.COMPONENT_ACTION_DISPATCHED].get('host')));

    hostVariables[_symbols.CONFIG].interceptors[type].set('host', interceptors);

    return;
  }

  hostVariables[_symbols.CONFIG].interceptors[type] = hostVariables[_symbols.CONFIG].interceptors[_constants.COMPONENT_ACTION_DISPATCHED]; // eslint-disable-line no-param-reassign

  var removeActionHandler = (0, _customEventActionHandler.addActionHandler)(host, 'host', host, type, false, false, _onAction.onAction);

  hostVariables[_symbols.ACTION_HANDLERS].push(removeActionHandler);

  hostVariables[_symbols.TARGETED_ACTION_TYPES] = [].concat(_toConsumableArray(hostVariables[_symbols.TARGETED_ACTION_TYPES]), [type]); // eslint-disable-line no-param-reassign
};

function applyRuntimeActionHandlers(host, type) {
  var parent = host;
  var parentVariables = (0, _utils.getHostVariables)(parent);

  while (parent) {
    if (shouldApplyActionHandler(parentVariables, type)) createActionHandler(parent, parentVariables, type);
    parent = parentVariables[_symbols.ANCESTOR_ROOT_NODE];
    parentVariables = (0, _utils.getHostVariables)(parent);
  }
}
//# sourceMappingURL=applyRuntimeActionHandlers.js.map