/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createEventActionDetails = createEventActionDetails;

var _uiInternal = require("@servicenow/ui-internal");

var _utils = require("../../utils");

var _createMetaData = require("./createMetaData");

var _appendToPayload = require("./appendToPayload");

var _symbols = require("../../symbols");

function createEventActionDetails(actionType, host, event, capture, eventHandler) {
  var _event$detail = event.detail,
      detail = _event$detail === void 0 ? {} : _event$detail;
  var _detail$type = detail.type,
      type = _detail$type === void 0 ? actionType : _detail$type,
      _detail$payload = detail.payload,
      payload = _detail$payload === void 0 ? eventHandler ? (0, _appendToPayload.appendToPayload)(host, {
    event: event,
    host: host
  }) : undefined : _detail$payload,
      _detail$error = detail.error,
      error = _detail$error === void 0 ? null : _detail$error,
      _detail$meta = detail.meta,
      meta = _detail$meta === void 0 ? (0, _createMetaData.createMetaData)(host, {
    eventHandler: eventHandler,
    capture: capture,
    effectType: _uiInternal.locations.EVENT,
    event: event,
    host: host,

    get path() {
      return event.composedPath().map(function (el) {
        var hostVariables = (0, _utils.getHostVariables)(el);
        return hostVariables && hostVariables[_symbols.COMPONENT_ID];
      }).filter(function (componentId) {
        return componentId;
      });
    }

  }, (0, _utils.getHostVariables)(host)) : _detail$meta;
  return {
    type: type,
    payload: payload,
    error: error,
    meta: meta
  };
}
//# sourceMappingURL=createEventActionDetails.js.map