/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wasActionDispatchedByDescendant = wasActionDispatchedByDescendant;

var _constants = require("../../constants");

function wasActionDispatchedByDescendant(composedPath, host) {
  /*
  	action was dispatched from within hook or event handlers in view
  		view(state, {dispatch}) {
  		return (
  			<div>
  				<p on-click={() => dispatch('SN-BUTTON#BUTTON_CLICKED', {})}>I am a paragraph. Click me.</p>
  			</div>
  		);
  	}
   */
  if (!composedPath.length) return true;
  var slotted = false;

  for (var i = 0; i < composedPath.length; i++) {
    var el = composedPath[i];
    if (el.tagName === 'SLOT') slotted = true;

    if (el.host) {
      /*
      	if the first shadowRoot encountered belongs to the host and a slot is not in the path
      	e.g., button dispatches and component that owns view should handle
      		view() {
      		return <p><sn-button></sn-button></p>;
      	}
       */
      if (el.host === host || el.host.tagName === _constants.SEISMIC_HOIST_TAG_NAME) return slotted ? false : true;
      /*
      	if content is slotted and first shadowRoot encountered does not belong to the host
      	e.g., button dispatches and component that owns view should handle
      		view() {
      		return <slotted-child><sn-button></sn-button></slotted-child>;
      	},
       */

      return slotted ? true : false;
    }
  }
}
//# sourceMappingURL=wasActionDispatchedByDescendant.js.map