/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = applyStyles;

var _uiInternal = require("@servicenow/ui-internal");

var _utils = require("./utils");

var _symbols = require("./symbols");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var view = function view() {
  return null;
};

var stylesMap = new Map();

function applyStyles(_x, _x2) {
  return _applyStyles.apply(this, arguments);
}

function _applyStyles() {
  _applyStyles = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(el, config) {
    var _config$styles, styles, _config$stylesheets, stylesheets, _config$renderer, _config$renderer$type, _config$renderer$type2, onStateChange, _config$renderer$type3, deprioritizeStyles, hostVariables, rootNode, tagName;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _config$styles = config.styles, styles = _config$styles === void 0 ? '' : _config$styles, _config$stylesheets = config.stylesheets, stylesheets = _config$stylesheets === void 0 ? [] : _config$stylesheets, _config$renderer = config.renderer;
            _config$renderer = _config$renderer === void 0 ? {} : _config$renderer;
            _config$renderer$type = _config$renderer.type;
            _config$renderer$type = _config$renderer$type === void 0 ? {} : _config$renderer$type;
            _config$renderer$type2 = _config$renderer$type.onStateChange, onStateChange = _config$renderer$type2 === void 0 ? function () {
              return null;
            } : _config$renderer$type2, _config$renderer$type3 = _config$renderer$type.deprioritizeStyles, deprioritizeStyles = _config$renderer$type3 === void 0 ? false : _config$renderer$type3; // eslint-disable-next-line nonblock-statement-body-position

            if (!(!styles && !stylesheets.length)) {
              _context.next = 7;
              break;
            }

            return _context.abrupt("return");

          case 7:
            hostVariables = (0, _utils.getHostVariables)(el);
            rootNode = hostVariables[_symbols.ROOT_NODE];
            tagName = el.tagName;

            if (!(Document && 'adoptedStyleSheets' in Document.prototype && (process.env.NODE_ENV === 'production' || process.env.NODE_ENV === 'test'))) {
              _context.next = 12;
              break;
            }

            return _context.abrupt("return", applyWithAdoptedStylesSheets(el, tagName, rootNode, styles, stylesheets));

          case 12:
            applyWithTag(el, tagName, rootNode, styles, stylesheets, deprioritizeStyles, onStateChange);

          case 13:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _applyStyles.apply(this, arguments);
}

function applyWithAdoptedStylesSheets(_x3, _x4, _x5) {
  return _applyWithAdoptedStylesSheets.apply(this, arguments);
}

function _applyWithAdoptedStylesSheets() {
  _applyWithAdoptedStylesSheets = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(el, tagName, rootNode) {
    var styles,
        stylesheets,
        externalStyles,
        styleEl,
        stylesheetsContent,
        previousDisplaValue,
        sheet,
        _args2 = arguments;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            styles = _args2.length > 3 && _args2[3] !== undefined ? _args2[3] : '';
            stylesheets = _args2.length > 4 ? _args2[4] : undefined;

            if (stylesMap.has(tagName)) {
              _context2.next = 18;
              break;
            }

            externalStyles = document.head.querySelector("#now-".concat((0, _uiInternal.toLower)(tagName), "-styles"));

            if (externalStyles) {
              styleEl = externalStyles.content.cloneNode(true).querySelector('style');
              styles = styleEl ? styleEl.textContent : styles; // eslint-disable-line no-param-reassign
            }

            stylesheetsContent = '';

            if (!(stylesheets && stylesheets.length)) {
              _context2.next = 15;
              break;
            }

            stylesheetsContent = _utils.styleSheetsContentCache.get(el, tagName);

            if (stylesheetsContent) {
              _context2.next = 15;
              break;
            }

            previousDisplaValue = el.style.display;
            el.style.setProperty('display', 'none', 'important');
            _context2.next = 13;
            return (0, _utils.resolveStylesheetsContent)(stylesheets);

          case 13:
            stylesheetsContent = _context2.sent;
            el.style.setProperty('display', previousDisplaValue);

          case 15:
            sheet = new CSSStyleSheet();
            sheet.replace("".concat(styles, " ").concat(stylesheetsContent));
            stylesMap.set(tagName, [sheet]);

          case 18:
            rootNode.adoptedStyleSheets = stylesMap.get(tagName); // eslint-disable-line no-param-reassign

          case 19:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));
  return _applyWithAdoptedStylesSheets.apply(this, arguments);
}

function applyWithTag(_x6, _x7, _x8, _x9, _x10, _x11, _x12) {
  return _applyWithTag.apply(this, arguments);
}

function _applyWithTag() {
  _applyWithTag = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee3(el, tagName, rootNode, styles, stylesheets, deprioritizeStyles, onStateChange) {
    var stylesheetsContent, version, previousDisplaValue, styleFragment;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            stylesheetsContent = ''; // eslint-disable-next-line nonblock-statement-body-position

            if (deprioritizeStyles) onStateChange(rootNode, view);

            if (!(stylesheets && stylesheets.length)) {
              _context3.next = 13;
              break;
            }

            version = (0, _utils.getHostVariables)(_symbols.VERSION) || '';
            stylesheetsContent = _utils.styleSheetsContentCache.get(version, tagName);

            if (stylesheetsContent) {
              _context3.next = 13;
              break;
            }

            previousDisplaValue = el.style.display;
            el.style.setProperty('display', 'none', 'important');
            _context3.next = 10;
            return (0, _utils.resolveStylesheetsContent)(stylesheets);

          case 10:
            stylesheetsContent = _context3.sent;
            el.style.setProperty('display', previousDisplaValue);
            if (stylesheetsContent) _utils.styleSheetsContentCache.set(version, tagName, stylesheetsContent);

          case 13:
            styleFragment = (0, _utils.createStyleFragment)(styles, stylesheetsContent, tagName);
            rootNode.appendChild(styleFragment);

          case 15:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));
  return _applyWithTag.apply(this, arguments);
}
//# sourceMappingURL=applyStyles.js.map