/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.removeAllConsumers = exports.removeConsumersForContextKey = exports.getConsumers = exports.removeConsumer = exports.addConsumer = void 0;
var consumerCache = {};

var addConsumer = function addConsumer(key, host) {
  if (!consumerCache[key]) {
    consumerCache[key] = new Set();
    return consumerCache[key].add(host);
  } else if (consumerCache[key].has(host)) {
    return;
  }

  consumerCache[key].add(host);
};

exports.addConsumer = addConsumer;

var removeConsumer = function removeConsumer(host) {
  for (var k in consumerCache) {
    if (consumerCache[k].has(host)) {
      consumerCache[k]["delete"](host);
    }
  }
};

exports.removeConsumer = removeConsumer;

var getConsumers = function getConsumers(key) {
  return consumerCache[key] && consumerCache[key].values() || new Set().values();
};

exports.getConsumers = getConsumers;

var removeConsumersForContextKey = function removeConsumersForContextKey(key) {
  if (consumerCache[key]) consumerCache[key].clear();
};

exports.removeConsumersForContextKey = removeConsumersForContextKey;

var removeAllConsumers = function removeAllConsumers() {
  for (var k in consumerCache) {
    delete consumerCache[k];
  }
};

exports.removeAllConsumers = removeAllConsumers;
//# sourceMappingURL=consumers.js.map