/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setContext = exports.getContext = exports["default"] = void 0;

var _symbols = require("../symbols");

var _log = _interopRequireDefault(require("../log"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var contexts = new Map();

var getContext = function getContext(contextKey) {
  if (contexts.has(contextKey)) {
    var context = contexts.get(contextKey) || {};
    var resolvedValue = context[_symbols.CONTEXT_RESOLVED_VALUE];

    if (resolvedValue !== undefined) {
      return resolvedValue;
    } else if (context["default"] !== undefined) {
      return context["default"];
    }
  }

  return undefined;
};

exports.getContext = getContext;

var setContext = function setContext(contextKey, value) {
  if (contexts.has(contextKey)) {
    contexts.get(contextKey)[_symbols.CONTEXT_RESOLVED_VALUE] = value;
  } else {
    if (process.env.NODE_ENV !== 'production') {
      (0, _log["default"])("The context \"".concat(contextKey === null || contextKey === void 0 ? void 0 : contextKey.toString(), "\" could not be accessed. Context must be defined using the \"createContext\" function. \"createContext\" can imported from @servicenow/ui-core."), {
        origin: 'context'
      });
    }
  }
};

exports.setContext = setContext;
var _default = contexts;
exports["default"] = _default;
//# sourceMappingURL=contexts.js.map