/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createContextProxy;

var _uiInternal = require("@servicenow/ui-internal");

var _log = _interopRequireDefault(require("../log"));

var _consumers = require("./consumers");

var _createContext = require("./createContext");

var _contexts = require("./contexts");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var shouldNotResolveContext = (0, _uiInternal.memoize)(function (key) {
  return !key.toString().includes(_createContext.prefix);
}, {
  profileName: 'shouldNotResolveContext'
});

function createContextProxy(host) {
  return new Proxy({}, {
    get: function get(target, prop) {
      if (shouldNotResolveContext(prop)) {
        if (process.env.NODE_ENV !== 'production') {
          (0, _log["default"])("The context ".concat(prop.toString(), " could not be accessed. Context must be defined using the \"createContext\" function. \"createContext\" can imported from @servicenow/ui-core."), {
            host: this,
            origin: 'createContextProxy'
          });
        }

        return target[prop];
      }

      (0, _consumers.addConsumer)(prop, host);
      return (0, _contexts.getContext)(prop);
    },
    set: function set(target, prop) {
      if (process.env.NODE_ENV !== 'production') {
        (0, _log["default"])("Cannot set context value ".concat(prop.toString(), ". updateContext must be used."), {
          host: this,
          origin: 'createContextProxy'
        });
      }
    }
  });
}
//# sourceMappingURL=createContextProxy.js.map