/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createCustomElementFactory;

var _stats = require("./stats");

var _uiConfig = require("@servicenow/ui-config");

var _createElementClass = _interopRequireDefault(require("./createElementClass"));

var _utils = require("./utils");

var _log = _interopRequireDefault(require("./log"));

var _validateConfig = _interopRequireWildcard(require("./validateConfig"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var CHECK_FOR_POSSIBLE_PRESENTATIONAL_COMPONENTS = _uiConfig.configConstants.CHECK_FOR_POSSIBLE_PRESENTATIONAL_COMPONENTS,
    POSSIBLE_PRESENTATIONAL_COMPONENTS = _uiConfig.configConstants.POSSIBLE_PRESENTATIONAL_COMPONENTS;
var checkForPossiblePresentationalComponents = (0, _uiConfig.getConfigItem)(CHECK_FOR_POSSIBLE_PRESENTATIONAL_COMPONENTS);
var possiblePresentationalComponents = (0, _uiConfig.getConfigItem)(POSSIBLE_PRESENTATIONAL_COMPONENTS);
/**
 * Define a custom element.
 * @function createCustomElement
 * @example
 * import {createCustomElement} from '@servicenow/ui-core';
 *
 * createCustomElement('sn-hello-world', {
 *   properties: {
 *     name: {
 *       default: 'Fred'
 *     }
 *   },
 *   transformState: ({properties}) => properties,
 *   view: ({name}) => <p>Hello {name}!</p>
 * });
 * @param {string} elementName - The tag name of the custom element.
 * @param {Object} config - The custom element configuration.
 * @param {function} [config.setInitialState] - Function that returns the initial state for a component.
 * @param {Object} [config.properties] - The properties used by a custom element.
 * @param {*} [config.properties.&lt;name&gt;.default] - The default value of the property.
 * @param {function} [config.properties.&lt;name&gt;.computed] - A function that computes the property value.
 * @param {boolean} [config.properties.&lt;name&gt;.reflect] - Reflect property change to corresponding attribute. This value is `false` by default, but for a limited, finite subset of keys that overlap with HTML Attribute keys, it is `true` and cannot be set to `false`.
 * @param {boolean} [config.properties.&lt;name&gt;.unstableParse] - Deprecated, use Type Schemas instead. Parse attribute values when corresponding properties are set. This value is `false` by default.
 * @param {boolean} [config.properties.&lt;name&gt;.unstablePreserveEmptyStr] - Deprecated, use Type Schemas instead. Preserve an empty string when attribute, property is present instead of converting the value to a boolean.
 * @param {boolean} [config.properties.&lt;name&gt;.readOnly] - Property cannot be externally set. This value is `false` by default.
 * @param {boolean} [config.properties.&lt;name&gt;.schema] - Schema that will validate the passed in value of the property during development. This value is `null` by default.
 * @param {boolean} [config.properties.&lt;name&gt;.selectable] - A selectable property can be selected or pointed by other component and map to its local property.
 * @param {boolean} [config.properties.&lt;name&gt;.shouldRenderWhenSet] - Property value change should trigger component render. This value is `true` by default.
 * @param {Object} [config.actions] - The actions the custom element dispatches.
 * @param {boolean} [config.actions.&lt;type&gt;.schema] - Schema that will validate if the payload of the action dispatched from this element is correct.
 * @param {Object} [config.actionHandlers] - Handlers to run when the action type is dispatched.
 * @param {function} [config.actionHandlers.&lt;type&gt;] - The action handler as an effect function or config object.
 * @param {function} [config.view] - A function that renders transformed component state and properties to virtual DOM.
 * @param {function} [config.transformState] - A function that executes before a component's state is passed to the view.
 * @param {Array} [config.behaviors] - Behaviors applied to the custom element.
 * @param {Object} [config.shadowRootOptions] - Options applied to the shadowRoot of a custom element.
 * @param {String} [config.shadowRootOptions.mode] - Mode of the shadowRoot.  Allowed values are `open` and `closed`.  When `closed` access is denied to `shadowRoot` from the element.  This value is `open` by default.
 */

function createCustomElementFactory(componentRegistry) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var _options$presentation = options.presentational,
      presentational = _options$presentation === void 0 ? false : _options$presentation;
  return function createCustomElement(name) {
    var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    if (customElements.get(name)) return (0, _log["default"])("The custom element, ".concat(name, ", has already been defined."));

    if (presentational) {
      (0, _validateConfig.validatePresentationalConfig)(name, config);
    } else {
      if (checkForPossiblePresentationalComponents && (0, _validateConfig.validatePresentationalConfig)(name, config, {
        shouldLog: false
      })) possiblePresentationalComponents.push(name);
      (0, _validateConfig["default"])(name, config);
    }

    var renderer = (0, _utils.resolveRenderer)(config);
    customElements.define(name, (0, _createElementClass["default"])(_objectSpread(_objectSpread({
      presentational: presentational
    }, config), {}, {
      renderer: renderer,
      name: name,
      componentRegistry: componentRegistry
    })));
    (0, _stats.set)('defined', name.toLowerCase());
  };
}
//# sourceMappingURL=createCustomElementFactory.js.map