/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createElementClass;

var _stats = require("./stats");

var _uiMetrics = require("@servicenow/ui-metrics");

var _schema = require("./utils/schema");

var _uiInternal = require("@servicenow/ui-internal");

var _config = require("./utils/config");

var _constants = require("./constants");

var _symbols = require("./symbols");

var _utils = require("./utils");

var _createContextProxy = _interopRequireDefault(require("./context/createContextProxy"));

var _consumers = require("./context/consumers");

var _validateContext = _interopRequireDefault(require("./context/validateContext"));

var _parse = require("./utils/parse");

var _log = _interopRequireDefault(require("./log"));

var _actions = require("./actions");

var _helpers = _interopRequireDefault(require("./utils/helpers"));

var _package = require("../package.json");

var _hoist = _interopRequireWildcard(require("./hoist"));

var _applyStyles = _interopRequireDefault(require("./applyStyles"));

var _removeStyles = _interopRequireDefault(require("./removeStyles"));

var _scheduler = require("./scheduler");

var _shapeRenderData = _interopRequireWildcard(require("./shape-render-data"));

var _propertySelectors = require("./property-selectors");

var _mergeBehaviorProperties = require("./utils/config/mergeBehaviorProperties");

var _normalizeBehaviors = require("./utils/config/normalizeBehaviors");

var _uiConfig = require("@servicenow/ui-config");

var _quickEquals = _interopRequireDefault(require("./utils/helpers/quickEquals"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

function _get(target, property, receiver) { if (typeof Reflect !== "undefined" && Reflect.get) { _get = Reflect.get; } else { _get = function _get(target, property, receiver) { var base = _superPropBase(target, property); if (!base) return; var desc = Object.getOwnPropertyDescriptor(base, property); if (desc.get) { return desc.get.call(receiver); } return desc.value; }; } return _get(target, property, receiver || target); }

function _superPropBase(object, property) { while (!Object.prototype.hasOwnProperty.call(object, property)) { object = _getPrototypeOf(object); if (object === null) break; } return object; }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _CustomElement() {
  return Reflect.construct(HTMLElement, [], this.__proto__.constructor);
}

;
Object.setPrototypeOf(_CustomElement.prototype, HTMLElement.prototype);
Object.setPrototypeOf(_CustomElement, HTMLElement);

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var ENABLE_SCHEDULER_QUEUE_COUNTS = _uiConfig.configConstants.ENABLE_SCHEDULER_QUEUE_COUNTS,
    METRICS_COMPONENT_INTERACTIVE_ENABLED = _uiConfig.configConstants.METRICS_COMPONENT_INTERACTIVE_ENABLED;
var enableSchedulerQueueCounts = (0, _uiConfig.getConfigItem)(ENABLE_SCHEDULER_QUEUE_COUNTS);
var COMPONENT_RENDER = _scheduler.types.COMPONENT_RENDER,
    DEFER_IDLE = _scheduler.types.DEFER_IDLE;
var ENABLE_PROPERTY_DEEP_COMPARE = _uiConfig.configConstants.ENABLE_PROPERTY_DEEP_COMPARE;
var enableDeepPropertyCompare = (0, _uiConfig.getConfigItem)(ENABLE_PROPERTY_DEEP_COMPARE);
var optOutDeepCompare = (0, _utils.shouldOptOutOfDeepCompareForPage)('property');
window.addEventListener('popstate', function () {
  return optOutDeepCompare = (0, _utils.shouldOptOutOfDeepCompareForPage)('property');
});
var shouldMarkInteractive = (0, _uiConfig.getConfigItem)(METRICS_COMPONENT_INTERACTIVE_ENABLED);
var idleCallback = shouldMarkInteractive ? function (el) {
  return (0, _utils.markComponentInteractive)(el, componentRegistryForInteractive);
} : function () {
  return null;
};
var componentRegistryForInteractive;
(0, _uiInternal.addPendingNetworkActionForHostCb)(idleCallback);
(0, _scheduler.addQueueEmptyForElCb)(idleCallback);

function clearRootNode(host, rootNode, onStateChange) {
  (0, _uiInternal.sandbox)(onStateChange, {
    args: [rootNode, function () {
      return null;
    }, {}, host[_symbols.HELPERS_DEFER]],
    host: host,
    details: {
      type: 'onStateChange'
    },
    location: _uiInternal.locations.RENDERER,
    dispatch: host[_symbols.DISPATCH],
    log: _log["default"]
  });
}

function getResolvedState(hostVariables, componentRegistry) {
  return _objectSpread(_objectSpread({}, hostVariables[_symbols.STATE]), {}, {
    properties: (0, _propertySelectors.resolveSelectedProperties)(componentRegistry, _utils.selectablePropertyResolver, _objectSpread({}, hostVariables[_symbols.PROPS]), hostVariables[_symbols.PROPERTY_DEFS])
  });
}

function createElementClass(config) {
  var componentRegistry = config.componentRegistry,
      presentational = config.presentational,
      initialState = config.state,
      _config$renderer = config.renderer,
      _config$renderer$type = _config$renderer.type,
      onStateChange = _config$renderer$type.onStateChange,
      rendererOnConnect = _config$renderer$type.onConnect,
      rendererOnDisconnect = _config$renderer$type.onDisconnect,
      _config$renderer$type2 = _config$renderer$type.deprioritizeStyles,
      deprioritizeStyles = _config$renderer$type2 === void 0 ? false : _config$renderer$type2,
      _config$renderer$tran = _config$renderer.transformState,
      transformState = _config$renderer$tran === void 0 ? function (props) {
    return props;
  } : _config$renderer$tran,
      useCoeffectsLikeObjectInView = _config$renderer.coeffects,
      view = _config$renderer.view,
      tagName = config.name;
  var initialized = false;
  var normalizedConfig;
  var onConnect;
  var onDisconnect;
  var combinedPropertyDefs;
  var shadowRootOptions;
  var validateProperties;
  var propertyKeys;
  var selectabledProperties = [];
  componentRegistryForInteractive = componentRegistryForInteractive || componentRegistry;

  function initialize() {
    if (initialized) return;
    initialized = true;
    normalizedConfig = (0, _config.normalizeConfig)(config);
    var lifecycleFns = (0, _utils.createLifecycleFunctions)(rendererOnConnect, rendererOnDisconnect, deprioritizeStyles);
    onConnect = lifecycleFns.onConnect;
    onDisconnect = lifecycleFns.onDisconnect;
    combinedPropertyDefs = normalizedConfig.properties;
    shadowRootOptions = normalizedConfig.shadowRootOptions;
    validateProperties = (0, _schema.buildPropertiesValidator)(combinedPropertyDefs);
    propertyKeys = Object.keys(combinedPropertyDefs);

    for (var k in combinedPropertyDefs) {
      if (combinedPropertyDefs[k].selectable) selectabledProperties.push(k);
    }

    var _propertyKeys = propertyKeys,
        propertyKeysCount = _propertyKeys.length;

    var _loop = function _loop(i) {
      var name = propertyKeys[i];
      accessors[name] = {
        get: function get() {
          return this.getProperty(name);
        },
        set: function set(value) {
          this[_symbols.SET_PROPERTY](name, value);
        }
      };
    };

    for (var i = 0; i < propertyKeysCount; i++) {
      _loop(i);
    }

    Object.defineProperties(SeismicElement.prototype, accessors);
    (0, _utils.assertValidPropertyNames)(propertyKeys);
  }

  var SeismicElement =
  /*#__PURE__*/
  function (_CustomElement2) {
    _inherits(SeismicElement, _CustomElement2);

    var _super = _createSuper(SeismicElement);

    function SeismicElement() {
      var _thisSuper, _this;

      _classCallCheck(this, SeismicElement);

      _this = _super.call(this);
      initialize();
      var variables = {};
      _this[_symbols.VARIABLES] = variables;
      variables[_symbols.SELECTABLE_PROPERTIES] = selectabledProperties;
      variables[_symbols.PRESENTATIONAL] = presentational;
      variables[_symbols.PROPS] = {};
      variables[_symbols.CHILDREN_TO_BE_INTERACTIVE] = {};
      variables[_symbols.INTERSECTED] = true;
      variables[_symbols.CONFIG] = normalizedConfig;
      variables[_symbols.PROPERTY_DEFS] = combinedPropertyDefs;

      if (!presentational) {
        variables[_symbols.TARGETED_ACTION_TYPES] = [];
        variables[_symbols.PREVIOUS_RENDER_STATE] = {};
        variables[_symbols.PREVIOUS_RENDER_PROPERTIES] = {};
        variables[_symbols.PREVIOUS_TREE_RENDER_STATE] = {};
        variables[_symbols.PREVIOUS_TREE_RENDER_PROPERTIES] = {};
        variables[_symbols.RENDERED_ONCE] = false;
        variables[_symbols.TREE_RENDERED_ONCE] = false;
        variables[_symbols.ACTION_HANDLERS] = [];
        variables[_symbols.RUNTIME_ACTION_HANDLERS] = [];
      }

      variables[_symbols.VERSION] = _package.version; // For CSS externalization. TODO Revisit this logic

      var shadowRoot = _this.attachShadow(shadowRootOptions);

      variables[_symbols.ROOT_NODE] = shadowRoot;
      variables[_symbols.ATTACHED_ROOT_NODE] = shadowRoot;

      _this.upgradeProperties(); // ui-metrics checks this to know if marks for this component are valid.
      // Components that are not bootstrapped don't have enough data to emit
      // a valid mark


      _this.__DO_NOT_USE_BOOTSTRAPPED__ = false;
      variables[_symbols.STYLE_PROXY] = new Proxy(_get((_thisSuper = _assertThisInitialized(_this), _getPrototypeOf(SeismicElement.prototype)), "style", _thisSuper), {
        set: function set(style, key, value) {
          style[key] = value; // eslint-disable-line no-param-reassign

          if (_this.__DO_NOT_USE_BOOTSTRAPPED__ && (0, _utils.isHoisted)(_assertThisInitialized(_this))) (0, _utils.getHoistTarget)(variables).style[key] = value;
          return true;
        },
        get: function get(style, key) {
          if (typeof style[key] === 'function') return style[key].bind(style);
          return style[key];
        }
      });
      (0, _stats.set)('constructed', _this.tagName.toLowerCase());
      return _this;
    }

    _createClass(SeismicElement, [{
      key: _symbols.CONFIG,
      get: function get() {
        return normalizedConfig;
      }
    }, {
      key: _symbols.PROPERTY_DEFS,
      get: function get() {
        return combinedPropertyDefs;
      }
    }, {
      key: "version",
      get: function get() {
        return _package.version;
      }
    }, {
      key: "upgradeProperties",
      value: function upgradeProperties() {
        var _propertyKeys2 = propertyKeys,
            propertyKeysCount = _propertyKeys2.length;

        for (var i = 0; i < propertyKeysCount; i++) {
          var name = propertyKeys[i];

          if (this.hasOwnProperty(name)) {
            var value = this[name];
            delete this[name];
            this[name] = value;
          }
        }
      }
    }, {
      key: "getComponentId",
      value: function getComponentId() {
        var variables = (0, _utils.getHostVariables)(this);
        return variables[_symbols.COMPONENT_ID];
      }
    }, {
      key: "setComponentId",
      value: function setComponentId() {
        var variables = (0, _utils.getHostVariables)(this);
        variables[_symbols.COMPONENT_ID] = this.getAttribute(_constants.COMPONENT_ID_ATTRIBUTE) || (0, _uiInternal.guid)();
        this.setAttribute(_constants.NOW_ID_ATTR, variables[_symbols.COMPONENT_ID], false);
        this.setAttribute(_constants.COMPONENT_ID_ATTRIBUTE, variables[_symbols.COMPONENT_ID], false);
      }
    }, {
      key: "getProperty",
      value: function getProperty(name) {
        var variables = (0, _utils.getHostVariables)(this);
        return variables[_symbols.PROPS][name];
      }
    }, {
      key: _symbols.SET_PROPERTY,
      value: function value(name, _value) {
        var updateProperties = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
        var interactionId = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : (0, _uiMetrics.createInteractionId)();
        var variables = (0, _utils.getHostVariables)(this);

        if (variables[_symbols.PROPS][_constants.NOW_MODE_PROP] === _constants.COMPONENT_MODE_DETACH) {
          if (name !== _constants.NOW_MODE_PROP) {
            if (!this[_symbols.DETACHED_COMPONENT_PROPERTY_MAP]) this[_symbols.DETACHED_COMPONENT_PROPERTY_MAP] = new Map();

            this[_symbols.DETACHED_COMPONENT_PROPERTY_MAP].set(name, {
              name: name,
              value: _value,
              updateProperties: updateProperties,
              interactionId: interactionId
            });

            return;
          }
        }

        var valueOrDefault = (0, _parse.property)(name, _value === undefined ? combinedPropertyDefs[name]["default"] : _value, combinedPropertyDefs[name], this, variables[_symbols.DISPATCH] || (0, _actions.createDispatcher)(this));

        if ((0, _quickEquals["default"])(valueOrDefault, variables[_symbols.PROPS][name])) {
          return;
        } // we might want to consider doing some of this in createUpdateProperties like we
        // do for createUpdateState, so that we never even get this far if the property
        // update is declarative


        if (enableDeepPropertyCompare && !optOutDeepCompare || combinedPropertyDefs[name].deepCompare ? (0, _uiInternal.isEqual)(variables[_symbols.PROPS][name], valueOrDefault) : variables[_symbols.PROPS][name] === valueOrDefault) return;

        if (this.__DO_NOT_USE_BOOTSTRAPPED__ !== true) {
          variables[_symbols.PROPS][name] = valueOrDefault;
          return;
        }

        if (!(0, _utils.shouldSetProperty)(tagName, name, combinedPropertyDefs, updateProperties)) {
          return (0, _log["default"])("Cannot set read only or computed property ".concat(name, "."), {
            host: this,
            origin: String(_symbols.SET_PROPERTY)
          });
        }

        if (variables[_symbols.PROPS][_constants.NOW_ENABLE_PROPERTY_SETTING_PREVENTION_PROP]) {
          var _setInteractionId;

          var event = variables[_symbols.DISPATCH](_constants.COMPONENT_PROPERTY_UPDATE_REQUESTED, {
            name: name,
            value: variables[_symbols.PROPS][name],
            nextValue: valueOrDefault,
            host: this
          }, (0, _uiMetrics.setInteractionId)((_setInteractionId = {}, _defineProperty(_setInteractionId, _constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED, true), _defineProperty(_setInteractionId, _constants.COMPONENT_INTERNAL_ACTION, true), _defineProperty(_setInteractionId, "internal", true), _defineProperty(_setInteractionId, "internalPropertyUpdateRequested", updateProperties), _setInteractionId), interactionId));

          if (event.defaultPrevented) return;
        }

        var previousValue = variables[_symbols.PROPS][name];
        variables[_symbols.PROPS][name] = valueOrDefault; // If nowPlaceholderHeight or Width changes, we want to replace the previous intersect styles so long as:
        // We are currently observing for an intersection (because otherwise we don't want to apply these styles)
        // We have not intersected yet (these styles should only be applied when the component has not intersected yet)

        if (variables[_symbols.USING_INTERSECTION_OBSERVER] && [_constants.NOW_PLACEHOLDER_HEIGHT_PROP, _constants.NOW_PLACEHOLDER_WIDTH_PROP].includes(name) && !variables[_symbols.INTERSECTED] && previousValue !== valueOrDefault) {
          var removeIntersectedStyles = (0, _utils.applyIntersectStyles)(this, config, {
            height: variables[_symbols.PROPS][_constants.NOW_PLACEHOLDER_HEIGHT_PROP],
            width: variables[_symbols.PROPS][_constants.NOW_PLACEHOLDER_WIDTH_PROP]
          });
          if (variables[_symbols.REMOVE_INTERSECTED_STYLES]) variables[_symbols.REMOVE_INTERSECTED_STYLES]();
          variables[_symbols.REMOVE_INTERSECTED_STYLES] = removeIntersectedStyles;
        }

        var propertyDetails = {
          previousValue: previousValue,
          value: valueOrDefault
        };
        (0, _utils.updateSelectedPropertyMappings)(valueOrDefault, name, variables[_symbols.COMPONENT_ID]);

        this[_symbols.UPDATE_AFFECTED_COMPONENTS]({
          name: name,
          propertyDetails: propertyDetails,
          interactionId: interactionId,
          includeSource: true
        });
      }
    }, {
      key: _symbols.UPDATE_AFFECTED_COMPONENTS,
      value: function value(_ref) {
        var name = _ref.name,
            propertyDetails = _ref.propertyDetails,
            interactionId = _ref.interactionId,
            includeSource = _ref.includeSource;
        var variables = (0, _utils.getHostVariables)(this);
        var allAffectedComponents = (0, _propertySelectors.getAffected)(name, variables[_symbols.COMPONENT_ID], componentRegistry, includeSource);
        (0, _utils.triggerComponentUpdates)(componentRegistry, allAffectedComponents, propertyDetails, interactionId, 'PROPERTY_SET');
      }
    }, {
      key: _symbols.HOIST_CHANGED,
      value: function value(_value2) {
        var variables = (0, _utils.getHostVariables)(this);
        if (_value2 && variables[_symbols.UNHOIST] || !_value2 && !variables[_symbols.UNHOIST]) return;

        if (!presentational) {
          variables[_symbols.ACTION_HANDLERS] = (0, _actions.removeActionHandlers)(variables[_symbols.ACTION_HANDLERS]);
          variables[_symbols.TARGETED_ACTION_TYPES] = [];
        }

        if (_value2) {
          // render null to host's shadowRoot to ensure cleanup
          clearRootNode(this, variables[_symbols.ATTACHED_ROOT_NODE], onStateChange);
          variables[_symbols.UNHOIST] = (0, _hoist["default"])(this, deprioritizeStyles, onStateChange);
        } else {
          typeof variables[_symbols.UNHOIST] === 'function' && variables[_symbols.UNHOIST]();
          delete variables[_symbols.UNHOIST];
        }

        if (!presentational) variables[_symbols.ACTION_HANDLERS] = (0, _actions.addActionHandlers)(this, normalizedConfig.interceptors);
      }
    }, {
      key: _symbols.HOIST_PARENT_CHANGED,
      value: function value(propName) {
        var _this2 = this;

        var variables = (0, _utils.getHostVariables)(this);
        var resolvedState = getResolvedState(variables, componentRegistry, this); // reflect hoist targeting properties immediately, these attributes are used when hoisting

        [_constants.HOIST_ATTR, _constants.NOW_HOIST_PROP, propName].forEach(function (p) {
          (0, _shapeRenderData.reflectProperty)(p, combinedPropertyDefs[p], resolvedState, _this2, {});
        });

        if (!presentational) {
          variables[_symbols.ACTION_HANDLERS] = (0, _actions.removeActionHandlers)(variables[_symbols.ACTION_HANDLERS]);
          variables[_symbols.TARGETED_ACTION_TYPES] = [];
        }

        typeof variables[_symbols.UNHOIST] === 'function' && variables[_symbols.UNHOIST]();
        variables[_symbols.UNHOIST] = (0, _hoist["default"])(this, deprioritizeStyles, onStateChange);
        if (!presentational) variables[_symbols.ACTION_HANDLERS] = (0, _actions.addActionHandlers)(this, normalizedConfig.interceptors);
      }
    }, {
      key: _symbols.GET_RESOLVED_PROPERTY,
      value: function value(propName) {
        var variables = (0, _utils.getHostVariables)(this);
        return (0, _propertySelectors.resolveSelectedProperty)(componentRegistry, _utils.selectablePropertyResolver, variables[_symbols.PROPS][propName], normalizedConfig.properties[propName]);
      }
    }, {
      key: "setAttribute",
      value: function setAttribute(name, value) {
        var isReflecting = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        if (!(0, _utils.shouldSetAttribute)(tagName, isReflecting, name, combinedPropertyDefs)) {
          return (0, _log["default"])("Cannot set read only or computed property ".concat(name), {
            host: this,
            origin: 'setAttribute'
          });
        }

        if ((0, _hoist.shouldHoistAttribute)(this, name)) (0, _hoist.hoistAttribute)(this, name, value);
        return HTMLElement.prototype.setAttribute.call(this, name, value);
      }
    }, {
      key: "removeAttribute",
      value: function removeAttribute(name) {
        if ((0, _hoist.shouldHoistAttribute)(this, name)) (0, _hoist.unhoistAttribute)(this, name);
        HTMLElement.prototype.removeAttribute.call(this, name);
      }
    }, {
      key: "generateDeferGetterAndSetter",
      value: function generateDeferGetterAndSetter() {
        var _this3 = this;

        var variables = (0, _utils.getHostVariables)(this);
        variables[_symbols.DEFERRED] = true;
        Object.defineProperty(this, _constants.DEFER_ATTRIBUTE, {
          get: function get() {
            return variables[_symbols.DEFERRED];
          },
          set: function set(value) {
            if (variables[_symbols.DEFERRED] === value) return;
            variables[_symbols.DEFERRED] = value;

            if (!variables[_symbols.CONNECTED]) {
              (0, _stats.set)('undefer', _this3.tagName.toLowerCase());

              _this3.connectedCallback();
            } else {
              (0, _log["default"])("".concat(_this3.tagName.toLowerCase(), "[component-id=\"").concat(variables[_symbols.COMPONENT_ID], "\"] cannot be deferred. It is already connected."), {
                host: _this3,
                origin: 'generateDeferGetterAndSetter'
              });
            }
          },
          configurable: true
        });
      }
    }, {
      key: "shouldLegacyDefer",
      value: function shouldLegacyDefer() {
        return this.hasAttribute(_constants.DEFER_ATTRIBUTE) && !this.__DO_NOT_USE_BOOTSTRAPPED__;
      }
    }, {
      key: "setInitialProperties",
      value: function setInitialProperties() {
        var variables = (0, _utils.getHostVariables)(this);
        var defaults = {};
        var _propertyKeys3 = propertyKeys,
            propertyKeysCount = _propertyKeys3.length;

        for (var i = 0; i < propertyKeysCount; i++) {
          var name = propertyKeys[i];
          defaults[name] = (0, _utils.getInitialPropertyValue)(this, name, combinedPropertyDefs);
        }

        var propertyDefs = combinedPropertyDefs;
        variables[_symbols.PROPS] = (0, _parse.properties)((0, _utils.resolveProperties)(initialState, (0, _utils.filterPropertyDefinitions)(propertyDefs, function (def) {
          return !def.computed;
        }), defaults), propertyDefs, {
          host: this,
          dispatch: variables[_symbols.DISPATCH]
        });
        var properties = variables[_symbols.PROPS];

        for (var k in properties) {
          (0, _utils.updateSelectedPropertyMappings)(properties[k], k, variables[_symbols.COMPONENT_ID], this.__DO_NOT_USE_BOOTSTRAPPED__);

          if (combinedPropertyDefs[k] && combinedPropertyDefs[k].selectable) {
            (0, _utils.triggerComponentUpdates)(componentRegistry, (0, _propertySelectors.getComponentAffected)(k, variables[_symbols.COMPONENT_ID]), {
              value: variables[_symbols.PROPS][k]
            }, undefined, 'INITIAL_PROPERTY_SET');
          }
        }
      }
    }, {
      key: "getResolvedProperties",
      value: function getResolvedProperties(interactionId) {
        var variables = (0, _utils.getHostVariables)(this);
        var state = getResolvedState(variables, componentRegistry, this);
        var resolvedProperties = (0, _utils.resolveComputedProperties)(state, this, variables, interactionId);
        return (0, _utils.unescapeReservedCharacters)(resolvedProperties).properties;
      }
    }, {
      key: "getProperties",
      value: function getProperties() {
        var variables = (0, _utils.getHostVariables)(this);
        return _objectSpread({}, variables[_symbols.PROPS]);
      }
    }, {
      key: "attributeChangedCallback",
      value: function attributeChangedCallback(name, oldValue, newValue) {
        var variables = (0, _utils.getHostVariables)(this);
        if (name === _constants.DEFER_ATTRIBUTE) return this[_constants.DEFER_ATTRIBUTE] = newValue;
        if (!variables[_symbols.CONNECTED]) return;
        var propName = (0, _utils.toPropertyName)(name);
        var propValue = this[propName];
        if ((0, _utils.isReflecting)(name)) return (0, _utils.clearReflection)(name); // vdom renderers set prop, then attribute values on nodes; this check prevents a double render
        // while allowing for direct attribute setting and removal, e.g., el.setAttribute, el.removeAttribute
        // 1. if newValue is null and propValue is falsey then the property was set already and removeAttribute was called
        // 2. if propValue is truthy and newValue is a string and values are equal then the property was set
        //    already and setAttribute was called
        // 3. if propValue is truthy and newValue is empty string setAttribute was called after setting property to true
        // 4. if propValue is falsey and newValue is null removeAttribute was called after setting property to false

        if (newValue === null && !propValue) return;
        if (propValue && typeof newValue === 'string' && propValue === newValue) return;

        if (typeof propValue === 'boolean') {
          if (propValue && newValue === '' && !combinedPropertyDefs[propName].unstablePreserveEmptyStr) return;
          if (!propValue && newValue === null) return;
        } // if newValue is null then removeAttribute was called, We should set the property to undefined, which will
        // be set to default in the setter if necessary


        if (variables[_symbols.CONNECTED] && oldValue !== newValue) this[propName] = newValue === null ? undefined : newValue;
      }
    }, {
      key: "connectedCallback",
      value: function connectedCallback() {
        var _this4 = this;

        var variables = (0, _utils.getHostVariables)(this); // if a node is quickly connected and disconnected then by the time
        // connectedCallback is called it can actually be disconnected

        if (!this.isConnected) return;

        if (this.shouldLegacyDefer()) {
          variables[_symbols.CONNECTED_DEFERRED_TIMESTAMP] = performance.now();
          (0, _stats.set)('defer', this.tagName.toLowerCase());
          return this.generateDeferGetterAndSetter();
        } // We haven't called `setInitialProperties()` yet until later in the
        // connect, so `this[NOW_PRIORITY_PROP]` may not have the default value
        // from the property config.


        var priorityLevel = this[_constants.NOW_PRIORITY_PROP] === undefined ? combinedPropertyDefs[_constants.NOW_PRIORITY_PROP] ? combinedPropertyDefs[_constants.NOW_PRIORITY_PROP]["default"] : undefined : this[_constants.NOW_PRIORITY_PROP];

        if (priorityLevel === _constants.PRIORITY_LEVEL_LOW && !variables[_symbols.LOW_PRIORITY_PENDING]) {
          this.setAttribute(_constants.NOW_PRIORITY_ATTR, priorityLevel);
          variables[_symbols.LOW_PRIORITY_PENDING] = true;
          (0, _scheduler.schedule)({
            fn: function fn() {
              if (!variables[_symbols.CONNECTED]) _this4.connectedCallback();

              _this4.removeAttribute(_constants.NOW_PRIORITY_ATTR);

              delete variables[_symbols.LOW_PRIORITY_PENDING];
            },
            type: DEFER_IDLE,
            meta: {
              el: this,
              interactionId: (0, _uiMetrics.createInteractionId)()
            }
          });
          return;
        }

        variables[_symbols.CONNECTED] = true;
        variables[_symbols.CONNECTED_TIMESTAMP] = performance.now();
        this.setComponentId();
        this.dispatch = (0, _actions.createDispatcher)(this);
        variables[_symbols.HELPERS_SCHEDULED] = (0, _helpers["default"])(this, combinedPropertyDefs, _constants.HELPERS_SCHEDULED, {
          presentational: presentational
        });
        variables[_symbols.HELPERS_IMMEDIATE] = (0, _helpers["default"])(this, combinedPropertyDefs, _constants.HELPERS_IMMEDIATE, {
          presentational: presentational
        });
        variables[_symbols.HELPERS_DEFER] = (0, _helpers["default"])(this, combinedPropertyDefs, _constants.HELPERS_DEFER, {
          presentational: presentational
        });
        this.helpers = variables[_symbols.HELPERS_SCHEDULED]; // @servicenow/ui-internal relies on this

        variables[_symbols.DISPATCH] = this.dispatch;
        variables[_symbols.CONDITIONAL_DISPATCH] = (0, _actions.createConditionalDispatch)(this);
        variables[_symbols.UNHOIST] = (0, _hoist["default"])(this, deprioritizeStyles, onStateChange);
        var rootNode = (0, _utils.getRootNode)(this);
        variables[_symbols.ANCESTOR_ROOT_NODE] = rootNode && rootNode.nowId ? rootNode : null;

        if (variables[_symbols.ANCESTOR_ROOT_NODE]) {
          variables[_symbols.ANCESTOR_ROOT_NODE][_symbols.VARIABLES][_symbols.CHILDREN_TO_BE_INTERACTIVE][variables[_symbols.COMPONENT_ID]] = true;
        }

        this.__DO_NOT_USE_OR_YOU_WILL_BE_FIRED_BY_YOUR_PARENT__ = variables[_symbols.ANCESTOR_ROOT_NODE] && variables[_symbols.ANCESTOR_ROOT_NODE].nowId || null;
        variables[_symbols.CHILDREN_TO_BE_RENDERED] = {};
        variables[_symbols.NEEDS_TO_KNOW_ABOUT_TREE_RENDERS] = normalizedConfig.shouldDispatch[_constants.COMPONENT_DOM_TREE_READY] || normalizedConfig.shouldDispatch[_constants.COMPONENT_TREE_RENDERED] || variables[_symbols.ANCESTOR_ROOT_NODE] && variables[_symbols.ANCESTOR_ROOT_NODE][_symbols.VARIABLES][_symbols.NEEDS_TO_KNOW_ABOUT_TREE_RENDERS];
        if (!presentational) variables[_symbols.ACTION_HANDLERS] = (0, _actions.addActionHandlers)(this, normalizedConfig.interceptors);
        if (componentRegistry[this.getComponentId()] !== undefined) (0, _log["default"])("The componentId- ".concat(this.getComponentId(), " already exists. Component identifiers must be unique."));
        componentRegistry[this.getComponentId()] = this; // This must be called here syncronously instead of in the callback of onConnect
        // just below. If any code gets added that could prevent this[RENDER] being called
        // between here and inside the onConnect callback just below, it will break the
        // TREE_RENDERED and TREE_READY lifecycles

        this[_symbols.NOTIFY_PARENT_THAT_CHILD_IS_RENDERING]();

        (0, _uiInternal.sandbox)(onConnect, {
          args: [variables[_symbols.ROOT_NODE], this.dispatch, function () {
            _this4.setInitialProperties();

            if (!presentational && !variables[_symbols.STATE]) {
              // Set variables[STATE] to just initialState and not include setInitialState
              // for backwards compatibility: so that initialState is still available
              // within a computed property's computed() method
              variables[_symbols.STATE] = _objectSpread({}, normalizedConfig.initialState);

              var _initialState = _objectSpread(_objectSpread({}, variables[_symbols.STATE]), normalizedConfig.setInitialState({
                host: _this4,
                properties: (0, _utils.unescapeReservedCharacters)((0, _utils.computeProperties)((0, _utils.camelCaseProperties)(getResolvedState(variables, componentRegistry, _this4)), _this4)).properties
              }));

              (0, _utils.emitNowUiEvent)(_constants.STATE_UPDATED, {
                componentId: variables[_symbols.COMPONENT_ID],
                previousState: undefined,
                nextState: _initialState
              });
              variables[_symbols.STATE] = _initialState;
            }

            _this4[_symbols.RENDER](); // this is to handle the case in which a renderer has not already set the properties on the host because it is async.
            // i do not want to unconditionally move hoisting initialization here without a design and supporting tests.
            // this change is backwards compatible with existing hoist use cases and renderers.


            if (typeof variables[_symbols.UNHOIST] !== 'function' && (0, _utils.isHoisted)(_this4)) {
              if (presentational) {
                variables[_symbols.UNHOIST] = (0, _hoist["default"])(_this4, deprioritizeStyles, onStateChange);
              } else {
                (0, _actions.removeActionHandlers)(variables[_symbols.ACTION_HANDLERS]);
                variables[_symbols.TARGETED_ACTION_TYPES] = [];
                variables[_symbols.UNHOIST] = (0, _hoist["default"])(_this4, deprioritizeStyles, onStateChange);
                variables[_symbols.ACTION_HANDLERS] = (0, _actions.addActionHandlers)(_this4, normalizedConfig.interceptors);
              }
            }

            (0, _applyStyles["default"])(_this4, config);
            variables[_symbols.USING_INTERSECTION_OBSERVER] = variables[_symbols.PROPS][_constants.NOW_PLACEHOLDER_HEIGHT_PROP] || variables[_symbols.PROPS][_constants.NOW_PLACEHOLDER_WIDTH_PROP];

            if (variables[_symbols.USING_INTERSECTION_OBSERVER]) {
              variables[_symbols.INTERSECTED] = false;

              if (variables[_symbols.PROPS][_constants.NOW_PLACEHOLDER_HEIGHT_PROP] || variables[_symbols.PROPS][_constants.NOW_PLACEHOLDER_WIDTH_PROP]) {
                variables[_symbols.REMOVE_INTERSECTED_STYLES] = (0, _utils.applyIntersectStyles)(_this4, config, {
                  height: variables[_symbols.PROPS][_constants.NOW_PLACEHOLDER_HEIGHT_PROP],
                  width: variables[_symbols.PROPS][_constants.NOW_PLACEHOLDER_WIDTH_PROP]
                });

                _utils.intersectionObserver.observe(_this4);
              }
            }

            if (!_this4.__DO_NOT_USE_BOOTSTRAPPED__) {
              _this4.__DO_NOT_USE_BOOTSTRAPPED__ = true;

              variables[_symbols.CONDITIONAL_DISPATCH](_constants.COMPONENT_BOOTSTRAPPED, {
                host: _this4
              }, {
                internal: true
              });
            }

            variables[_symbols.CONDITIONAL_DISPATCH](_constants.COMPONENT_CONNECTED, {
              host: _this4
            }, {
              internal: true
            });
          }],
          host: this,
          details: {
            type: 'onConnect'
          },
          location: _uiInternal.locations.RENDERER,
          dispatch: variables[_symbols.DISPATCH],
          log: _log["default"]
        });
      }
    }, {
      key: "disconnectedCallback",
      value: function disconnectedCallback() {
        var variables = (0, _utils.getHostVariables)(this);
        if (!variables[_symbols.CONNECTED]) return;
        variables[_symbols.CONNECTED] = false; // render null to ensure that view specific clean up hooks are called

        clearRootNode(this, variables[_symbols.ROOT_NODE], onStateChange);
        (0, _removeStyles["default"])(this);
        if (variables[_symbols.USING_INTERSECTION_OBSERVER] && !variables[_symbols.INTERSECTED]) _utils.intersectionObserver.unobserve(this);

        if (!normalizedConfig.shouldDispatch[_constants.COMPONENT_BOOTSTRAPPED] && !normalizedConfig.shouldDispatch[_constants.COMPONENT_CONNECTED] || normalizedConfig.shouldDispatch[_constants.COMPONENT_BOOTSTRAPPED] && variables[_symbols.BOOTSTRAPPED_HANDLED] || normalizedConfig.shouldDispatch[_constants.COMPONENT_CONNECTED] && variables[_symbols.CONNECTED_HANDLED]) {
          variables[_symbols.CONDITIONAL_DISPATCH](_constants.COMPONENT_DISCONNECTED, {
            host: this
          }, _defineProperty({
            internal: true
          }, _constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED, true));
        }

        (0, _uiInternal.sandbox)(onDisconnect, {
          args: [variables[_symbols.ROOT_NODE], this.dispatch, function () {
            typeof variables[_symbols.UNHOIST] === 'function' && variables[_symbols.UNHOIST]();
          }],
          host: this,
          details: {
            type: 'onDisconnect'
          },
          location: _uiInternal.locations.RENDERER,
          dispatch: variables[_symbols.DISPATCH],
          log: _log["default"]
        });

        if (presentational) {
          (0, _propertySelectors.removeMappings)(variables[_symbols.COMPONENT_ID]);
        } else {
          variables[_symbols.CONNECTED_HANDLED] = false;
          variables[_symbols.ACTION_HANDLERS] = (0, _actions.removeActionHandlers)(variables[_symbols.ACTION_HANDLERS]);
          variables[_symbols.TARGETED_ACTION_TYPES] = [];
          (0, _propertySelectors.removeMappings)(variables[_symbols.COMPONENT_ID]);
          variables[_symbols.RENDERED_ONCE] = false;
          variables[_symbols.TREE_RENDERED_ONCE] = false;
          (0, _uiInternal.clearPendingNetworkActionsForHost)(this);
        }

        (0, _consumers.removeConsumer)(this); // there are cases where connectedCallback has not executed, but disconnectedCallback does

        this.getComponentId() && delete componentRegistry[this.getComponentId()];
      }
    }, {
      key: _symbols.RENDER,
      value: function value() {
        var _this5 = this;

        var interactionId = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _uiMetrics.createInteractionId)();
        var source = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _uiMetrics.types.CONNECTION;
        var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
        var variables = (0, _utils.getHostVariables)(this);
        options.cause = source; // eslint-disable-line no-param-reassign

        if (enableSchedulerQueueCounts) {
          options.highPriorityTaskCount = (0, _scheduler.countHighPriorityTasks)(this);
          options.mediumPriorityTaskCount = (0, _scheduler.countMediumPriorityTasks)(this);
          options.pendingNetworkActionCount = (0, _uiInternal.pendingNetworkActionCountForHost)(this);
        } // tell parent that a child is rendering. Once this is called,
        // this[NOTIFY_PARENT_THAT_CHILD_TREE_HAS_RENDERED]() must be called at some point,
        // either directly or via this[CHECK_IF_TREE_RENDERED]()


        this[_symbols.NOTIFY_PARENT_THAT_CHILD_IS_RENDERING]({
          interactionId: interactionId
        });

        if (!this.isConnected) {
          this[_symbols.NOTIFY_PARENT_THAT_CHILD_TREE_HAS_RENDERED]({
            interactionId: interactionId
          });

          return;
        }

        (0, _scheduler.schedule)({
          fn: function fn() {
            var _variables$CONDITIONA2;

            if (!_this5.isConnected) {
              _this5[_symbols.NOTIFY_PARENT_THAT_CHILD_TREE_HAS_RENDERED]({
                interactionId: interactionId
              });

              return;
            }

            if (variables[_symbols.PROPS][_constants.NOW_MODE_PROP] === _constants.COMPONENT_MODE_SUSPEND) {
              _this5[_symbols.NOTIFY_PARENT_THAT_CHILD_TREE_HAS_RENDERED]({
                interactionId: interactionId
              });

              return;
            }

            if (variables[_symbols.PROPS][_constants.NOW_MODE_PROP] === _constants.COMPONENT_MODE_DETACH) {
              var fragment = document.createDocumentFragment();

              if (!_this5[_symbols.DETACHED_SHADOW_ROOT_CHILDREN_FRAGMENT]) {
                while (variables[_symbols.ROOT_NODE].firstChild) {
                  fragment.appendChild(variables[_symbols.ROOT_NODE].removeChild(variables[_symbols.ROOT_NODE].firstChild));
                }

                _this5[_symbols.DETACHED_SHADOW_ROOT_CHILDREN_FRAGMENT] = fragment;

                _this5[_symbols.NOTIFY_PARENT_THAT_CHILD_TREE_HAS_RENDERED]({
                  interactionId: interactionId
                });
              }

              return;
            } // If we are observing for an intersection, and it has not happened yet, then cancel the render


            if (variables[_symbols.USING_INTERSECTION_OBSERVER] && !variables[_symbols.INTERSECTED]) {
              _this5[_symbols.NOTIFY_PARENT_THAT_CHILD_TREE_HAS_RENDERED]({
                interactionId: interactionId
              });

              return;
            }

            var state = getResolvedState(variables, componentRegistry);
            validateProperties(state.properties, _this5);
            (0, _validateContext["default"])(variables[_symbols.CONTEXT], _this5);
            var updatedState = (0, _shapeRenderData["default"])(state, _this5, variables, transformState, variables[_symbols.DISPATCH], interactionId);

            var event = variables[_symbols.CONDITIONAL_DISPATCH](_constants.COMPONENT_RENDER_REQUESTED, {
              previousRenderState: variables[_symbols.PREVIOUS_RENDER_STATE],
              previousRenderProperties: variables[_symbols.PREVIOUS_RENDER_PROPERTIES],
              host: _this5
            }, (_variables$CONDITIONA2 = {
              firstRenderOnConnect: !variables[_symbols.RENDERED_ONCE]
            }, _defineProperty(_variables$CONDITIONA2, _constants.COMPONENT_INTERNAL_ACTION, true), _defineProperty(_variables$CONDITIONA2, "internal", true), _defineProperty(_variables$CONDITIONA2, _constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED, true), _defineProperty(_variables$CONDITIONA2, "renderCause", options.cause), _variables$CONDITIONA2));

            if (event && event.defaultPrevented) {
              _this5[_symbols.NOTIFY_PARENT_THAT_CHILD_TREE_HAS_RENDERED]({
                interactionId: interactionId
              });

              return;
            }

            var shouldSkipRender = false;

            if (_this5[_symbols.DETACHED_SHADOW_ROOT_CHILDREN_FRAGMENT]) {
              if (_this5[_symbols.DETACHED_COMPONENT_PROPERTY_MAP]) {
                _this5[_symbols.DETACHED_COMPONENT_PROPERTY_MAP].forEach(function (_ref2) {
                  var name = _ref2.name,
                      value = _ref2.value,
                      updateProperties = _ref2.updateProperties,
                      interactionId = _ref2.interactionId;
                  shouldSkipRender = shouldSkipRender || _this5[_symbols.VARIABLES][_symbols.PROPS][name] !== value;

                  _this5[_symbols.SET_PROPERTY](name, value, updateProperties, interactionId);
                });
              }

              variables[_symbols.ROOT_NODE].appendChild(_this5[_symbols.DETACHED_SHADOW_ROOT_CHILDREN_FRAGMENT]);

              delete _this5[_symbols.DETACHED_SHADOW_ROOT_CHILDREN_FRAGMENT];
              delete _this5[_symbols.DETACHED_COMPONENT_PROPERTY_MAP];
              if (shouldSkipRender) return;
            }

            (0, _utils.measure)(function () {
              var _createOnStateChangeA = (0, _utils.createOnStateChangeArgs)(useCoeffectsLikeObjectInView, variables[_symbols.COMPONENT_ID], updatedState, (0, _createContextProxy["default"])(_this5), (0, _utils.applyMetrics)(variables[_symbols.HELPERS_DEFER]), _this5),
                  _createOnStateChangeA2 = _slicedToArray(_createOnStateChangeA, 2),
                  state = _createOnStateChangeA2[0],
                  helpers = _createOnStateChangeA2[1];

              (0, _uiInternal.sandbox)(onStateChange, {
                args: [variables[_symbols.ROOT_NODE], view, state, helpers, interactionId],
                host: _this5,
                details: {
                  type: 'onStateChange'
                },
                location: _uiInternal.locations.RENDERER,
                dispatch: variables[_symbols.DISPATCH],
                log: _log["default"]
              });
            }, _this5, "render-".concat(tagName, "#").concat(variables[_symbols.COMPONENT_ID]), interactionId, options);

            if (!variables[_symbols.RENDERED_ONCE]) {
              variables[_symbols.CONDITIONAL_DISPATCH](_constants.COMPONENT_DOM_READY, {
                previousRenderState: variables[_symbols.PREVIOUS_RENDER_STATE],
                previousRenderProperties: variables[_symbols.PREVIOUS_RENDER_PROPERTIES],
                host: _this5
              }, _defineProperty({
                internal: true,
                firstRenderOnConnect: true
              }, _constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED, true));
            }

            variables[_symbols.CONDITIONAL_DISPATCH](_constants.COMPONENT_RENDERED, {
              previousRenderState: variables[_symbols.PREVIOUS_RENDER_STATE],
              previousRenderProperties: variables[_symbols.PREVIOUS_RENDER_PROPERTIES],
              host: _this5
            }, _defineProperty({
              internal: true,
              firstRenderOnConnect: !variables[_symbols.RENDERED_ONCE]
            }, _constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED, true));

            variables[_symbols.RENDERED_ONCE] = true;
            variables[_symbols.RENDERED_ONCE_TIMESTAMP] = variables[_symbols.RENDERED_ONCE_TIMESTAMP] || performance.now();
            variables[_symbols.PREVIOUS_RENDER_STATE] = updatedState;
            variables[_symbols.PREVIOUS_RENDER_PROPERTIES] = state.properties;

            _this5[_symbols.CHECK_IF_TREE_RENDERED](interactionId, options);
          },
          type: COMPONENT_RENDER,
          meta: {
            el: this,
            interactionId: interactionId
          }
        });
      }
    }, {
      key: _symbols.INTERSECTION_TRIGGERED,
      value: function value() {
        var variables = (0, _utils.getHostVariables)(this);
        variables[_symbols.INTERSECTED] = true;

        if (variables[_symbols.REMOVE_INTERSECTED_STYLES]) {
          variables[_symbols.REMOVE_INTERSECTED_STYLES]();

          variables[_symbols.REMOVE_INTERSECTED_STYLES] = null;
        }

        this[_symbols.RENDER]((0, _uiMetrics.createInteractionId)(), _uiMetrics.types.INTERSECTION);
      }
    }, {
      key: _symbols.CHECK_IF_TREE_RENDERED,
      value: function value(interactionId) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        var variables = (0, _utils.getHostVariables)(this);

        if (enableSchedulerQueueCounts) {
          options.highPriorityTaskCount = (0, _scheduler.countHighPriorityTasks)(this);
          options.mediumPriorityTaskCount = (0, _scheduler.countMediumPriorityTasks)(this);
          options.pendingNetworkActionCount = (0, _uiInternal.pendingNetworkActionCountForHost)(this);
        } // Check if all children are complete. if so, dispatch that this component is also complete


        if (Object.keys(variables[_symbols.CHILDREN_TO_BE_RENDERED]).length) return;

        if (!variables[_symbols.TREE_RENDERED_ONCE]) {
          variables[_symbols.CONDITIONAL_DISPATCH](_constants.COMPONENT_DOM_TREE_READY, {
            previousRenderState: variables[_symbols.PREVIOUS_TREE_RENDER_STATE],
            previousRenderProperties: variables[_symbols.PREVIOUS_TREE_RENDER_PROPERTIES],
            host: this
          }, _defineProperty({
            internal: true,
            firstRenderOnConnect: true
          }, _constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED, true));
        }

        (0, _uiMetrics.mark)(this, interactionId, _uiMetrics.types.RENDER_TREE, options);

        variables[_symbols.CONDITIONAL_DISPATCH](_constants.COMPONENT_TREE_RENDERED, {
          previousRenderState: variables[_symbols.PREVIOUS_TREE_RENDER_STATE],
          previousRenderProperties: variables[_symbols.PREVIOUS_TREE_RENDER_PROPERTIES],
          host: this
        }, _defineProperty({
          internal: true,
          firstRenderOnConnect: !variables[_symbols.TREE_RENDERED_ONCE]
        }, _constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED, true));

        variables[_symbols.TREE_RENDERED_ONCE] = true;
        variables[_symbols.TREE_RENDERED_ONCE_TIMESTAMP] = variables[_symbols.TREE_RENDERED_ONCE_TIMESTAMP] || performance.now();
        variables[_symbols.PREVIOUS_TREE_RENDER_STATE] = variables[_symbols.PREVIOUS_RENDER_STATE];
        variables[_symbols.PREVIOUS_TREE_RENDER_PROPERTIES] = variables[_symbols.PREVIOUS_RENDER_PROPERTIES];

        this[_symbols.NOTIFY_PARENT_THAT_CHILD_TREE_HAS_RENDERED]({
          interactionId: interactionId
        });
      }
    }, {
      key: _symbols.CHILD_IS_RENDERING,
      value: function value(child) {
        var variables = (0, _utils.getHostVariables)(this);
        var childVariables = (0, _utils.getHostVariables)(child);
        variables[_symbols.CHILDREN_TO_BE_RENDERED][childVariables[_symbols.COMPONENT_ID]] = true;
      }
    }, {
      key: _symbols.CHILD_HAS_RENDERED,
      value: function value(_ref3) {
        var child = _ref3.child,
            interactionId = _ref3.interactionId;
        var variables = (0, _utils.getHostVariables)(this);
        var childVariables = (0, _utils.getHostVariables)(child);
        delete variables[_symbols.CHILDREN_TO_BE_RENDERED][childVariables[_symbols.COMPONENT_ID]];

        this[_symbols.CHECK_IF_TREE_RENDERED](interactionId);
      }
    }, {
      key: _symbols.NOTIFY_PARENT_THAT_CHILD_IS_RENDERING,
      value: function value() {
        var variables = (0, _utils.getHostVariables)(this);
        var parentHost = variables[_symbols.ANCESTOR_ROOT_NODE];
        if (parentHost && (0, _utils.getHostVariables)(parentHost)[_symbols.NEEDS_TO_KNOW_ABOUT_TREE_RENDERS]) parentHost[_symbols.CHILD_IS_RENDERING](this);
      }
    }, {
      key: _symbols.NOTIFY_PARENT_THAT_CHILD_TREE_HAS_RENDERED,
      value: function value(_ref4) {
        var interactionId = _ref4.interactionId;
        var variables = (0, _utils.getHostVariables)(this);
        var parentHost = variables[_symbols.ANCESTOR_ROOT_NODE];
        if (parentHost && (0, _utils.getHostVariables)(parentHost)[_symbols.NEEDS_TO_KNOW_ABOUT_TREE_RENDERS]) parentHost[_symbols.CHILD_HAS_RENDERED]({
          child: this,
          interactionId: interactionId
        });
      }
    }], [{
      key: "observedAttributes",
      get: function get() {
        var pKeys = Object.keys((0, _mergeBehaviorProperties.mergeBehaviorProperties)((0, _normalizeBehaviors.normalizeBehaviors)(config.behaviors || [], tagName), config.properties || {}, tagName));
        return pKeys.map(_utils.toAttributeName);
      }
    }]);

    return SeismicElement;
  }(_CustomElement);

  var accessors = {
    nowElementConfig: {
      get: function get() {
        initialize();
        return normalizedConfig;
      }
    },
    nowRootNode: {
      get: function get() {
        if (!initialized || shadowRootOptions.mode === _constants.SHADOW_ROOT_MODE_CLOSED) return null;
        return (0, _utils.getHostVariables)(this)[_symbols.ROOT_NODE];
      }
    },
    style: {
      get: function get() {
        return (0, _utils.getHostVariables)(this)[_symbols.STYLE_PROXY];
      },
      set: function set() {
        (0, _log["default"])('setting style property directly is not supported');
      }
    }
  };
  return SeismicElement;
}
//# sourceMappingURL=createElementClass.js.map