/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = effectRunner;

var _modifiers = require("./modifiers");

var _runEffect = _interopRequireDefault(require("./runEffect"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function effectRunner(effect, ctx) {
  var effectFn = effect.effect,
      _effect$modifier = effect.modifier,
      modifier = _effect$modifier === void 0 ? {} : _effect$modifier;

  for (var _len = arguments.length, args = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    args[_key - 2] = arguments[_key];
  }

  var coeffects = args[args.length - 1];
  if (!coeffects) return (0, _runEffect["default"])(effectFn, args, ctx);
  var targetElement = (0, _modifiers.getTargetElement)(coeffects);

  switch (modifier.name) {
    case _modifiers.modifierTypes.DEBOUNCE:
      return (0, _modifiers.debounce)(targetElement, coeffects.action.type, effectFn, modifier, ctx, _runEffect["default"]).apply(void 0, args);

    case _modifiers.modifierTypes.TAKE_LATEST:
      return (0, _runEffect["default"])((0, _modifiers.takeLatest)(targetElement, coeffects.action.type, effectFn), args, ctx).then(function () {
        delete _modifiers.hostMap.get(targetElement).get(effectFn)[coeffects.action.type];
      });

    case _modifiers.modifierTypes.THROTTLE:
      return (0, _modifiers.throttle)(targetElement, coeffects.action.type, effectFn, modifier, ctx, _runEffect["default"]).apply(void 0, args);

    default:
      return (0, _runEffect["default"])(effectFn, args, ctx);
  }
}
//# sourceMappingURL=effectRunner.js.map