/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = throttle;

var _hostMap = _interopRequireDefault(require("./hostMap"));

var _uiInternal = require("@servicenow/ui-internal");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function createThrottle(fn, _ref, ctx, runEffectFn) {
  var _ref$limit = _ref.limit,
      limit = _ref$limit === void 0 ? 0 : _ref$limit;
  var free = true;
  return function throttledEffect() {
    if (!free) return;
    free = false;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    runEffectFn(fn, args, ctx);
    (0, _uiInternal.defer)(function () {
      free = true;
    }, limit);
  };
}
/**
 * Throttle modifier for effect functions
 *
 * @param host
 * @param actionType
 * @param effectFn
 * @param modifier
 */


function throttle(host, actionType, effectFn, modifier, ctx, runEffectFn) {
  if (!_hostMap["default"].has(host)) _hostMap["default"].set(host, new WeakMap());

  var effectMap = _hostMap["default"].get(host);

  if (!effectMap.has(effectFn)) effectMap.set(effectFn, {});
  var actionMappings = effectMap.get(effectFn);
  if (!actionMappings[actionType]) actionMappings[actionType] = createThrottle(effectFn, modifier, ctx, runEffectFn);
  return actionMappings[actionType];
}
//# sourceMappingURL=throttle.js.map