/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = runEffect;

var _uiMetrics = require("@servicenow/ui-metrics");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/**
 * Runs effect function (includes async and generator functions)
 *
 * @param fn
 * @param args
 * @returns {Promise}
 */
function runEffect(fn, args, _ref) {
  var host = _ref.host,
      interactionId = _ref.interactionId,
      type = _ref.type;
  (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.EFFECT_START, {
    type: type
  });
  var generator = fn.apply(void 0, _toConsumableArray(args));

  if (!generator || !generator.next) {
    (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.EFFECT_END, {
      type: type
    });
    return Promise.resolve(generator);
  }

  function runner(verb, arg) {
    if (fn.cancelled) {
      (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.EFFECT_END, {
        type: type
      });
      return Promise.resolve(generator["return"]());
    }

    var result;

    try {
      result = generator[verb](arg);
    } catch (err) {
      (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.EFFECT_END, {
        type: type
      });
      return Promise.reject(err);
    }

    if (result.done) {
      (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.EFFECT_END, {
        type: type
      });
      return Promise.resolve(result.value);
    }

    (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.EFFECT_END, {
      type: type
    });
    return Promise.resolve(result.value).then(onResolved, onRejected);
  }

  var onResolved = runner.bind(runner, 'next');
  var onRejected = runner.bind(runner, 'throw');
  return onResolved();
}
//# sourceMappingURL=runEffect.js.map