/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createTargetAttr = createTargetAttr;
exports.createTarget = createTarget;
exports.shouldHoistAttribute = shouldHoistAttribute;
exports.hoistAttribute = hoistAttribute;
exports.unhoistAttribute = unhoistAttribute;
exports.moveStyles = moveStyles;
exports.moveChildren = moveChildren;
exports["default"] = hoist;
exports.omitAttrs = void 0;

var _uiInternal = require("@servicenow/ui-internal");

var _constants = require("./constants");

var _symbols = require("./symbols");

var _utils = require("./utils");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {})); return true; } catch (e) { return false; } }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _CustomElement() {
  return Reflect.construct(HTMLElement, [], this.__proto__.constructor);
}

;
Object.setPrototypeOf(_CustomElement.prototype, HTMLElement.prototype);
Object.setPrototypeOf(_CustomElement, HTMLElement);

var SeismicHoistElement =
/*#__PURE__*/
function (_CustomElement2) {
  _inherits(SeismicHoistElement, _CustomElement2);

  var _super = _createSuper(SeismicHoistElement);

  function SeismicHoistElement() {
    var _this;

    _classCallCheck(this, SeismicHoistElement);

    _this = _super.call(this);
    var variables = {};
    _this[_symbols.VARIABLES] = variables;
    variables[_symbols.RETARGETED_ACTION_TYPES] = [];
    return _this;
  }

  return SeismicHoistElement;
}(_CustomElement);

var hoistTagName = _constants.SEISMIC_HOIST_TAG_NAME.toLowerCase();

window.customElements && !window.customElements.get(hoistTagName) && window.customElements.define(hoistTagName, SeismicHoistElement);

var cast = function cast(sourceEl, name, value) {
  return sourceEl[_symbols.VARIABLES][_symbols.PROPERTY_DEFS][name] && sourceEl[_symbols.VARIABLES][_symbols.PROPERTY_DEFS][name].unstableParse ? JSON.stringify(sourceEl[_symbols.VARIABLES][_symbols.PROPS][name]) : value;
};

function createTargetAttr(tagName, componentId) {
  return "".concat((0, _uiInternal.toLower)(tagName), "[component-id=\"").concat(componentId, "\"]");
}

function createTarget(sourceEl) {
  var targetEl = document.createElement(hoistTagName);
  var sourceVariables = (0, _utils.getHostVariables)(sourceEl);
  var targetVariables = (0, _utils.getHostVariables)(targetEl);
  var shadowRootOptions = sourceVariables[_symbols.CONFIG].shadowRootOptions;
  targetVariables[_symbols.ATTACHED_ROOT_NODE] = targetEl.attachShadow(shadowRootOptions);
  targetEl.setAttribute('target-for', createTargetAttr(sourceEl.tagName, sourceVariables[_symbols.COMPONENT_ID]));
  var attributes = sourceEl.attributes;
  var attributeCount = sourceEl.attributes.length;

  for (var i = 0; i < attributeCount; i++) {
    var _attributes$i = attributes[i],
        name = _attributes$i.name,
        value = _attributes$i.value;
    if (!omitAttrs.includes(name)) targetEl.setAttribute(name, cast(sourceEl, name, value));
  }

  targetEl.sourceElement = sourceEl;
  return targetEl;
}

var omitAttrs = [_constants.NOW_MODE_ATTR, _constants.NOW_COMPONENT_NAME_ATTR, _constants.NOW_HOIST_ATTR, _constants.NOW_HOIST_APPEND_TO_ATTR, _constants.NOW_HOIST_PREPEND_TO_ATTR, _constants.HOIST_ATTR, _constants.HOIST_APPEND_TO_ATTR, _constants.COMPONENT_ID_ATTRIBUTE, _constants.COMPONENT_NAME_ATTR];
exports.omitAttrs = omitAttrs;

var getParent = function getParent(element) {
  var parent = element.parentNode;
  if (parent && parent.host) return parent.host;
  return parent;
};

var overflowRegex = /(auto|scroll)/;

var getFirstScrollParent = function getFirstScrollParent(sourceEl) {
  var style = getComputedStyle(sourceEl);
  var excludeStaticParent = style.position === 'absolute';
  var parent = sourceEl;
  if (style.position === 'fixed') return document.body;

  while ((parent = getParent(parent)) !== null) {
    if (parent === document.body) break;
    style = getComputedStyle(parent);
    if (excludeStaticParent && style.position === 'static') continue;
    if (overflowRegex.test(style.overflow + style.overflowY + style.overflowX)) return parent;
  }

  return document.body;
};

var shouldPrepend = function shouldPrepend(sourceEl) {
  return sourceEl[_symbols.GET_RESOLVED_PROPERTY](_constants.NOW_HOIST_PREPEND_TO_PROP) || sourceEl.getAttribute(_constants.NOW_HOIST_PREPEND_TO_ATTR);
};

var shouldAppend = function shouldAppend(sourceEl) {
  return sourceEl[_symbols.GET_RESOLVED_PROPERTY](_constants.HOIST_APPEND_TO_PROP) || sourceEl.getAttribute(_constants.HOIST_APPEND_TO_ATTR) || sourceEl[_symbols.GET_RESOLVED_PROPERTY](_constants.NOW_HOIST_APPEND_TO_PROP) || sourceEl.getAttribute(_constants.NOW_HOIST_APPEND_TO_ATTR);
};

var getAppendToEl = function getAppendToEl(sourceEl) {
  // TODO: log and warn about legacy attributes, props
  var hoistAppendToSelector = shouldAppend(sourceEl);
  var hoistPrependToSelector = shouldPrepend(sourceEl);
  if (hoistAppendToSelector === _constants.HOIST_FIRST_SCROLL_PARENT || hoistPrependToSelector === _constants.HOIST_FIRST_SCROLL_PARENT) return getFirstScrollParent(sourceEl);
  var selectorOrNode = hoistPrependToSelector || hoistAppendToSelector;
  if (!selectorOrNode) return document.body;
  var node = typeof selectorOrNode === 'string' ? document.querySelector(selectorOrNode) : selectorOrNode instanceof HTMLElement ? selectorOrNode : undefined;
  return node || document.body;
};

function shouldHoistAttribute(host, name) {
  return (0, _utils.isHoisted)(host) && !omitAttrs.includes(name);
}

function hoistAttribute(host, name, value) {
  var hostVariables = (0, _utils.getHostVariables)(host);
  var target = (0, _utils.getHoistTarget)(hostVariables);
  if (target.tagName === _constants.SEISMIC_HOIST_TAG_NAME) target.setAttribute(name, value);
}

function unhoistAttribute(host, name) {
  var hostVariables = (0, _utils.getHostVariables)(host);
  var target = (0, _utils.getHoistTarget)(hostVariables);
  if (target.tagName === _constants.SEISMIC_HOIST_TAG_NAME) target.removeAttribute(name);
}

function moveStyles(sourceEl, targetEl, deprioritizeStyles, onStateChange) {
  var sourceVariables = (0, _utils.getHostVariables)(sourceEl);
  var targetVariables = (0, _utils.getHostVariables)(targetEl);
  if (deprioritizeStyles) onStateChange(targetVariables[_symbols.ATTACHED_ROOT_NODE], function () {
    return null;
  });

  if (sourceVariables[_symbols.ATTACHED_ROOT_NODE].adoptedStyleSheets && sourceVariables[_symbols.ATTACHED_ROOT_NODE].adoptedStyleSheets.length && !targetVariables[_symbols.ATTACHED_ROOT_NODE].adoptedStyleSheets.length) {
    targetVariables[_symbols.ATTACHED_ROOT_NODE].adoptedStyleSheets = _toConsumableArray(sourceVariables[_symbols.ATTACHED_ROOT_NODE].adoptedStyleSheets); // eslint-disable-line no-param-reassign
  } else {
    var sourceElRootNode = sourceVariables[_symbols.ATTACHED_ROOT_NODE];
    var sourceElRootNodeChildCount = sourceElRootNode.children.length;

    if (sourceElRootNodeChildCount && sourceElRootNode.firstChild.tagName === 'STYLE' && !targetVariables[_symbols.ATTACHED_ROOT_NODE].length) {
      targetVariables[_symbols.ATTACHED_ROOT_NODE].appendChild(sourceElRootNode.removeChild(sourceElRootNode.firstChild));

      while (sourceElRootNode.firstChild) {
        sourceElRootNode.removeChild(sourceElRootNode.firstChild);
      }
    }
  }
}

function moveChildren(sourceEl, targetEl) {
  if (sourceEl.children.length) {
    var fragment = document.createDocumentFragment();

    while (sourceEl.firstChild) {
      fragment.appendChild(sourceEl.removeChild(sourceEl.firstChild));
    }

    targetEl.appendChild(fragment);
  }
}

function connectTarget(sourceEl, targetEl) {
  var hoistAppendToEl = getAppendToEl(sourceEl);
  if (shouldPrepend(sourceEl)) hoistAppendToEl.prepend(targetEl);else hoistAppendToEl.appendChild(targetEl);
}

function disconnectTarget(targetEl) {
  targetEl.parentNode.removeChild(targetEl);
}

function hoist(sourceEl, deprioritizeStyles, onStateChange) {
  if (!sourceEl[_symbols.GET_RESOLVED_PROPERTY](_constants.HOIST_ATTR) && !sourceEl.hasAttribute(_constants.HOIST_ATTR) && !sourceEl[_symbols.GET_RESOLVED_PROPERTY](_constants.NOW_HOIST_PROP) && !sourceEl.hasAttribute(_constants.NOW_HOIST_ATTR)) return;
  var targetEl = createTarget(sourceEl);
  var sourceVariables = (0, _utils.getHostVariables)(sourceEl);
  var targetVariables = (0, _utils.getHostVariables)(targetEl); // these blocks cover the case when the hoist property is set to true after the source has already been connected and
  // possibly rendered. this appends the styles to the target shadowRoot then clears out the remaining child nodes
  // that were rendered in the source shadowRoot. it would be more efficient to never render the source shadowRoot,
  // but there isn't a way to know if a node will be hoisted in the future.

  moveStyles(sourceEl, targetEl, deprioritizeStyles, onStateChange);
  moveChildren(sourceEl, targetEl);
  connectTarget(sourceEl, targetEl);
  sourceVariables[_symbols.ROOT_NODE] = targetVariables[_symbols.ATTACHED_ROOT_NODE]; // eslint-disable-line no-param-reassign

  return function unhoist() {
    sourceVariables[_symbols.ROOT_NODE] = sourceVariables[_symbols.ATTACHED_ROOT_NODE]; // eslint-disable-line no-param-reassign

    moveStyles(targetEl, sourceEl, deprioritizeStyles, onStateChange);
    moveChildren(targetEl, sourceEl);
    disconnectTarget(targetEl);
  };
}
//# sourceMappingURL=hoist.js.map