/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createAddMapping;

var _constants = require("../constants");

/**
 * Create a function for mapping a provider compoent id, property selector, to a consumer component id, property name and path
 * Sample mapping Obj
 * mappings: {
 * 	providerComponent@providerProp: {
 * 		selectorId@selectorProp: 'pathValue'
 * 	}
 * }
 * @function createAddMapping
 * @private
 * @param {Object} mappings
 * @returns {function} - A function for adding a mapping
 */
function createAddMapping(mappings) {
  /**
   * @function addMapping
   * @private
   * @param {Object} providerDetail - A object that will contain providerId, providerPropName, providerPropPath
   * @param {Object} consumerDetail - A object that will contain consumerId, consumerPropName
   */
  return function addMapping(providerDetail, consumerDetail) {
    var providerId = providerDetail.providerId,
        providerPropName = providerDetail.providerPropName,
        providerPropPath = providerDetail.providerPropPath;
    var consumerId = consumerDetail.consumerId,
        consumerPropName = consumerDetail.consumerPropName;
    var providerKey = "".concat(providerId).concat(_constants.PROPERTY_SELECTOR_SIGIL).concat(providerPropName);
    var consumerKey = "".concat(consumerId).concat(_constants.PROPERTY_SELECTOR_SIGIL).concat(consumerPropName);
    if (!mappings[providerKey]) mappings[providerKey] = {}; // eslint-disable-line no-param-reassign
    // eslint-disable-next-line no-param-reassign

    mappings[providerKey][consumerKey] = providerPropPath;
  };
}
//# sourceMappingURL=createAddMapping.js.map