/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _constants = require("../constants");

var _isSelectableProperty = _interopRequireDefault(require("./isSelectableProperty"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Create a function for getting components affected by a property name and source component id
 * @function createGetAffected
 * @private
 *
 * @param {Object} mappings - A object that will contain mappings
 * @returns {function} - A function for getting components affected by a property name and source component id
 */
var _default = function _default(mappings) {
  return function getAffected(propertyName, providerId, componentRegistry) {
    var includeSource = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;
    var sourceAffected = includeSource ? _defineProperty({}, "".concat(providerId).concat(_constants.PROPERTY_SELECTOR_SIGIL).concat(propertyName), '') : {};

    var componentsAffected = _objectSpread({}, mappings["".concat(providerId).concat(_constants.PROPERTY_SELECTOR_SIGIL).concat(propertyName)]);

    if (!Object.keys(componentsAffected).length) return sourceAffected;
    if (!(0, _isSelectableProperty["default"])(componentRegistry, providerId, propertyName)) return sourceAffected;

    (function getAffectedSelectorChain(componentMap) {
      for (var selectorMap in componentMap) {
        if (mappings[selectorMap]) {
          componentsAffected = _objectSpread(_objectSpread({}, componentsAffected), mappings[selectorMap]);
          getAffectedSelectorChain(mappings[selectorMap]);
        }
      }
    })(componentsAffected);

    return _objectSpread(_objectSpread({}, sourceAffected), componentsAffected);
  };
};

exports["default"] = _default;
//# sourceMappingURL=createGetAffected.js.map