/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _constants = require("../constants");

/**
 * Create a function removing mapped components by component id and optionally property name
 * @function createRemoveMappings
 * @private
 *
 * @param {Object} mappings - A object that will contain mappings
 * @returns {function} - A function for removing mapped components by component id and optionally property name
 */
var _default = function _default(mappings) {
  return function removeMappings(elementId, propertyName) {
    if (elementId && propertyName) {
      for (var key in mappings) {
        var selectorKey = mappings[key]["".concat(elementId).concat(_constants.PROPERTY_SELECTOR_SIGIL).concat(propertyName)];
        if (selectorKey === '' || selectorKey) delete mappings[key]["".concat(elementId).concat(_constants.PROPERTY_SELECTOR_SIGIL).concat(propertyName)]; // eslint-disable-line no-param-reassign

        if (Object.keys(mappings[key]).length === 0) delete mappings[key]; // eslint-disable-line no-param-reassign
      }
    }

    if (elementId && !propertyName) {
      for (var _key in mappings) {
        for (var subKey in mappings[_key]) {
          var componentId = subKey.split(_constants.PROPERTY_SELECTOR_SIGIL)[0];
          if (componentId === elementId) delete mappings[_key][subKey]; // eslint-disable-line no-param-reassign
        }

        if (Object.keys(mappings[_key]).length === 0) delete mappings[_key]; // eslint-disable-line no-param-reassign
      }
    }
  };
};

exports["default"] = _default;
//# sourceMappingURL=createRemoveMappings.js.map