/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = getSelectorParts;

var _uiInternal = require("@servicenow/ui-internal");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var getParts = (0, _uiInternal.memoize)(function (value) {
  var slashIndex = value.indexOf('/');
  if (slashIndex === -1) return [];

  var _getDelimeterDetail = getDelimeterDetail(value),
      _getDelimeterDetail2 = _slicedToArray(_getDelimeterDetail, 2),
      delimeter = _getDelimeterDetail2[0],
      pathSplitIndex = _getDelimeterDetail2[1];

  return [value.substring(1, slashIndex), value.substring(slashIndex + 1, pathSplitIndex === -1 ? value.length : delimeter === '.' ? pathSplitIndex : pathSplitIndex + 1), pathSplitIndex !== -1 ? value.substring(pathSplitIndex + 1) : ''];
}, {
  profileName: 'getParts'
});

var getDelimeterDetail = function getDelimeterDetail(selector) {
  var selectorLength = selector.length;

  for (var i = 0; i < selectorLength; i++) {
    var value = selector.charAt(i);
    if (value === '.') return [value, i];
    if (value === '[') return [value, i - 1];
  }

  return ['', -1];
};
/**
 * Returns property selector parts, id and property name
 * @function getSelectorParts
 * @private
 *
 * @param {Object} mappings - A object that will contain mappings
 * @returns {Array} - Property selector parts, id, property name and path
 */


function getSelectorParts(value) {
  if (typeof value !== 'string') return [];
  if (value[0] !== '@') return [];
  return getParts(value);
}
//# sourceMappingURL=getSelectorParts.js.map