/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = resolveSelectedProperties;

var _index = require("./index");

/**
 * Returns property selector parts for a set of properties
 * @function resolveSelectedProperties
 * @private
 *
 * @param {Object} componentRegistry - Component registry
 * @param {function} propertyResolver - Returns the value for a property
 * @param {Object} properties - A set of properties to be resolved
 * @param {Object} propertiesDef - property definition from the config
 * @returns {Object} - A set of resolved properties
 */
function resolveSelectedProperties(componentRegistry, propertyResolver, properties) {
  var propertiesDef = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

  if (!hasSelectedProperty(componentRegistry, propertyResolver, properties, propertiesDef)) {
    return properties;
  }

  var resolved = {};

  for (var propName in properties) {
    resolved[propName] = (0, _index.resolveSelectedProperty)(componentRegistry, propertyResolver, properties[propName], propertiesDef[propName]);
  }

  return resolved;
}

function hasSelectedProperty(componentRegistry, propertyResolver, properties) {
  var propertiesDef = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

  for (var propName in properties) {
    var resolvedProp = (0, _index.resolveSelectedProperty)(componentRegistry, propertyResolver, properties[propName], propertiesDef[propName]);

    if (properties[propName] === resolvedProp) {
      continue;
    }

    return true;
  }
}
//# sourceMappingURL=resolveSelectedProperties.js.map