/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = resolveSelectedProperty;

var _ = require("./");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function resolveSelectorChain(id, name, path, propertyResolver, componentRegistry) {
  var property = propertyResolver(componentRegistry, id, name, path);

  if (property && property[0] === '@') {
    var _getSelectorParts = (0, _.getSelectorParts)(property),
        _getSelectorParts2 = _slicedToArray(_getSelectorParts, 3),
        _id = _getSelectorParts2[0],
        _name = _getSelectorParts2[1],
        _path = _getSelectorParts2[2];

    return resolveSelectorChain(_id, _name, _path, propertyResolver, componentRegistry);
  }

  return property;
}
/**
 * Returns property selector parts for a single property
 * @function resolveSelectedProperty
 * @private
 *
 * @param {Object} componentRegistry - Component registry
 * @param {function} propertyResolver - Returns the value for a property
 * @param {Object} property - property to be resolved
 * @param {Object} propertyDef - property definition from the config
 * @returns {Object} - The  resolved property
 */


function resolveSelectedProperty(componentRegistry, propertyResolver, property, propertyDef) {
  var _getSelectorParts3 = (0, _.getSelectorParts)(property),
      _getSelectorParts4 = _slicedToArray(_getSelectorParts3, 3),
      id = _getSelectorParts4[0],
      name = _getSelectorParts4[1],
      path = _getSelectorParts4[2];

  if (!id || !name) return property;
  var defaultValue = propertyDef && propertyDef["default"];
  if (!(0, _.isSelectableProperty)(componentRegistry, id, name)) return !(0, _.isSelector)(defaultValue) ? defaultValue : undefined;
  var resolvedValue = resolveSelectorChain(id, name, path, propertyResolver, componentRegistry);
  if (resolvedValue !== undefined) return resolvedValue;
  return !(0, _.isSelector)(defaultValue) ? defaultValue : undefined;
}
//# sourceMappingURL=resolveSelectedProperty.js.map