/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.schedule = schedule;
exports.countHighPriorityTasks = countHighPriorityTasks;
exports.countMediumPriorityTasks = countMediumPriorityTasks;
exports.addQueueEmptyForElCb = addQueueEmptyForElCb;
exports.countAllHighPriorityTasks = countAllHighPriorityTasks;
exports.countAllMediumPriorityTasks = countAllMediumPriorityTasks;
exports.types = void 0;

var _uiInternal = require("@servicenow/ui-internal");

var _uiConfig = require("@servicenow/ui-config");

var _uiMetrics = require("@servicenow/ui-metrics");

var _queue = _interopRequireDefault(require("./queue"));

var _metrics = _interopRequireDefault(require("./metrics"));

var _types;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var ENABLE_SCHEDULER_STATS = _uiConfig.configConstants.ENABLE_SCHEDULER_STATS,
    SCHEDULER_RAF_TIMEOUT = _uiConfig.configConstants.SCHEDULER_RAF_TIMEOUT;
var enableSchedulerStats = (0, _uiConfig.getConfigItem)(ENABLE_SCHEDULER_STATS);
var COMPONENT_RENDER = 'COMPONENT_RENDER';
var STATE_SET = 'STATE_SET';
var ACTION_HANDLER = 'ACTION_HANDLER';
var LEGACY_LIFECYCLE_FN = 'LEGACY_LIFECYCLE_FN';
var IMMEDIATE = 'IMMEDIATE';
var DEFER_IDLE = 'DEFER_IDLE';
var REFLECT_PROPERTIES = 'REFLECT_PROPERTIES';
var DISPATCH = 'DISPATCH';
var CLEANUP = 'CLEANUP';
var PROPERTY_SET = 'PROPERTY_SET';
var DEFER_HELPER_EXECUTION = 'DEFER_HELPER_EXECUTION';
var WHEN_COMPONENT_IS_IDLE = 'WHEN_COMPONENT_IS_IDLE';
var WHEN_ALL_COMPONENTS_IDLE = 'WHEN_ALL_COMPONENTS_IDLE';
var constants = {
  COMPONENT_RENDER: COMPONENT_RENDER,
  STATE_SET: STATE_SET,
  ACTION_HANDLER: ACTION_HANDLER,
  LEGACY_LIFECYCLE_FN: LEGACY_LIFECYCLE_FN,
  IMMEDIATE: IMMEDIATE,
  DEFER_IDLE: DEFER_IDLE,
  REFLECT_PROPERTIES: REFLECT_PROPERTIES,
  DISPATCH: DISPATCH,
  CLEANUP: CLEANUP,
  PROPERTY_SET: PROPERTY_SET,
  DEFER_HELPER_EXECUTION: DEFER_HELPER_EXECUTION,
  WHEN_COMPONENT_IS_IDLE: WHEN_COMPONENT_IS_IDLE,
  WHEN_ALL_COMPONENTS_IDLE: WHEN_ALL_COMPONENTS_IDLE
};
exports.types = constants;
var P0 = 'PRIORITY_0';
var P0_5 = 'PRIORITY_0_5';
var P1 = 'PRIORITY_1';
var P2 = 'PRIORITY_2';
var P3 = 'PRIORITY_3';
var p0_5Queue = new _queue["default"](false, []);
var p1Queue = new _queue["default"](true, [], whenQueueIsEmptyForElement);
var p2Queue = new _queue["default"](false, [], whenQueueIsEmptyForElement);
var p3Queue = new _queue["default"](true, [WHEN_COMPONENT_IS_IDLE, WHEN_ALL_COMPONENTS_IDLE]);
var queueEmptyForElCallbacks = [];
var stats = {};
var types = (_types = {}, _defineProperty(_types, IMMEDIATE, {
  priority: P0
}), _defineProperty(_types, DEFER_HELPER_EXECUTION, {
  priority: P0_5
}), _defineProperty(_types, COMPONENT_RENDER, {
  priority: P1
}), _defineProperty(_types, STATE_SET, {
  priority: P2
}), _defineProperty(_types, PROPERTY_SET, {
  priority: P2
}), _defineProperty(_types, ACTION_HANDLER, {
  priority: P2
}), _defineProperty(_types, LEGACY_LIFECYCLE_FN, {
  priority: P2
}), _defineProperty(_types, DISPATCH, {
  priority: P2
}), _defineProperty(_types, DEFER_IDLE, {
  priority: P3
}), _defineProperty(_types, REFLECT_PROPERTIES, {
  priority: P3
}), _defineProperty(_types, WHEN_COMPONENT_IS_IDLE, {
  priority: P3
}), _defineProperty(_types, WHEN_ALL_COMPONENTS_IDLE, {
  priority: P3
}), _types);
var idleTimeout = process.env.UNSAFE_SEISMIC_TASK_TIMEOUT ? 100 : -1;
var highPriorityPending = false;
var mediumPriorityPending = false;
var lowPriorityPending = false;
globalThis.nowUiFramework = globalThis.nowUiFramework || {};

globalThis.nowUiFramework.__getSchedulerStats = function () {
  return Object.entries(stats).sort(function (_ref, _ref2) {
    var _ref3 = _slicedToArray(_ref, 1),
        priority1 = _ref3[0];

    var _ref4 = _slicedToArray(_ref2, 1),
        priority2 = _ref4[0];

    return priority1.localeCompare(priority2);
  }).map(function (_ref5) {
    var _P0$P0_5$P1$P2$P3$pri;

    var _ref6 = _slicedToArray(_ref5, 2),
        priority = _ref6[0],
        _ref6$ = _ref6[1],
        maxQueueDepth = _ref6$.maxQueueDepth,
        totalQueued = _ref6$.totalQueued,
        totalQueueTime = _ref6$.totalQueueTime,
        maxQueueTime = _ref6$.maxQueueTime,
        totalExecutions = _ref6$.totalExecutions,
        totalExecutionTime = _ref6$.totalExecutionTime,
        maxExecutionTime = _ref6$.maxExecutionTime;

    return {
      priority: priority,
      inQueue: (_P0$P0_5$P1$P2$P3$pri = {}, _defineProperty(_P0$P0_5$P1$P2$P3$pri, P0, 0), _defineProperty(_P0$P0_5$P1$P2$P3$pri, P0_5, p0_5Queue.length), _defineProperty(_P0$P0_5$P1$P2$P3$pri, P1, p1Queue.length), _defineProperty(_P0$P0_5$P1$P2$P3$pri, P2, p2Queue.length), _defineProperty(_P0$P0_5$P1$P2$P3$pri, P3, p3Queue.length), _P0$P0_5$P1$P2$P3$pri)[priority],
      maxQueueDepth: maxQueueDepth,
      totalQueued: totalQueued,
      averageQueueTime: totalQueueTime / totalQueued,
      maxQueueTime: maxQueueTime,
      totalExecutions: totalExecutions,
      averageExecutionTime: totalExecutionTime / totalExecutions,
      maxExecutionTime: maxExecutionTime
    };
  });
};

globalThis.nowUiFramework.__resetSchedulerStats = function () {
  stats = {};
};

var getDefaultStats = function getDefaultStats() {
  return {
    maxQueueDepth: 0,
    totalQueued: 0,
    totalQueueTime: 0,
    maxQueueTime: 0,
    totalExecutions: 0,
    totalExecutionTime: 0,
    maxExecutionTime: 0
  };
};

var trackTaskScheduled = function trackTaskScheduled(task, priority, queue) {
  if (!enableSchedulerStats) return;
  stats[priority] = stats[priority] || getDefaultStats();
  task.scheduled = performance.now();
  stats[priority].totalQueued++;
  var queueSize = queue ? queue.length : 0;
  if (queueSize > stats[priority].maxQueueDepth) stats[priority].maxQueueDepth = queueSize;
};

var trackTaskStart = function trackTaskStart(task, priority) {
  if (!enableSchedulerStats) return;
  stats[priority] = stats[priority] || getDefaultStats();
  task.started = performance.now();
  var queueTime = task.started - task.scheduled;
  stats[priority].totalQueueTime += queueTime;
  if (queueTime > stats[priority].maxQueueTime) stats[priority].maxQueueTime = queueTime;
};

var trackTaskCompleted = function trackTaskCompleted(task, priority) {
  if (!enableSchedulerStats) return;
  stats[priority] = stats[priority] || getDefaultStats();
  var executionTime = performance.now() - task.started;
  stats[priority].totalExecutionTime += executionTime;
  stats[priority].totalExecutions++;
  if (executionTime > stats[priority].maxExecutionTime) stats[priority].maxExecutionTime = executionTime;
};

function schedule(task) {
  var type = task.type,
      fn = task.fn,
      _task$meta = task.meta,
      meta = _task$meta === void 0 ? {} : _task$meta;
  if (!fn) throw new Error('a fn is required for scheduling a task');
  if (!type) throw new Error('a type is required for scheduling a task');
  var _types$type$priority = types[type].priority,
      priority = _types$type$priority === void 0 ? P2 : _types$type$priority;

  switch (priority) {
    case P0:
      (0, _metrics["default"])(meta, _uiMetrics.types.SCHEDULE, type, 'P0', []);
      trackTaskScheduled(task, P0);
      trackTaskStart(task, priority);
      fn(meta);
      trackTaskCompleted(task, priority);
      break;

    case P0_5:
      (0, _metrics["default"])(meta, _uiMetrics.types.SCHEDULE, type, 'P0_5', p0_5Queue);
      p0_5Queue.push(task);
      trackTaskScheduled(task, priority, p0_5Queue);
      runDeferredTask();
      break;

    case P1:
      (0, _metrics["default"])(meta, _uiMetrics.types.SCHEDULE, type, 'P1', p1Queue);
      p1Queue.push(task);
      trackTaskScheduled(task, priority, p1Queue);
      runHighPriorityTasks();
      break;

    case P2:
      (0, _metrics["default"])(meta, _uiMetrics.types.SCHEDULE, type, 'P2', p2Queue);
      p2Queue.push(task);
      trackTaskScheduled(task, priority, p2Queue);
      runMediumPriorityTasks();
      break;

    case P3:
      (0, _metrics["default"])(meta, _uiMetrics.types.SCHEDULE, type, 'P3', p3Queue);
      p3Queue.push(task);
      trackTaskScheduled(task, priority, p3Queue);
      runLowPriorityTasks();
      break;
  }
}

var isRendering = false;
var requestAnimationFrameTimeout = (0, _uiConfig.getConfigItem)(SCHEDULER_RAF_TIMEOUT);

function runDeferredTask() {
  (0, _uiInternal.defer)(function () {
    var task = p0_5Queue.shift();

    if (task) {
      (0, _metrics["default"])(task.meta, _uiMetrics.types.EXECUTE, task.type, 'P0_5', p0_5Queue);
      trackTaskStart(task, P0_5);
      task.fn(task.meta);
      trackTaskCompleted(task, P0_5);
    }
  }, 0);
}

function runHighPriorityTasks() {
  if (highPriorityPending) return;
  highPriorityPending = true;
  isRendering = true;
  var requestAnimationFrameTimeoutId = (0, _uiInternal.defer)(function () {
    flushIdleQueues();
    isRendering = false;
  }, requestAnimationFrameTimeout);
  (0, _uiInternal.raf)(function () {
    clearTimeout(requestAnimationFrameTimeoutId);
    var task;
    var start = Date.now();

    while (task = p1Queue.shift()) {
      (0, _metrics["default"])(task.meta, _uiMetrics.types.EXECUTE, task.type, 'P1', p1Queue);
      trackTaskStart(task, P1);
      task.fn(task.meta);
      trackTaskCompleted(task, P1);
      if (Date.now() - start >= 100) break;
    }

    isRendering = false;
    highPriorityPending = false;
    if (p1Queue.length) runHighPriorityTasks();
    if (p2Queue.length) runMediumPriorityTasks();
    if (p3Queue.length) runLowPriorityTasks();
  });
}

function runMediumPriorityTasks() {
  if (mediumPriorityPending) return;
  if (isRendering) return;
  mediumPriorityPending = true;
  (0, _uiInternal.requestIdleCallbackPollyfill)(function (deadline) {
    var task;

    while (deadline.timeRemaining() > 1 && (task = p2Queue.shift())) {
      (0, _metrics["default"])(task.meta, _uiMetrics.types.EXECUTE, task.type, 'P2', p2Queue);
      trackTaskStart(task, P2);
      task.fn(task.meta);
      trackTaskCompleted(task, P2);
    }

    mediumPriorityPending = false;
    if (p2Queue.length) runMediumPriorityTasks();
    if (p3Queue.length) runLowPriorityTasks();
  }, {
    timeout: idleTimeout
  });
}

function runLowPriorityTasks() {
  if (mediumPriorityPending) return;
  if (lowPriorityPending) return;
  if (isRendering) return;
  lowPriorityPending = true;
  (0, _uiInternal.requestIdleCallbackPollyfill)(function (deadline) {
    var task;

    while (deadline.timeRemaining() > 1 && (task = p3Queue.shift())) {
      (0, _metrics["default"])(task.meta, _uiMetrics.types.EXECUTE, task.type, 'P3', p3Queue);
      trackTaskStart(task, P3);
      task.fn(task.meta);
      trackTaskCompleted(task, P3);
    }

    lowPriorityPending = false;
    if (p3Queue.length) runLowPriorityTasks();
  }, {
    timeout: idleTimeout
  });
}

function flushIdleQueues() {
  var task;

  while (task = p0_5Queue.shift()) {
    task.fn(task.meta);
  }

  while (task = p1Queue.shift()) {
    task.fn(task.meta);
  }

  highPriorityPending = false;

  while (task = p2Queue.shift()) {
    task.fn(task.meta);
  }

  while (task = p3Queue.shift()) {
    task.fn(task.meta);
  }
}

function isQueueFlushEnabled() {
  return process.env.UNSAFE_SEISMIC_EMIT_CUSTOM_EVENT || window.nowUiFramework.queueFlushEnabled === true;
}

var flushing = false; // eslint-disable-next-line no-inner-declarations

function flushQueues() {
  flushing = true;
  var task;

  while (task = p0_5Queue.shift()) {
    task.fn(task.meta);
  }

  while (task = p1Queue.shift()) {
    task.fn(task.meta);
  }

  while (task = p2Queue.shift()) {
    task.fn(task.meta);
  }

  while (task = p3Queue.shift()) {
    task.fn(task.meta);
  }

  (0, _uiInternal.defer)(function () {
    if (p0_5Queue.length || p1Queue.length || p2Queue.length || p3Queue.length) return flushQueues();
    flushing = false;
    var event = new CustomEvent('SEISMIC_QUEUE_FLUSHED', {
      composed: true,
      bubbles: true
    });
    return document.dispatchEvent(event);
  }, 0);
}

document.addEventListener('FLUSH_SEISMIC_QUEUE', function () {
  if (!flushing && isQueueFlushEnabled()) flushQueues();
});
document.addEventListener('visibilitychange', function () {
  if ((0, _uiInternal.isThrottled)()) flushIdleQueues();
}, false);

function countHighPriorityTasks(element) {
  return p1Queue.sizeForElement(element);
}

function countAllHighPriorityTasks() {
  return p1Queue.length;
}

function countAllMediumPriorityTasks() {
  return p2Queue.length;
}

function countMediumPriorityTasks(element) {
  return p2Queue.sizeForElement(element);
}

function addQueueEmptyForElCb(fn) {
  queueEmptyForElCallbacks.push(fn);
}

function whenQueueIsEmptyForElement(el) {
  var len = queueEmptyForElCallbacks.length;

  for (var i = 0; i < len; i++) {
    queueEmptyForElCallbacks[i](el);
  }
}
//# sourceMappingURL=index.js.map