/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * Maintains task insert order but (when dedupe==true) will skip earlier tasks
 * if there's another task for the same element enqueued later
 */
var Queue =
/*#__PURE__*/
function () {
  function Queue(dedupe) {
    var doNotDedup = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    var whenQueueIsEmptyForElementCb = arguments.length > 2 ? arguments[2] : undefined;

    _classCallCheck(this, Queue);

    this._els = new WeakMap();
    this._tasks = [];
    this._dedupe = dedupe;
    this._doNotDedup = doNotDedup;
    this._whenQueueIsEmptyForElementCb = whenQueueIsEmptyForElementCb;
  }

  _createClass(Queue, [{
    key: "push",
    value: function push(task) {
      var el = task.meta.el;

      this._els.set(el, this.sizeForElement(el) + 1);

      this._tasks.push(task);
    } // this value may be a little misleading because it doesn't account for
    // possible task duplicates

  }, {
    key: "length",
    get: function get() {
      return this._tasks.length;
    }
  }, {
    key: "sizeForElement",
    value: function sizeForElement(el) {
      return this._els.get(el) || 0;
    }
  }, {
    key: "shift",
    value: function shift() {
      while (this._tasks.length) {
        var task = this._tasks.shift();

        var el = task.meta.el;

        var existingCount = this._els.get(el);

        if (existingCount === 1) {
          this._els["delete"](el);

          if (this._whenQueueIsEmptyForElementCb) this._whenQueueIsEmptyForElementCb(el);
        } else {
          this._els.set(el, existingCount - 1);
        }

        if (existingCount === 1 || !this._dedupe || this._doNotDedup.indexOf(task.type) !== -1 || !el.isConnected) return task;
      }
    }
  }]);

  return Queue;
}();

exports["default"] = Queue;
//# sourceMappingURL=queue.js.map