/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.reflectProperty = reflectProperty;
exports["default"] = reflectProperties;

var _uiMetrics = require("@servicenow/ui-metrics");

var _uiInternal = require("@servicenow/ui-internal");

var _utils = require("../utils");

var _reflectQueue = _interopRequireDefault(require("../utils/reflectQueue"));

var _scheduler = require("../scheduler");

var _symbols = require("../symbols");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// TODO: why is this required?
var REFLECT_PROPERTIES = _scheduler.types.REFLECT_PROPERTIES;
var isAria = (0, _uiInternal.memoize)(function (name) {
  return name.indexOf('aria') === 0;
}, {
  profileName: 'isAria'
});

var shouldRemoveAttribute = function shouldRemoveAttribute(name, value) {
  if (isAria(name)) return !(0, _uiInternal.toString)(value);
  return !value;
};

var getAttributeValue = function getAttributeValue(name, value) {
  if (isAria(name)) return (0, _uiInternal.toString)(value);
  return value === true ? '' : value;
};

var filterRelectedProperties = (0, _uiInternal.memoize)(function (host) {
  var hostVariables = (0, _utils.getHostVariables)(host);
  var properties = (0, _utils.filterPropertyDefinitions)(hostVariables[_symbols.PROPERTY_DEFS], function (d) {
    return d.reflect;
  });
  var keys = Object.keys(properties);
  return {
    properties: properties,
    keys: keys
  };
}, {
  createCacheKey: function createCacheKey(host) {
    return host.tagName;
  },
  profileName: 'filterRelectedProperties'
});
var filterUnstableParseProperties = (0, _uiInternal.memoize)(function (host) {
  var hostVariables = (0, _utils.getHostVariables)(host);
  return (0, _utils.filterPropertyDefinitions)(hostVariables[_symbols.PROPERTY_DEFS], function (d) {
    return d.unstableParse;
  });
}, {
  createCacheKey: function createCacheKey(host) {
    return host.tagName;
  },
  profileName: 'filterUnstableParseProperties'
});

var resolveValue = function resolveValue(propName, propDef, state) {
  if (propDef.computed) {
    var computedValue = propDef.computed(state);
    if (computedValue !== undefined) return computedValue;
  }

  var stateValue = state.properties[propName];
  if (stateValue !== undefined) return stateValue;
  return propDef["default"] || null;
};

function reflectProperty(propName, propDef, state, host, usingUnstableParse) {
  var value = resolveValue(propName, propDef, state);
  var attributeName = (0, _utils.toAttributeName)(propName);
  var attributeValue = getAttributeValue(attributeName, value);

  if (shouldRemoveAttribute(attributeName, value) && host.hasAttribute(attributeName)) {
    _reflectQueue["default"].push(attributeName);

    host.removeAttribute(attributeName);
  } else {
    if (host.getAttribute(attributeName) !== attributeValue && (value !== null && value !== undefined && value !== false || isAria(attributeName) && attributeValue)) {
      _reflectQueue["default"].push(attributeName);

      if (usingUnstableParse[propName]) host.setAttribute(attributeName, JSON.stringify(attributeValue), true);else host.setAttribute(attributeName, attributeValue, true);
    }
  }
}

function reflectProperties(state, host, interactionId) {
  var _filterRelectedProper = filterRelectedProperties(host),
      reflected = _filterRelectedProper.properties,
      keys = _filterRelectedProper.keys;

  var usingUnstableParse = filterUnstableParseProperties(host);
  (0, _scheduler.schedule)({
    fn: function fn() {
      (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.PROPERTY_REFLECTION, {
        properties: keys
      });

      for (var propName in reflected) {
        reflectProperty(propName, reflected[propName], state, host, usingUnstableParse);
      }
    },
    type: REFLECT_PROPERTIES,
    meta: {
      el: host,
      interactionId: interactionId
    }
  });
  return state;
}
//# sourceMappingURL=reflectProperties.js.map