/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.set = exports.get = exports.defineRegistry = void 0;

var _package = require("../package.json");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

var componentRegistry = {};
var stats = {
  constructedByTagName: {},
  definedTagNames: [],
  deferredByTagName: {}
};

var defineRegistry = function defineRegistry(registry) {
  componentRegistry = registry;
  stats.constructedByTagName = {};
  stats.definedTagNames = [];
  stats.deferredByTagName = {};
};

exports.defineRegistry = defineRegistry;

var get = function get() {
  var countByTagName = {};

  for (var k in componentRegistry) {
    var tagName = componentRegistry[k].tagName.toLowerCase();
    if (!countByTagName[tagName]) countByTagName[tagName] = 0;
    countByTagName[tagName]++; // eslint-disable-line no-plusplus
  }

  var totalConstructed = 0;

  for (var _k in stats.constructedByTagName) {
    totalConstructed += stats.constructedByTagName[_k];
  }

  var unusedTagNames = [];

  for (var i = 0; i < stats.definedTagNames.length; i++) {
    var _tagName = stats.definedTagNames[i];

    if (!stats.constructedByTagName[_tagName]) {
      unusedTagNames.push(_tagName);
    }
  }

  return {
    version: _package.version,
    definitions: {
      total: stats.definedTagNames.length,
      unusedTagNames: unusedTagNames,
      tagNames: _toConsumableArray(stats.definedTagNames)
    },
    constructed: {
      total: totalConstructed,
      countByTagName: stats.constructedByTagName
    },
    connected: {
      total: Object.keys(componentRegistry).length,
      countByTagName: countByTagName,
      deferredByTagName: stats.deferredByTagName,
      nodes: _objectSpread({}, componentRegistry)
    }
  };
};

exports.get = get;

var set = function set(statName, tagName) {
  switch (statName) {
    case 'constructed':
      stats.constructedByTagName[tagName] = stats.constructedByTagName[tagName] || 0;
      stats.constructedByTagName[tagName]++; // eslint-disable-line no-plusplus

      break;

    case 'defined':
      stats.definedTagNames.push(tagName);
      break;

    case 'defer':
      stats.deferredByTagName[tagName] = stats.deferredByTagName[tagName] || 0;
      stats.deferredByTagName[tagName]++; // eslint-disable-line no-plusplus

      break;

    case 'undefer':
      stats.deferredByTagName[tagName] = stats.deferredByTagName[tagName] || 0;
      if (stats.deferredByTagName[tagName]) stats.deferredByTagName[tagName]--; // eslint-disable-line no-plusplus

      break;
  }
};

exports.set = set;
//# sourceMappingURL=stats.js.map