/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = applyIntersectStyles;

var _getHostVariables = _interopRequireDefault(require("./getHostVariables"));

var _symbols = require("../symbols");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function applyIntersectStyles(el, config, options) {
  var width = options.width,
      height = options.height;
  if (!width && !height) return;
  var hostVariables = (0, _getHostVariables["default"])(el);
  var rootNode = hostVariables[_symbols.ROOT_NODE];
  var styles = ":host {".concat(width ? "width: ".concat(width, ";") : '').concat(height ? "height: ".concat(height, ";") : '', "}");
  return applyWithTag(rootNode, styles);
}

function applyWithTag(rootNode, styles) {
  var styleElement = window.document.createElement('style');
  styleElement.type = 'text/css';
  styleElement.innerHTML = styles;
  rootNode.appendChild(styleElement);
  return function () {
    return rootNode.removeChild(styleElement);
  };
}
//# sourceMappingURL=applyIntersectStyles.js.map