/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = applyMetrics;

var _uiMetrics = require("@servicenow/ui-metrics");

var _normalizeActionMeta = require("./normalizeActionMeta");

function applyMetricsToDispatch(dispatch) {
  return function (type, payload) {
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var error = arguments.length > 3 ? arguments[3] : undefined;
    // eslint-disable-next-line no-param-reassign
    meta = (0, _normalizeActionMeta.normalizeActionMeta)(meta, {});
    (0, _uiMetrics.applyInteractionId)(meta);
    return dispatch(type, payload, meta, error);
  };
}

function applyMetrics(helpers) {
  var interactionId = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : (0, _uiMetrics.createInteractionId)();
  var withMetrics = typeof helpers === 'function' ? applyMetricsToDispatch(helpers) : {};
  if (helpers.updateState) withMetrics.updateState = function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return helpers.updateState.apply(helpers, args.concat([interactionId]));
  };
  if (helpers.updateContext) withMetrics.updateContext = function () {
    for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    return helpers.updateContext.apply(helpers, args.concat([interactionId]));
  };
  if (helpers.updateProperties) withMetrics.updateProperties = function () {
    for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    return helpers.updateProperties.apply(helpers, args.concat([interactionId]));
  };
  if (helpers.dispatch) withMetrics.dispatch = applyMetricsToDispatch(helpers.dispatch);
  return withMetrics;
}
//# sourceMappingURL=applyMetrics.js.map