/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createInterceptors = createInterceptors;

var _uiInternal = require("@servicenow/ui-internal");

var _getHostVariables = _interopRequireDefault(require("../getHostVariables"));

var _symbols = require("../../symbols");

var _log = _interopRequireDefault(require("../../log"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var INTERCEPTOR = _uiInternal.locations.INTERCEPTOR;

function sandboxInterceptor(effectInterceptor) {
  var fn = effectInterceptor.fn,
      type = effectInterceptor.type,
      position = effectInterceptor.position;
  return _objectSpread(_objectSpread({}, effectInterceptor), {}, {
    fn: function sandboxedInterceptor(context) {
      var host = context.coeffects.host;
      if (type === 'effect') // don't need to wrap seismic defined interceptors
        return fn(context);
      var hostVariables = (0, _getHostVariables["default"])(host);
      return (0, _uiInternal.sandbox)(fn, {
        args: [context],
        location: INTERCEPTOR,
        host: host,
        details: {
          type: type,
          position: position
        },
        dispatch: hostVariables[_symbols.DISPATCH],
        log: _log["default"]
      });
    }
  });
}

function forEachInterceptor(interceptors, target, map) {
  map.set(target, interceptors.map(sandboxInterceptor));
}

function createInterceptors(actionHandlers) {
  var actionHandlerInterceptors = {};

  for (var k in actionHandlers) {
    actionHandlerInterceptors[k] = new Map();
    var handlerCount = actionHandlers[k].length;

    var _loop = function _loop(i) {
      var _actionHandlers$k$i = actionHandlers[k][i],
          _actionHandlers$k$i$i = _actionHandlers$k$i.interceptors,
          before = _actionHandlers$k$i$i.before,
          after = _actionHandlers$k$i$i.after,
          args = _actionHandlers$k$i.args,
          effect = _actionHandlers$k$i.effect,
          _actionHandlers$k$i$h = _actionHandlers$k$i.handlers,
          handlers = _actionHandlers$k$i$h === void 0 ? [] : _actionHandlers$k$i$h,
          _actionHandlers$k$i$t = _actionHandlers$k$i.target,
          target = _actionHandlers$k$i$t === void 0 ? 'host' : _actionHandlers$k$i$t,
          behaviorOptions = _actionHandlers$k$i.behaviorOptions,
          capture = _actionHandlers$k$i.capture,
          eventHandler = _actionHandlers$k$i.eventHandler,
          modifier = _actionHandlers$k$i.modifier,
          descendantOnly = _actionHandlers$k$i.descendantOnly;
      var combinedInterceptors = actionHandlerInterceptors[k].has(target) ? actionHandlerInterceptors[k].get(target) : [];
      var effectInterceptor = effect ? {
        fn: function createEffectHandlerInterceptor(context) {
          return _objectSpread(_objectSpread({}, context), {}, {
            effects: [].concat(_toConsumableArray(context.effects), [{
              effect: effect,
              args: [].concat(_toConsumableArray(args), [context.coeffects]),
              modifier: modifier,
              options: behaviorOptions,
              descendantOnly: descendantOnly
            }])
          });
        },
        type: 'effect',
        capture: capture,
        eventHandler: eventHandler
      } : null;
      var handlerInterceptors = handlers.map(function (handler) {
        return {
          fn: function createHandlersInterceptor(context) {
            var effect = handler.effect,
                _handler$args = handler.args,
                args = _handler$args === void 0 ? [] : _handler$args;
            return _objectSpread(_objectSpread({}, context), {}, {
              effects: [].concat(_toConsumableArray(context.effects), [{
                effect: effect,
                args: [].concat(_toConsumableArray(args), [context.coeffects]),
                modifier: modifier,
                options: behaviorOptions,
                descendantOnly: descendantOnly
              }])
            });
          },
          type: 'effect',
          capture: capture,
          eventHandler: eventHandler
        };
      });
      combinedInterceptors = [].concat(_toConsumableArray(combinedInterceptors), _toConsumableArray(before), _toConsumableArray(effectInterceptor ? [effectInterceptor] : []), _toConsumableArray(handlerInterceptors), _toConsumableArray(after));
      actionHandlerInterceptors[k].set(target, combinedInterceptors);
    };

    for (var i = 0; i < handlerCount; i++) {
      _loop(i);
    }

    actionHandlerInterceptors[k].forEach(forEachInterceptor);
  }

  return actionHandlerInterceptors;
}
//# sourceMappingURL=createInterceptors.js.map