/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getActionsToDispatch = getActionsToDispatch;

var _constants = require("../../constants");

var actions = {
  COMPONENT_CONNECTED: _constants.COMPONENT_CONNECTED,
  COMPONENT_DISCONNECTED: _constants.COMPONENT_DISCONNECTED,
  COMPONENT_BOOTSTRAPPED: _constants.COMPONENT_BOOTSTRAPPED,
  COMPONENT_RENDERED: _constants.COMPONENT_RENDERED,
  COMPONENT_DOM_READY: _constants.COMPONENT_DOM_READY,
  COMPONENT_PROPERTY_CHANGED: _constants.COMPONENT_PROPERTY_CHANGED,
  COMPONENT_PROPERTY_UPDATE_REQUESTED: _constants.COMPONENT_PROPERTY_UPDATE_REQUESTED,
  COMPONENT_RENDER_REQUESTED: _constants.COMPONENT_RENDER_REQUESTED,
  COMPONENT_DOM_TREE_READY: _constants.COMPONENT_DOM_TREE_READY,
  COMPONENT_TREE_RENDERED: _constants.COMPONENT_TREE_RENDERED,
  COMPONENT_STATE_UPDATED: _constants.COMPONENT_STATE_UPDATED,
  COMPONENT_INTERACTIVE: _constants.COMPONENT_INTERACTIVE
};

function getActionsToDispatch(interceptors) {
  var shouldDispatch = {};

  for (var k in actions) {
    var type = actions[k];
    var intercpetorsByType = interceptors[type];
    if (intercpetorsByType && intercpetorsByType.has('host')) shouldDispatch[type] = true;
  }

  return shouldDispatch;
}
//# sourceMappingURL=getActionsToDispatch.js.map