/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mergeBehaviorActions = mergeBehaviorActions;

var _log = _interopRequireDefault(require("../../log"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function mergeBehaviorActions(actions, behaviors, tagName) {
  var mergedActions = {};
  var behaviorsCount = behaviors.length;
  var behaviorNames = {};

  for (var i = 0; i < behaviorsCount; i++) {
    var _behaviors$i = behaviors[i],
        name = _behaviors$i.name,
        _behaviors$i$behavior = _behaviors$i.behavior.actions,
        behaviorActions = _behaviors$i$behavior === void 0 ? {} : _behaviors$i$behavior;

    for (var k in behaviorActions) {
      if (mergedActions[k]) {
        (0, _log["default"])("Behavior ".concat(name, " overwrote action ").concat(k, " for ").concat(tagName, "."), {
          tagName: tagName,
          origin: 'mergeBehaviorActions'
        });
      }

      mergedActions[k] = behaviorActions[k];
      behaviorNames[k] = name;
    }
  }

  for (var _k in actions) {
    if (mergedActions[_k]) {
      (0, _log["default"])("The component definition for ".concat(tagName, " overwrote action ").concat(_k, " defined by the ").concat(behaviorNames[_k], " behavior."), {
        tagName: tagName,
        origin: 'mergeBehaviorActions'
      });
    }

    mergedActions[_k] = actions[_k];
  }

  return mergedActions;
}
//# sourceMappingURL=mergeBehaviorActions.js.map