/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mergeBehaviorInitialState = mergeBehaviorInitialState;

var _log = _interopRequireDefault(require("../../log"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// this code executes before an instance is created. @servicenow/ui-internal/sandbox currently relies on host and dispatch.
// refactoring @servicenow/ui-internal/sandbox to support this single use case - all other user code is executed after an
// instance is created - didn't seem worth it. Even if we move to a lazy configuration initialization in the constructor
// dispatch won't be created until connectedCallback is executed.
function sandbox(fn, options, message) {
  try {
    return fn(options);
  } catch (error) {
    (0, _log["default"])(message, {
      origin: 'sandbox',
      host: options ? options.host : null
    });
    return {};
  }
}

function mergeBehaviorInitialState(behaviors, componentInitialState, componentSetInitialState, tagName) {
  var behaviorState = {};
  var behaviorsCount = behaviors.length;

  for (var i = 0; i < behaviorsCount; i++) {
    var _behaviors$i = behaviors[i],
        name = _behaviors$i.name,
        options = _behaviors$i.options,
        _behaviors$i$behavior = _behaviors$i.behavior,
        behaviorInitialState = _behaviors$i$behavior.initialState,
        behaviorOptions = _behaviors$i$behavior.options;
    behaviorState[name] = typeof behaviorInitialState === 'function' ? sandbox(behaviorInitialState, options || behaviorOptions, "The initialState function for the ".concat(name, " behavior threw an error when defining <").concat(tagName, " />")) : behaviorInitialState;
  }

  function setInitialState(_ref) {
    var host = _ref.host,
        properties = _ref.properties;
    var behaviorStateDynamic = behaviorState;

    for (var _i = 0; _i < behaviorsCount; _i++) {
      var _behaviors$_i = behaviors[_i],
          _name = _behaviors$_i.name,
          _options = _behaviors$_i.options,
          _behaviors$_i$behavio = _behaviors$_i.behavior,
          behaviorSetInitialState = _behaviors$_i$behavio.setInitialState,
          _behaviorOptions = _behaviors$_i$behavio.options;

      if (typeof behaviorSetInitialState === 'function') {
        behaviorStateDynamic[_name] = _objectSpread(_objectSpread({}, behaviorState[_name]), sandbox(behaviorSetInitialState, {
          host: host,
          properties: properties,
          options: _options || _behaviorOptions
        }, "The setInitialState function for the ".concat(_name, " behavior threw an error when defining <").concat(tagName, " />")));
      }
    }

    return _objectSpread(_objectSpread({}, Object.keys(behaviorStateDynamic).length ? {
      behaviors: behaviorStateDynamic
    } : {}), typeof componentSetInitialState === 'function' && sandbox(componentSetInitialState, {
      host: host,
      properties: properties
    }, "The setInitialState function for ".concat(tagName, " threw an error")));
  }

  var initialState = _objectSpread(_objectSpread({}, Object.keys(behaviorState).length ? {
    behaviors: behaviorState
  } : {}), componentInitialState);

  return {
    initialState: initialState,
    setInitialState: setInitialState
  };
}
//# sourceMappingURL=mergeBehaviorInitialState.js.map