/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mergeBehaviorProperties = void 0;

var _uiInternal = require("@servicenow/ui-internal");

var _toPropertyName = _interopRequireDefault(require("../toPropertyName"));

var _constants = require("../../constants");

var _log = _interopRequireDefault(require("../../log"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var mergeBehaviorProperties = (0, _uiInternal.memoize)(mergeBehaviorPropertiesInner, {
  profileName: 'mergeBehaviorProperties',
  createCacheKey: function createCacheKey(b, p, t) {
    return t;
  }
});
exports.mergeBehaviorProperties = mergeBehaviorProperties;

function mergeBehaviorPropertiesInner(behaviors, properties, tagName) {
  var _objectSpread2;

  var mergedProperties = {};
  var behaviorsCount = behaviors.length;
  var behaviorNames = {};

  for (var k in properties) {
    mergedProperties[k] = properties[k];
  }

  for (var i = 0; i < behaviorsCount; i++) {
    var _behaviors$i = behaviors[i],
        name = _behaviors$i.name,
        behaviorProperties = _behaviors$i.behavior.properties;

    for (var _k in behaviorProperties) {
      if (mergedProperties[_k]) {
        (0, _log["default"])("Behavior ".concat(name, " overwrote property ").concat(_k, " for ").concat(tagName, "."), {
          tagName: tagName,
          origin: 'mergeBehaviorProperties'
        });
      }

      mergedProperties[_k] = behaviorProperties[_k];
      behaviorNames[_k] = name;
    }
  }

  return _objectSpread(_objectSpread({}, mergedProperties), {}, (_objectSpread2 = {}, _defineProperty(_objectSpread2, (0, _toPropertyName["default"])(_constants.DEFER_ATTRIBUTE), {}), _defineProperty(_objectSpread2, (0, _toPropertyName["default"])(_constants.HOIST_ATTR), _objectSpread(_objectSpread({}, mergedProperties[(0, _toPropertyName["default"])(_constants.HOIST_ATTR)] || {}), {}, {
    reflect: true
  })), _defineProperty(_objectSpread2, (0, _toPropertyName["default"])(_constants.HOIST_APPEND_TO_ATTR), _objectSpread(_objectSpread({}, mergedProperties[(0, _toPropertyName["default"])(_constants.HOIST_APPEND_TO_ATTR)] || {}), {}, {
    reflect: true
  })), _defineProperty(_objectSpread2, (0, _toPropertyName["default"])(_constants.COMPONENT_NAME_ATTR), {
    reflect: true
  }), _defineProperty(_objectSpread2, _constants.NOW_HOIST_PROP, _objectSpread(_objectSpread({}, mergedProperties[_constants.NOW_HOIST_PROP] || {}), {}, {
    reflect: true
  })), _defineProperty(_objectSpread2, _constants.NOW_HOIST_APPEND_TO_PROP, _objectSpread(_objectSpread({}, mergedProperties[_constants.NOW_HOIST_APPEND_TO_PROP] || {}), {}, {
    reflect: true
  })), _defineProperty(_objectSpread2, _constants.NOW_HOIST_PREPEND_TO_PROP, _objectSpread(_objectSpread({}, mergedProperties[_constants.NOW_HOIST_PREPEND_TO_PROP] || {}), {}, {
    reflect: true
  })), _defineProperty(_objectSpread2, _constants.NOW_PLACEHOLDER_HEIGHT_PROP, _objectSpread(_objectSpread({}, mergedProperties[_constants.NOW_PLACEHOLDER_HEIGHT_PROP] || {}), {}, {
    schema: {
      type: 'string'
    }
  })), _defineProperty(_objectSpread2, _constants.NOW_PLACEHOLDER_WIDTH_PROP, _objectSpread(_objectSpread({}, mergedProperties[_constants.NOW_PLACEHOLDER_WIDTH_PROP] || {}), {}, {
    schema: {
      type: 'string'
    }
  })), _defineProperty(_objectSpread2, _constants.NOW_ID_PROP, {}), _defineProperty(_objectSpread2, _constants.NOW_MODE_PROP, {}), _defineProperty(_objectSpread2, _constants.NOW_COMPONENT_NAME_PROP, {}), _defineProperty(_objectSpread2, _constants.NOW_ENABLE_PROPERTY_SETTING_PREVENTION_PROP, {}), _defineProperty(_objectSpread2, _constants.NOW_LOGGING_LEVELS_PROP, _objectSpread(_objectSpread({}, mergedProperties[_constants.NOW_LOGGING_LEVELS_PROP] || []), {}, {
    schema: {
      type: ['array', 'boolean'],
      items: {
        type: 'string'
      }
    }
  })), _defineProperty(_objectSpread2, _constants.NOW_METRIC_TYPES_PROP, _objectSpread(_objectSpread({}, mergedProperties[_constants.NOW_METRIC_TYPES_PROP] || {}), {}, {
    schema: {
      type: 'array',
      items: {
        type: 'string'
      }
    }
  })), _defineProperty(_objectSpread2, _constants.NOW_METRICS_LEVEL_PROP, _objectSpread(_objectSpread({}, mergedProperties[_constants.NOW_METRICS_LEVEL_PROP] || {}), {}, {
    schema: {
      type: 'integer',
      minimum: 0,
      maximum: 5
    }
  })), _defineProperty(_objectSpread2, _constants.NOW_PRIORITY_PROP, _objectSpread(_objectSpread({}, mergedProperties[_constants.NOW_PRIORITY_PROP] || {}), {}, {
    schema: {
      type: 'string',
      "enum": [_constants.PRIORITY_LEVEL_LOW, _constants.PRIORITY_LEVEL_MEDIUM]
    }
  })), _defineProperty(_objectSpread2, _constants.NOW_AVOID_RENDER_PROP, _objectSpread(_objectSpread({}, mergedProperties[_constants.NOW_AVOID_RENDER_PROP] || {
    "default": false
  }), {}, {
    schema: {
      type: 'boolean'
    }
  })), _objectSpread2));
}
//# sourceMappingURL=mergeBehaviorProperties.js.map