/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mergeEventActionHandlers = mergeEventActionHandlers;

var _normalizeActionHandler = require("./normalizeActionHandler");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function mergeEventActionHandlers(actionHandlers, eventHandlers, tagName) {
  var eventHandlerCount = eventHandlers.length;

  for (var i = 0; i < eventHandlerCount; i++) {
    var _eventHandlers$i = eventHandlers[i],
        _eventHandlers$i$even = _eventHandlers$i.events,
        events = _eventHandlers$i$even === void 0 ? [] : _eventHandlers$i$even,
        effect = _eventHandlers$i.effect,
        _eventHandlers$i$modi = _eventHandlers$i.modifiers,
        modifiers = _eventHandlers$i$modi === void 0 ? [] : _eventHandlers$i$modi,
        _eventHandlers$i$modi2 = _eventHandlers$i.modifier,
        modifier = _eventHandlers$i$modi2 === void 0 ? {} : _eventHandlers$i$modi2,
        _eventHandlers$i$inte = _eventHandlers$i.interceptors,
        interceptors = _eventHandlers$i$inte === void 0 ? [] : _eventHandlers$i$inte,
        target = _eventHandlers$i.target,
        capture = _eventHandlers$i.capture,
        behaviorOptions = _eventHandlers$i.behaviorOptions;
    var eventsCount = events.length;

    for (var j = 0; j < eventsCount; j++) {
      var eventName = events[j];
      actionHandlers[eventName] = actionHandlers[eventName] || []; // eslint-disable-line no-param-reassign

      var normalizedActionHandler = _objectSpread(_objectSpread({}, (0, _normalizeActionHandler.normalizeActionHandler)({
        effect: effect,
        modifiers: modifiers,
        modifier: modifier,
        interceptors: interceptors,
        target: target,
        capture: capture,
        eventHandler: true
      }, tagName)), behaviorOptions && {
        behaviorOptions: behaviorOptions
      });

      actionHandlers[eventName].push(normalizedActionHandler);
    }
  }

  return actionHandlers;
}
//# sourceMappingURL=mergeEventActionHandlers.js.map