/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.normalizeActionHandler = normalizeActionHandler;

var _normalizeModifiers = _interopRequireDefault(require("./normalizeModifiers"));

var _log = _interopRequireDefault(require("../../log"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function typeInterceptor(fn, type, position) {
  return {
    fn: fn,
    type: type,
    position: position
  };
}

function mapInterceptors(interceptors) {
  var mapped = {
    before: [],
    after: []
  };
  var interceptorCount = interceptors.length;

  for (var i = 0; i < interceptorCount; i++) {
    var interceptor = interceptors[i];
    var before = interceptor.before,
        after = interceptor.after;

    if (!before && !after) {
      mapped.before.push(typeInterceptor(interceptor));
      continue;
    }

    if (before) mapped.before.push(typeInterceptor(before, 'before', mapped.before.length));
    if (after) mapped.after.unshift(typeInterceptor(after, 'after', mapped.after.length));
  }

  return mapped;
}

function normalizeHandlers(handlers, tagName) {
  if (handlers.length) {
    (0, _log["default"])("Please update action handler definition in <".concat(tagName, ">"), {
      tagName: tagName,
      origin: 'normalizeHandlers',
      docs: 'https://developer.servicenow.com/dev.do#!/guide/orlando/now-experience/ui-framework/now-experience-ui-framework-101/action-handlers'
    });
  }

  return handlers.map(function (handler) {
    return typeof handler === 'function' ? {
      effect: handler,
      args: []
    } : _objectSpread(_objectSpread({}, handler), {}, {
      effect: handler.effect || handler.fn,
      args: handler.args || []
    });
  });
}

function normalizeActionHandler(actionHandler, tagName) {
  var _actionHandler$handle = actionHandler.handlers,
      handlers = _actionHandler$handle === void 0 ? [] : _actionHandler$handle,
      _actionHandler$args = actionHandler.args,
      args = _actionHandler$args === void 0 ? [] : _actionHandler$args,
      _actionHandler$interc = actionHandler.interceptors,
      interceptors = _actionHandler$interc === void 0 ? [] : _actionHandler$interc,
      stopPropagation = actionHandler.stopPropagation,
      isPrivate = actionHandler["private"],
      target = actionHandler.target,
      _actionHandler$modifi = actionHandler.modifiers,
      modifiers = _actionHandler$modifi === void 0 ? [] : _actionHandler$modifi,
      _actionHandler$modifi2 = actionHandler.modifier,
      modifier = _actionHandler$modifi2 === void 0 ? {} : _actionHandler$modifi2,
      _actionHandler$captur = actionHandler.capture,
      capture = _actionHandler$captur === void 0 ? false : _actionHandler$captur,
      _actionHandler$eventH = actionHandler.eventHandler,
      eventHandler = _actionHandler$eventH === void 0 ? false : _actionHandler$eventH,
      _actionHandler$descen = actionHandler.descendantOnly,
      descendantOnly = _actionHandler$descen === void 0 ? false : _actionHandler$descen;

  if (typeof actionHandler === 'function') {
    return {
      stopPropagation: stopPropagation,
      effect: actionHandler,
      handlers: [],
      args: args,
      interceptors: mapInterceptors(interceptors),
      target: target,
      modifier: (0, _normalizeModifiers["default"])(modifiers, modifier),
      capture: capture,
      eventHandler: eventHandler,
      descendantOnly: descendantOnly
    };
  }

  function effectDelegate() {
    // Effect delegate function to enable stubbing in tests
    return actionHandler.effect.apply(actionHandler, arguments);
  }

  return {
    stopPropagation: stopPropagation || isPrivate,
    effect: typeof actionHandler.effect === 'function' ? effectDelegate : actionHandler.effect,
    handlers: normalizeHandlers(handlers, tagName),
    args: args,
    interceptors: mapInterceptors(interceptors),
    target: target,
    modifier: (0, _normalizeModifiers["default"])(modifiers, modifier),
    capture: capture,
    eventHandler: eventHandler,
    descendantOnly: descendantOnly
  };
}
//# sourceMappingURL=normalizeActionHandler.js.map