/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.normalizeBehaviors = void 0;

var _normalizeActionHandlers = require("./normalizeActionHandlers");

var _uiInternal = require("@servicenow/ui-internal");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var normalizeBehaviors = (0, _uiInternal.memoize)(normalizeBehaviorsInner, {
  profileName: 'normalizeBehaviors',
  createCacheKey: function createCacheKey(b, t) {
    return t;
  }
});
exports.normalizeBehaviors = normalizeBehaviors;

function normalizeBehaviorsInner() {
  var behaviors = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  var tagName = arguments.length > 1 ? arguments[1] : undefined;
  var behaviorsCount = behaviors.length;
  var normalizedBehaviors = [];

  for (var i = 0; i < behaviorsCount; i++) {
    var _ref = behaviors[i] || {},
        behavior = _ref.behavior,
        _ref$options = _ref.options,
        options = _ref$options === void 0 ? {} : _ref$options;

    var resolvedBehavior = behavior || behaviors[i] || {};
    var name = resolvedBehavior.name;
    var _resolvedBehavior$ini = resolvedBehavior.initialState,
        initialState = _resolvedBehavior$ini === void 0 ? {} : _resolvedBehavior$ini,
        actionHandlers = resolvedBehavior.actionHandlers,
        _resolvedBehavior$eve = resolvedBehavior.eventHandlers,
        eventHandlers = _resolvedBehavior$eve === void 0 ? [] : _resolvedBehavior$eve,
        actions = resolvedBehavior.actions;
    normalizedBehaviors[i] = {
      behavior: _objectSpread(_objectSpread({}, resolvedBehavior), {}, {
        actionHandlers: (0, _normalizeActionHandlers.normalizeActionHandlers)(actionHandlers, tagName),
        initialState: initialState,
        eventHandlers: eventHandlers,
        actions: actions,
        options: options
      }),
      name: name
    };
  }

  return normalizedBehaviors;
}
//# sourceMappingURL=normalizeBehaviors.js.map