/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createStyleFragment;

var _uiInternal = require("@servicenow/ui-internal");

var _constants = require("../constants");

var doc = window.document;
var styleFragments = {};
/**
 * @function
 * @param {String} [style]
 * @param {Array.<String>} [stylesheetsContent]
 * @param {String} [tagName]
 * @returns {HTMLElement}
 */

function createStyleFragment(style, stylesheetsContent, tagName) {
  var cachedStyleFragment = styleFragments[tagName];
  if (cachedStyleFragment) return cachedStyleFragment.content.cloneNode(true);
  var styleFragment;
  var externalStylesFragment = document.head.querySelector("#now-".concat((0, _uiInternal.toLower)(tagName), "-styles"));

  if (externalStylesFragment) {
    styleFragment = mergeStylesheets(externalStylesFragment, stylesheetsContent);
  } else {
    var styles = '';
    styleFragment = doc.createElement('template');
    if (style) styles += style;
    if (stylesheetsContent) styles += stylesheetsContent;
    styleFragment.innerHTML = "<style type=\"text/css\">".concat(styles, "</style>");
  }

  if (tagName !== _constants.SEISMIC_HOIST_TAG_NAME) styleFragments[tagName] = styleFragment;
  return styleFragment.content.cloneNode(true);
}

function mergeStylesheets(externalStylesFragment, stylesheetsContent) {
  if (!stylesheetsContent) return externalStylesFragment; // this case should be rare, if ever; ideally we should deprecate support for config.stylesheets

  var externalStylesEl = externalStylesFragment.content.cloneNode(true).children[0];
  externalStylesEl.textContent = "".concat(stylesheetsContent, "\n").concat(externalStylesEl.textContent);
  var mergedStyleFragment = doc.createElement('template');
  mergedStyleFragment.content.appendChild(externalStylesEl);
  return mergedStyleFragment;
}
//# sourceMappingURL=createStyleFragment.js.map