/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createDispatch;

var _uiMetrics = require("@servicenow/ui-metrics");

var _getStatePropsContext = _interopRequireDefault(require("./getStatePropsContext"));

var _deferHelperExecution = _interopRequireDefault(require("./deferHelperExecution"));

var _constants = require("../../constants");

var _normalizeActionMeta = require("../normalizeActionMeta");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var wrapDispatch = function wrapDispatch(host, shouldWrap) {
  if (shouldWrap === _constants.HELPERS_SCHEDULED) return host.dispatch;
  return function dispatch(type, payload) {
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var error = arguments.length > 3 ? arguments[3] : undefined;
    meta[_constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED] = true; // eslint-disable-line no-param-reassign

    return host.dispatch(type, payload, meta, error);
  };
};

function createDispatch(host, executionTiming) {
  return function dispatch(typeOrFn, payload) {
    var meta = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var error = arguments.length > 3 ? arguments[3] : undefined;
    // eslint-disable-next-line no-param-reassign
    meta = (0, _normalizeActionMeta.normalizeActionMeta)(meta, {
      host: host
    });
    var interactionId = (0, _uiMetrics.getInteractionId)(meta);
    if (executionTiming === _constants.HELPERS_DEFER) (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.DEFER, {
      deferred: _uiMetrics.types.DISPATCH,
      type: typeOrFn
    });
    (0, _deferHelperExecution["default"])(executionTiming, function () {
      var wrappedDispatch = wrapDispatch(host, executionTiming);

      if (typeof typeOrFn === 'function') {
        var statePropsContext = (0, _getStatePropsContext["default"])(host, interactionId);

        var _typeOrFn = typeOrFn(statePropsContext),
            type = _typeOrFn.type,
            _payload = _typeOrFn.payload,
            _typeOrFn$meta = _typeOrFn.meta,
            _meta = _typeOrFn$meta === void 0 ? {} : _typeOrFn$meta,
            _error = _typeOrFn.error,
            _typeOrFn$shouldDispa = _typeOrFn.shouldDispatch,
            shouldDispatch = _typeOrFn$shouldDispa === void 0 ? true : _typeOrFn$shouldDispa;

        if (shouldDispatch) {
          (0, _uiMetrics.applyInteractionId)(_meta);
          wrappedDispatch(type, _payload, _meta, _error);
        }
      } else {
        (0, _uiMetrics.applyInteractionId)(meta);
        wrappedDispatch(typeOrFn, payload, meta, error);
      }
    }, _objectSpread(_objectSpread({}, meta), {}, {
      el: host,
      interactionId: interactionId
    }));
  };
}
//# sourceMappingURL=createDispatch.js.map