/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createUpdateContext;

var _getStatePropsContext = _interopRequireDefault(require("./getStatePropsContext"));

var _uiMetrics = require("@servicenow/ui-metrics");

var _immer = require("immer");

var _utils = require("./utils");

var operations = _interopRequireWildcard(require("../operators/operations"));

var _getHostVariables = _interopRequireDefault(require("../getHostVariables"));

var _scheduler = require("../../scheduler");

var _deferHelperExecution = _interopRequireDefault(require("./deferHelperExecution"));

var _constants = require("../../constants");

var _log = _interopRequireDefault(require("../../log"));

var _renderContextConsumers = _interopRequireDefault(require("../../context/renderContextConsumers"));

var _contexts = require("../../context/contexts");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var IMMEDIATE = _scheduler.types.IMMEDIATE,
    STATE_SET = _scheduler.types.STATE_SET;
var ASSIGN = operations.declarativeOperations.ASSIGN;

var resolveUpdate = function resolveUpdate(host, hostVariables, update, interactionId) {
  if (!(0, _utils.isUpdater)(update)) return update;
  var statePropsContext = (0, _getStatePropsContext["default"])(host, interactionId);
  return update(statePropsContext);
};

function handleContextUpdate(host, hostVariables, update, interactionId) {
  var resolvedUpdate = resolveUpdate(host, hostVariables, update, interactionId);
  if (!resolvedUpdate) return {
    didUpdate: false,
    shouldRender: false
  };
  var contextKey = resolvedUpdate.context;
  var previousContext = (0, _contexts.getContext)(contextKey);
  var _resolvedUpdate$shoul = resolvedUpdate.shouldUpdate,
      shouldUpdate = _resolvedUpdate$shoul === void 0 ? true : _resolvedUpdate$shoul,
      _resolvedUpdate$opera = resolvedUpdate.operation,
      operation = _resolvedUpdate$opera === void 0 ? previousContext !== null && _typeof(previousContext) === 'object' ? ASSIGN : '' : _resolvedUpdate$opera,
      _resolvedUpdate$path = resolvedUpdate.path,
      path = _resolvedUpdate$path === void 0 ? '' : _resolvedUpdate$path,
      start = resolvedUpdate.start,
      deleteCount = resolvedUpdate.deleteCount,
      addItems = resolvedUpdate.addItems,
      _resolvedUpdate$shoul2 = resolvedUpdate.shouldRender,
      shouldRender = _resolvedUpdate$shoul2 === void 0 ? true : _resolvedUpdate$shoul2,
      value = resolvedUpdate.value;
  if (!shouldUpdate) return {
    didUpdate: false,
    shouldRender: false
  };
  var nextContext = (0, _immer.produce)((0, _contexts.getContext)(contextKey) || {}, function (draftState) {
    if (!operation) return value;

    if (!(0, _utils.isValidOperation)(operation)) {
      (0, _log["default"])("Invalid update context operation: ".concat(operation), {
        host: host,
        origin: 'handleContextUpdate'
      });
      return;
    }

    try {
      operations[operation](draftState, {
        path: path,
        value: value,
        start: start,
        deleteCount: deleteCount,
        addItems: Array.isArray(addItems) ? addItems : []
      });
    } catch (error) {
      (0, _log["default"])('Error while performing context update', {
        error: error,
        host: host,
        origin: 'handleContextUpdate'
      });
    }
  });
  if (nextContext === previousContext) return {
    didUpdate: false,
    shouldRender: false
  };
  (0, _contexts.setContext)(contextKey, nextContext);
  return {
    didUpdate: true,
    shouldRender: shouldRender
  };
}

function handleContextUpdateBatch(host, hostVariables, batch, interactionId) {
  return batch.reduce(function (acc, update) {
    var _handleContextUpdate = handleContextUpdate(host, hostVariables, update, interactionId),
        shouldRender = _handleContextUpdate.shouldRender,
        didUpdate = _handleContextUpdate.didUpdate;

    var result = _objectSpread({}, acc);

    if (didUpdate) result.didUpdate = true;
    if (shouldRender) result.shouldRender = true;
    return result;
  }, {
    didUpdate: false,
    shouldRender: false
  });
}

var resolveUpdateForMark = function resolveUpdateForMark(update) {
  return update === 'function' ? 'FUNCTION' : update;
};

var transformUpdateForMark = function transformUpdateForMark(update, isBatch) {
  if (isBatch) return update.map(resolveUpdateForMark);
  return resolveUpdateForMark(update);
};

function resolveInteractionId(interactionId, options) {
  if (interactionId) return interactionId;
  if (typeof options === 'string') return options;
  return (0, _uiMetrics.createInteractionId)();
}

function createUpdateContext(host, executionTiming) {
  var hostVariables = (0, _getHostVariables["default"])(host);
  return function updateContext(update) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var interactionId = arguments.length > 2 ? arguments[2] : undefined;
    var resolvedInteractionId = resolveInteractionId(interactionId, options);
    if (executionTiming === _constants.HELPERS_DEFER) (0, _uiMetrics.mark)(host, resolvedInteractionId, _uiMetrics.types.DEFER, {
      deferred: _uiMetrics.types.UPDATE_CONTEXT
    });
    var meta = {
      el: host,
      interactionId: resolvedInteractionId
    };
    (0, _deferHelperExecution["default"])(executionTiming, function () {
      // TODO: Remove this immediate option for a better scheduling alternative
      // https://code.devsnc.com/gist/dylan-mozlowski/07bf0ab13b2c8a12f0cc6e59bf4cf911
      var _options$SCHEDULE_IMM = options[_constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED],
          optionallyImmediate = _options$SCHEDULE_IMM === void 0 ? false : _options$SCHEDULE_IMM;
      (0, _scheduler.schedule)({
        fn: function fn() {
          var shouldBatch = (0, _utils.isBatch)(update);
          (0, _uiMetrics.mark)(host, resolvedInteractionId, _uiMetrics.types.UPDATE_CONTEXT, {
            update: transformUpdateForMark(update, shouldBatch)
          });
          var updateHandler = shouldBatch ? handleContextUpdateBatch : handleContextUpdate;

          var _updateHandler = updateHandler(host, hostVariables, update, resolvedInteractionId),
              didUpdate = _updateHandler.didUpdate,
              shouldRender = _updateHandler.shouldRender;

          if (!didUpdate) return;

          if (shouldRender) {
            var contexts = shouldBatch ? update.map(function (_ref) {
              var context = _ref.context;
              return context;
            }) : [update.context];

            for (var i = 0; i < contexts.length; i++) {
              (0, _renderContextConsumers["default"])(contexts[i], resolvedInteractionId, {
                update: transformUpdateForMark(update, shouldBatch)
              });
            }
          }
        },
        meta: meta,
        type: executionTiming !== _constants.HELPERS_SCHEDULED || optionallyImmediate ? IMMEDIATE : STATE_SET
      });
    }, meta);
  };
}
//# sourceMappingURL=createUpdateContext.js.map