/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createUpdateProperties;

var _uiMetrics = require("@servicenow/ui-metrics");

var _log = _interopRequireDefault(require("../../log"));

var operations = _interopRequireWildcard(require("../operators/operations"));

var _uiInternal = require("@servicenow/ui-internal");

var _scheduler = require("../../scheduler");

var _parse = require("../parse");

var _getHostVariables = _interopRequireDefault(require("../getHostVariables"));

var _getStatePropsContext = _interopRequireDefault(require("./getStatePropsContext"));

var _deferHelperExecution = _interopRequireDefault(require("./deferHelperExecution"));

var _constants = require("../../constants");

var _symbols = require("../../symbols");

var _quickEquals = _interopRequireDefault(require("./quickEquals"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var IMMEDIATE = _scheduler.types.IMMEDIATE,
    PROPERTY_SET = _scheduler.types.PROPERTY_SET;
var ASSIGN = operations.declarativeOperations.ASSIGN;

var setProperties = function setProperties(host, properties, propertyDefs, interactionId) {
  var hostVariables = (0, _getHostVariables["default"])(host);
  var dispatch = hostVariables[_symbols.HELPERS_SCHEDULED].dispatch;
  var parsedProperties = (0, _parse.properties)(properties, propertyDefs, {
    host: host,
    dispatch: dispatch
  });

  for (var k in parsedProperties) {
    // for backwards compatibility an undefined must still be set on the instance
    if (!propertyDefs.hasOwnProperty(k)) {
      host[k] = parsedProperties[k]; // eslint-disable-line no-param-reassign

      if (process.env.NODE_ENV === 'development') {
        (0, _log["default"])("".concat(host.tagName.toLowerCase(), " is missing a property definition for \"").concat(k, "\".  Properties set with missing definitions will not be available to the view or action handlers."), {
          host: host,
          origin: 'setProperties'
        });
      }
    } else {
      // FIXME: Not handling cases where component starts with no old props
      var old = hostVariables[_symbols.PROPS][k];
      var current = parsedProperties[k];

      if ((0, _quickEquals["default"])(old, current)) {
        continue;
      }

      host[_symbols.SET_PROPERTY](k, parsedProperties[k], true, interactionId); // eslint-disable-line no-param-reassign

    }
  }
};

function createUpdateProperties(host, propertyDefs, executionTiming) {
  return function updateProperties(update, interactionId) {
    if (executionTiming === _constants.HELPERS_DEFER) (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.DEFER, {
      deferred: _uiMetrics.types.UPDATE_PROPERTIES
    });
    var meta = {
      el: host,
      interactionId: interactionId
    };
    (0, _deferHelperExecution["default"])(executionTiming, function () {
      (0, _scheduler.schedule)({
        fn: function fn() {
          (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.UPDATE_PROPERTIES, {
            update: typeof update === 'function' ? 'DECLARATIVE' : 'OBJECT'
          });

          if (typeof update === 'function') {
            var stateProps = (0, _getStatePropsContext["default"])(host, interactionId);

            var _update = update(stateProps),
                _update$shouldUpdate = _update.shouldUpdate,
                shouldUpdate = _update$shouldUpdate === void 0 ? true : _update$shouldUpdate,
                _update$operation = _update.operation,
                operation = _update$operation === void 0 ? ASSIGN : _update$operation,
                path = _update.path,
                value = _update.value,
                start = _update.start,
                deleteCount = _update.deleteCount,
                addItems = _update.addItems;

            if (shouldUpdate) {
              try {
                var propertyPath = (0, _uiInternal.splitPath)(path);
                var propertyLen = propertyPath.length;

                var _propertyPath = _slicedToArray(propertyPath, 1),
                    name = _propertyPath[0];

                var propertyValue = host[name];

                if (propertyValue === null || _typeof(propertyValue) !== 'object') {
                  setProperties(host, _defineProperty({}, name, value), propertyDefs, interactionId);
                } else {
                  var _propertyValue = {};
                  var pvRef = _propertyValue;
                  var hostRef = host;

                  for (var i = 0; i < propertyLen; i++) {
                    var _name = propertyPath[i];
                    var _value = hostRef[_name];
                    if (_value === null || _typeof(_value) !== 'object') break;
                    if (Array.isArray(_value)) pvRef[_name] = _value.slice();else pvRef[_name] = _objectSpread({}, _value);
                    pvRef = pvRef[_name];
                    hostRef = hostRef[_name];
                  }

                  operations[operation](_propertyValue, {
                    path: propertyLen ? propertyPath : [],
                    value: value,
                    start: start,
                    deleteCount: deleteCount,
                    addItems: Array.isArray(addItems) ? addItems : []
                  });
                  setProperties(host, _propertyValue, propertyDefs, interactionId);
                }
              } catch (error) {
                (0, _log["default"])('Error while performing properties update', {
                  error: error,
                  host: host,
                  origin: 'updateProperties'
                });
              }
            }
          } else {
            setProperties(host, update, propertyDefs, interactionId);
          }
        },
        type: executionTiming !== _constants.HELPERS_SCHEDULED ? IMMEDIATE : PROPERTY_SET,
        meta: meta
      });
    }, meta);
  };
}
//# sourceMappingURL=createUpdateProperties.js.map