/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createUpdateState;

var _uiMetrics = require("@servicenow/ui-metrics");

var _immer = require("immer");

var _utils = require("./utils");

var operations = _interopRequireWildcard(require("../operators/operations"));

var _getHostVariables = _interopRequireDefault(require("../getHostVariables"));

var _scheduler = require("../../scheduler");

var _deferHelperExecution = _interopRequireDefault(require("./deferHelperExecution"));

var _symbols = require("../../symbols");

var _constants = require("../../constants");

var _log = _interopRequireDefault(require("../../log"));

var _ = require("../");

var _uiInternal = require("@servicenow/ui-internal");

var _uiConfig = require("@servicenow/ui-config");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || _typeof(obj) !== "object" && typeof obj !== "function") { return { "default": obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj["default"] = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var ENABLE_DECLARATIVE_STATE_UPDATE_DEEP_COMPARE = _uiConfig.configConstants.ENABLE_DECLARATIVE_STATE_UPDATE_DEEP_COMPARE;
var optOutDeepCompare = (0, _.shouldOptOutOfDeepCompareForPage)('state');
var shouldDeepCompareValues = (0, _uiConfig.getConfigItem)(ENABLE_DECLARATIVE_STATE_UPDATE_DEEP_COMPARE);
var IMMEDIATE = _scheduler.types.IMMEDIATE,
    STATE_SET = _scheduler.types.STATE_SET;
var ASSIGN = operations.declarativeOperations.ASSIGN;
window.addEventListener('popstate', function () {
  return optOutDeepCompare = (0, _.shouldOptOutOfDeepCompareForPage)('state');
});

function handleStateUpdate(host, hostVariables, update, interactionId) {
  var resolvedUpdate = (0, _utils.resolveUpdate)(host, hostVariables, update, interactionId);
  var _resolvedUpdate$shoul = resolvedUpdate.shouldUpdate,
      shouldUpdate = _resolvedUpdate$shoul === void 0 ? true : _resolvedUpdate$shoul,
      _resolvedUpdate$opera = resolvedUpdate.operation,
      operation = _resolvedUpdate$opera === void 0 ? ASSIGN : _resolvedUpdate$opera,
      _resolvedUpdate$path = resolvedUpdate.path,
      path = _resolvedUpdate$path === void 0 ? '' : _resolvedUpdate$path,
      start = resolvedUpdate.start,
      deleteCount = resolvedUpdate.deleteCount,
      addItems = resolvedUpdate.addItems,
      _resolvedUpdate$shoul2 = resolvedUpdate.shouldRender,
      shouldRender = _resolvedUpdate$shoul2 === void 0 ? true : _resolvedUpdate$shoul2,
      value = resolvedUpdate.value;
  if (!shouldUpdate || !(0, _utils.shouldUpdateState)(hostVariables[_symbols.STATE], resolvedUpdate)) return {
    didUpdate: false,
    shouldRender: false
  }; // this an extremly naive implementation. it doesn't account
  // for operations like push or pop that you'd want to perform
  // before diffing the values

  if (shouldDeepCompareValues && !optOutDeepCompare && path && operation && value) {
    var nextValue = value;
    var currentValue = (0, _uiInternal.get)(hostVariables[_symbols.STATE], path);
    if ((0, _uiInternal.isSelectiveEqual)(currentValue, nextValue)) return {
      didUpdate: false,
      shouldRender: false
    };
  }

  var previousState = hostVariables[_symbols.STATE];
  var props = hostVariables[_symbols.PROPS];
  var avoidRender = props && props[_constants.NOW_AVOID_RENDER_PROP];

  if (avoidRender && (0, _utils.isValidOptimisationOperation)(operation)) {
    try {
      var target = (0, _utils.isDeclarativeUpdate)(resolvedUpdate) ? value : resolvedUpdate;
      var source = path ? (0, _uiInternal.get)(previousState, path) : previousState;
      if ((0, _uiInternal.isSelectiveEqual)(source, target)) return {
        didUpdate: false,
        shouldRender: false
      };
    } catch (error) {
      (0, _log["default"])('Error while performing same state re-render optimisation pre-validation', {
        error: error,
        host: host,
        origin: 'handleStateUpdate'
      });
    }
  }

  var nextState = (0, _immer.produce)(hostVariables[_symbols.STATE] || {}, function (draftState) {
    if (operation === 'replace') return value;

    if (!(0, _utils.isValidOperation)(operation)) {
      (0, _log["default"])("Invalid update state operation: ".concat(operation), {
        host: host,
        origin: 'handleStateUpdate'
      });
      return;
    }

    try {
      operations[operation](draftState, {
        path: path,
        value: (0, _utils.isDeclarativeUpdate)(resolvedUpdate) ? value : resolvedUpdate,
        start: start,
        deleteCount: deleteCount,
        addItems: Array.isArray(addItems) ? addItems : []
      });
    } catch (error) {
      (0, _log["default"])('Error while performing state update', {
        error: error,
        host: host,
        origin: 'handleStateUpdate'
      });
    }
  });
  if (nextState === previousState) return {
    didUpdate: false,
    shouldRender: false
  };
  hostVariables[_symbols.STATE] = nextState; // eslint-disable-line no-param-reassign

  return {
    didUpdate: true,
    shouldRender: shouldRender
  };
}

function handleStateUpdateBatch(host, hostVariables, batch, interactionId) {
  return batch.reduce(function (acc, update) {
    var _handleStateUpdate = handleStateUpdate(host, hostVariables, update, interactionId),
        shouldRender = _handleStateUpdate.shouldRender,
        didUpdate = _handleStateUpdate.didUpdate;

    var result = _objectSpread({}, acc);

    if (didUpdate) result.didUpdate = true;
    if (shouldRender) result.shouldRender = true;
    return result;
  }, {
    didUpdate: false,
    shouldRender: false
  });
}

var resolveUpdateForMark = function resolveUpdateForMark(update) {
  return typeof update === 'function' ? 'FUNCTION' : update;
};

var transformUpdateForMark = function transformUpdateForMark(update, isBatch) {
  if (isBatch) return update.map(resolveUpdateForMark);
  return resolveUpdateForMark(update);
};

function resolveInteractionId(interactionId, options) {
  if (interactionId) return interactionId;
  if (typeof options === 'string') return options;
  return (0, _uiMetrics.createInteractionId)();
}

function createUpdateState(host, executionTiming) {
  var hostVariables = (0, _getHostVariables["default"])(host);
  return function updateState(update) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var interactionId = arguments.length > 2 ? arguments[2] : undefined;
    var resolvedInteractionId = resolveInteractionId(interactionId, options);
    if (executionTiming === _constants.HELPERS_DEFER) (0, _uiMetrics.mark)(host, resolvedInteractionId, _uiMetrics.types.DEFER, {
      deferred: _uiMetrics.types.UPDATE_STATE
    });
    var meta = {
      el: host,
      interactionId: resolvedInteractionId
    };
    (0, _deferHelperExecution["default"])(executionTiming, function () {
      // TODO: Remove this immediate option for a better scheduling alternative
      // https://code.devsnc.com/gist/dylan-mozlowski/07bf0ab13b2c8a12f0cc6e59bf4cf911
      var _options$SCHEDULE_IMM = options[_constants.SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED],
          optionallyImmediate = _options$SCHEDULE_IMM === void 0 ? false : _options$SCHEDULE_IMM;
      (0, _scheduler.schedule)({
        fn: function fn() {
          var resolvedUpdate = (0, _utils.resolveUpdate)(host, hostVariables, update, resolveInteractionId);
          var shouldBatch = (0, _utils.isBatch)(resolvedUpdate);
          (0, _uiMetrics.mark)(host, resolvedInteractionId, _uiMetrics.types.UPDATE_STATE, {
            update: transformUpdateForMark(resolvedUpdate, shouldBatch)
          });
          var previousState = hostVariables[_symbols.STATE];
          var updateHandler = shouldBatch ? handleStateUpdateBatch : handleStateUpdate;

          var _updateHandler = updateHandler(host, hostVariables, resolvedUpdate, resolvedInteractionId),
              didUpdate = _updateHandler.didUpdate,
              shouldRender = _updateHandler.shouldRender;

          if (!didUpdate) return;
          (0, _.emitNowUiEvent)(_constants.STATE_UPDATED, {
            componentId: hostVariables[_symbols.COMPONENT_ID],
            previousState: previousState,
            nextState: hostVariables[_symbols.STATE]
          });

          if (shouldRender) {
            host[_symbols.RENDER](resolvedInteractionId, _uiMetrics.types.UPDATE_STATE, {
              update: transformUpdateForMark(resolvedUpdate, shouldBatch)
            });
          }

          hostVariables[_symbols.CONDITIONAL_DISPATCH](_constants.COMPONENT_STATE_UPDATED, {
            previousState: previousState,
            update: update
          }, {
            internal: true
          });
        },
        meta: meta,
        type: executionTiming !== _constants.HELPERS_SCHEDULED || optionallyImmediate ? IMMEDIATE : STATE_SET
      });
    }, meta);
  };
}
//# sourceMappingURL=createUpdateState.js.map