/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = deferHelperExecution;

var _constants = require("../../constants");

var _scheduler = require("../../scheduler");

var DEFER_HELPER_EXECUTION = _scheduler.types.DEFER_HELPER_EXECUTION; // the origin event handler for an action is ran as a microtask because it has to wait to see if
// preventDefault has been called by an ancestor. because of this back-to-back dispatches
// triggered from a user event such as a click can cause stale state or properties in helper
// callbacks (or just event handlers) in the view. so far this is only replicable programmtically
// doing something like btn.click(); btn.click(); where these clicks dispatch an action that then updates
// state or properties in an effect hander and the dispatch callback in the view relies on state
// or properties. eventually this use case needs to be accounted for in the scheduler design.

function deferHelperExecution(executionTiming, fn, meta) {
  return executionTiming === _constants.HELPERS_DEFER ? (0, _scheduler.schedule)({
    fn: fn,
    type: DEFER_HELPER_EXECUTION,
    meta: meta
  }) : fn();
}
//# sourceMappingURL=deferHelperExecution.js.map