/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = createHelpers;

var _log = _interopRequireDefault(require("../../log"));

var _createUpdateState = _interopRequireDefault(require("./createUpdateState"));

var _createUpdateContext = _interopRequireDefault(require("./createUpdateContext"));

var _createUpdateProperties = _interopRequireDefault(require("./createUpdateProperties"));

var _createDispatch = _interopRequireDefault(require("./createDispatch"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function createHelpers(host, propertyDefs) {
  var executionTiming = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : null;
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var wrappedDispatch = (0, _createDispatch["default"])(host, executionTiming);

  var dispatch = function dispatch() {
    (0, _log["default"])("Please update the view for ".concat(host.tagName.toLowerCase(), " from using dispatch as the second argument to an object with dispatch, updateState, and updateProperties"), {
      host: host,
      origin: 'createHelpers'
    });
    wrappedDispatch.apply(void 0, arguments);
  };

  if (options.presentational) return {
    dispatch: wrappedDispatch
  };
  dispatch.dispatch = wrappedDispatch;
  dispatch.updateState = (0, _createUpdateState["default"])(host, executionTiming);
  dispatch.updateProperties = (0, _createUpdateProperties["default"])(host, propertyDefs, executionTiming);
  dispatch.updateContext = (0, _createUpdateContext["default"])(host, executionTiming);
  return dispatch;
}
//# sourceMappingURL=index.js.map