/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isDeclarativeUpdate = isDeclarativeUpdate;
exports.isValidOperation = isValidOperation;
exports.isBatch = isBatch;
exports.isUpdater = isUpdater;
exports.resolveUpdate = resolveUpdate;
exports.isValidOptimisationOperation = isValidOptimisationOperation;
exports.shouldUpdateState = shouldUpdateState;

var operations = _interopRequireWildcard(require("../operators/operations"));

var _getStatePropsContext = _interopRequireDefault(require("./getStatePropsContext"));

var _uiInternal = require("@servicenow/ui-internal");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var _operations$declarati = operations.declarativeOperations,
    SET = _operations$declarati.SET,
    ASSIGN = _operations$declarati.ASSIGN,
    MERGE = _operations$declarati.MERGE;

var isEmptyStateUpdate = function isEmptyStateUpdate(value) {
  return (// (value instanceof Map && value.size === 0) ||
    (0, _uiInternal.isPlainObject)(value) && Object.keys(value).length === 0
  );
};

function isPrimitive(value) {
  var type = _typeof(value);

  if (type === 'object' || type === 'function') return false;
  if (type === 'string') return true;
  if (type === 'bigint') return true;
  if (type === 'boolean') return true;
  if (type === 'number') return true;
  return false;
}

var isPrimitiveOrNull = function isPrimitiveOrNull(value) {
  return isPrimitive(value) || value === null;
};

function isDeclarativeUpdate() {
  var update = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var operation = update.operation,
      path = update.path,
      value = update.value;
  return operation && (path || value);
}

function isValidOperation(operation) {
  return operations[operation] !== undefined;
}

function isBatch(update) {
  return Array.isArray(update) && update.every(function (u) {
    return isDeclarativeUpdate(u) || isUpdater(u);
  });
}

function isUpdater(update) {
  return typeof update === 'function';
}

function resolveUpdate(host, hostVariables, update, interactionId) {
  if (!isUpdater(update)) return update;
  var statePropsContext = (0, _getStatePropsContext["default"])(host, interactionId);
  return update(statePropsContext) || statePropsContext;
}

function isValidOptimisationOperation(operation) {
  return operation && [SET, ASSIGN, MERGE].includes(operation);
}

function shouldUpdateState(currentState, update) {
  var declarativeUpdate = !!isDeclarativeUpdate(update);
  var _update$path = update.path,
      path = _update$path === void 0 ? '' : _update$path,
      value = update.value,
      operation = update.operation;
  if (declarativeUpdate && operation !== SET && operation !== ASSIGN && operation !== MERGE) return true;
  var updateKeys = declarativeUpdate ? null : Object.keys(update);

  if (updateKeys === null || updateKeys.length === 1) {
    var currentValue = declarativeUpdate ? (0, _uiInternal.get)(currentState, path) : currentState[updateKeys[0]];
    var nextValue = declarativeUpdate ? value : update[updateKeys[0]];

    if (isPrimitiveOrNull(nextValue) && isPrimitiveOrNull(currentValue) && nextValue === currentValue) {
      return false;
    }

    if (isEmptyStateUpdate(nextValue) && isEmptyStateUpdate(currentValue)) {
      return false;
    }

    if ((0, _uiInternal.isPlainObject)(currentValue) && (0, _uiInternal.isPlainObject)(nextValue)) {
      var nextValueKeys = Object.keys(nextValue);
      var currentValueKeys = Object.keys(currentValue);

      if (nextValueKeys.length <= 2 && nextValueKeys.length === currentValueKeys.length) {
        var currentValue1 = currentValue[nextValueKeys[0]];
        var currentValue2 = currentValue[nextValueKeys[1]];
        var nextValue1 = nextValue[nextValueKeys[0]];
        var nextValue2 = nextValue[nextValueKeys[1]];
        if (declarativeUpdate && operation === SET && (nextValueKeys[0] !== currentValueKeys[0] || nextValueKeys[1] !== currentValueKeys[1])) return true;

        if (isPrimitiveOrNull(nextValue1) && isPrimitiveOrNull(currentValue1) && nextValue1 === currentValue1 && isPrimitiveOrNull(nextValue2) && isPrimitiveOrNull(currentValue2) && nextValue2 === currentValue2) {
          return false;
        }
      }
    }
  }

  return true;
}
//# sourceMappingURL=utils.js.map