/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = markComponentInteractive;

var _uiInternal = require("@servicenow/ui-internal");

var _uiMetrics = require("@servicenow/ui-metrics");

var _propertySelectors = require("../property-selectors");

var _constants = require("../constants");

var _symbols = require("../symbols");

var _getHostVariables = _interopRequireDefault(require("./getHostVariables"));

var _scheduler = require("../scheduler");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var isProvider = (0, _uiInternal.memoize)(function (el) {
  var variables = (0, _getHostVariables["default"])(el);
  return variables[_symbols.SELECTABLE_PROPERTIES] && variables[_symbols.SELECTABLE_PROPERTIES].length;
}, {
  createCacheKey: function createCacheKey(el) {
    return el.tagName;
  },
  profileName: 'isProvider'
});

var areProvidersInteractive = function areProvidersInteractive(el, componentRegistry) {
  var variables = (0, _getHostVariables["default"])(el);
  var properties = variables[_symbols.PROPS];

  for (var k in properties) {
    var value = properties[k];

    var _getSelectorParts = (0, _propertySelectors.getSelectorParts)(value),
        _getSelectorParts2 = _slicedToArray(_getSelectorParts, 1),
        id = _getSelectorParts2[0];

    if (id) {
      var providerEl = componentRegistry[id];

      if (providerEl) {
        var providerVariables = (0, _getHostVariables["default"])(providerEl);
        if (providerVariables && !providerVariables[_symbols.INTERACTIVE_TIMESTAMP]) return false;
      }
    }
  }

  return true;
};

var notifySubscribers = function notifySubscribers(el, componentRegistry) {
  var variables = (0, _getHostVariables["default"])(el);
  var selectabledProperties = variables[_symbols.SELECTABLE_PROPERTIES];
  var selectabledPropertiesLength = selectabledProperties && selectabledProperties.length || 0;

  for (var i = 0; i < selectabledPropertiesLength; i++) {
    var propertyName = selectabledProperties[i];
    var affectedComponents = (0, _propertySelectors.getAffected)(propertyName, variables[_symbols.COMPONENT_ID], componentRegistry, false);

    for (var id in affectedComponents) {
      var _id$split = id.split(_constants.PROPERTY_SELECTOR_SIGIL),
          _id$split2 = _slicedToArray(_id$split, 1),
          componentId = _id$split2[0];

      var subscriberEl = componentRegistry[componentId];
      if (subscriberEl) markComponentInteractive(subscriberEl, componentRegistry);
    }
  }
};

function markComponentInteractive(el, componentRegistry) {
  var variables = (0, _getHostVariables["default"])(el);
  if (!variables || variables[_symbols.INTERACTIVE_TIMESTAMP]) return;
  if ((0, _scheduler.countHighPriorityTasks)(el)) return false;
  if ((0, _scheduler.countMediumPriorityTasks)(el)) return false;
  if ((0, _uiInternal.pendingNetworkActionCountForHost)(el)) return false;
  var childrenToBeInteractive = variables[_symbols.CHILDREN_TO_BE_INTERACTIVE];
  if (!childrenToBeInteractive) return false;

  for (var k in childrenToBeInteractive) {
    return false;
  }

  var providersInteractive = areProvidersInteractive(el, componentRegistry);
  if (!providersInteractive) return false;
  var interactiveTimestamp = performance.now();
  variables[_symbols.INTERACTIVE_TIMESTAMP] = interactiveTimestamp;
  (0, _uiMetrics.mark)(el, (0, _uiMetrics.createInteractionId)(), _uiMetrics.types.INTERACTIVE, {});

  variables[_symbols.CONDITIONAL_DISPATCH](_constants.COMPONENT_INTERACTIVE, {
    host: el,
    interactiveTimestamp: interactiveTimestamp
  }, {
    internal: true
  });

  var parentEl = variables[_symbols.ANCESTOR_ROOT_NODE];

  if (parentEl) {
    var parentVariables = (0, _getHostVariables["default"])(parentEl);
    delete parentVariables[_symbols.CHILDREN_TO_BE_INTERACTIVE][variables[_symbols.COMPONENT_ID]];
    markComponentInteractive(parentEl, componentRegistry);
  }

  if (isProvider(el)) notifySubscribers(el, componentRegistry);
}
//# sourceMappingURL=markComponentInteractive.js.map