/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = measure;

var _uiMetrics = require("@servicenow/ui-metrics");

var performanceApiExists = performance && typeof performance.mark === 'function' && typeof performance.measure === 'function';
var _window = window,
    __ENABLE_NOW_UI_PERFORMANCE__ = _window.__ENABLE_NOW_UI_PERFORMANCE__;
var uiPerfEnabled = process.env.NODE_ENV === 'performance' || process.env.NODE_ENV === 'test' || __ENABLE_NOW_UI_PERFORMANCE__;

try {
  delete window.__ENABLE_NOW_UI_PERFORMANCE__;
  Object.defineProperty(window, '__ENABLE_NOW_UI_PERFORMANCE__', {
    configurable: true,
    get: function get() {
      return __ENABLE_NOW_UI_PERFORMANCE__;
    },
    set: function set(value) {
      __ENABLE_NOW_UI_PERFORMANCE__ = value;
      uiPerfEnabled = process.env.NODE_ENV === 'performance' || process.env.NODE_ENV === 'test' || value;
    }
  });
} catch (e) {// fail silently
}

function measure(fn, host, marker, interactionId, options) {
  if (!performanceApiExists) return fn();
  (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.RENDER_START, options);
  var startMarker;

  if (uiPerfEnabled) {
    startMarker = "".concat(marker, "-start");
    performance.mark(startMarker);
  }

  fn();
  (0, _uiMetrics.mark)(host, interactionId, _uiMetrics.types.RENDER_END, options);
  if (uiPerfEnabled) performance.measure(marker, startMarker);
}
//# sourceMappingURL=measure.js.map