/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.declarativeOperations = exports.concat = exports.splice = exports.unshift = exports.shift = exports.push = exports.pop = exports.merge = exports.assign = exports.set = void 0;

var _uiInternal = require("@servicenow/ui-internal");

var _assign2 = require("./assign");

var _merge2 = require("./merge");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function operateAtPath(operationFn, obj, path, value) {
  if (!path) return operationFn(obj, value);
  var location = (0, _uiInternal.get)(obj, path);
  if (location === undefined || _typeof(location) !== 'object') return (0, _uiInternal.set)(obj, path, value);
  return operationFn(location, value);
} // Object


var set = function set(obj, _ref) {
  var path = _ref.path,
      value = _ref.value;
  return (0, _uiInternal.set)(obj, path, value);
};

exports.set = set;

var assign = function assign(obj, _ref2) {
  var path = _ref2.path,
      value = _ref2.value;
  return operateAtPath(_assign2.assign, obj, path, value);
};

exports.assign = assign;

var merge = function merge(obj, _ref3) {
  var path = _ref3.path,
      value = _ref3.value;
  return operateAtPath(_merge2.merge, obj, path, value);
}; // Array


exports.merge = merge;

var pop = function pop(obj, _ref4) {
  var path = _ref4.path;
  return (0, _uiInternal.get)(obj, path).pop();
};

exports.pop = pop;

var push = function push(obj, _ref5) {
  var path = _ref5.path,
      value = _ref5.value;
  return (0, _uiInternal.get)(obj, path).push(value);
};

exports.push = push;

var shift = function shift(obj, _ref6) {
  var path = _ref6.path;
  return (0, _uiInternal.get)(obj, path).shift();
};

exports.shift = shift;

var unshift = function unshift(obj, _ref7) {
  var path = _ref7.path,
      value = _ref7.value;
  return (0, _uiInternal.get)(obj, path).unshift(value);
};

exports.unshift = unshift;

var splice = function splice(obj, _ref8) {
  var _get2;

  var path = _ref8.path,
      _ref8$start = _ref8.start,
      start = _ref8$start === void 0 ? 0 : _ref8$start,
      _ref8$deleteCount = _ref8.deleteCount,
      deleteCount = _ref8$deleteCount === void 0 ? 0 : _ref8$deleteCount,
      _ref8$addItems = _ref8.addItems,
      addItems = _ref8$addItems === void 0 ? [] : _ref8$addItems;
  return (_get2 = (0, _uiInternal.get)(obj, path)).splice.apply(_get2, [start, deleteCount].concat(_toConsumableArray(addItems)));
};

exports.splice = splice;

var concat = function concat(obj, _ref9) {
  var path = _ref9.path,
      value = _ref9.value;
  var arr = (0, _uiInternal.get)(obj, path);
  return arr.push.apply(arr, value); // eslint-disable-line prefer-spread
};

exports.concat = concat;
var declarativeOperations = {
  SET: 'set',
  ASSIGN: 'assign',
  MERGE: 'merge',
  POP: 'pop',
  PUSH: 'push',
  SHIFT: 'shift',
  UNSHIFT: 'unshift',
  SPLICE: 'splice',
  CONCAT: 'concat'
}; // TODO: Map / Set / Date

exports.declarativeOperations = declarativeOperations;
//# sourceMappingURL=operations.js.map