/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = parseProperty;

var _constants = require("../../constants");

var _isValuePropertySelector = _interopRequireDefault(require("./isValuePropertySelector"));

var _uiInternal = require("@servicenow/ui-internal");

var _log = _interopRequireDefault(require("../../log"));

var _refProperty = require("./refProperty");

var _types = _interopRequireDefault(require("./types"));

var _parse = _interopRequireDefault(require("./parse"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function parseProperty(name, value) {
  var def = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var host = arguments.length > 3 ? arguments[3] : undefined;
  var dispatch = arguments.length > 4 ? arguments[4] : undefined;
  var unstableParse = def[_constants.PARSE_PROP_CONF_KEY];
  var schemaType = def.schema && def.schema.type;

  var isValidParseType = _types["default"].includes(schemaType);

  var resolvedValue = def.resolveRef ? (0, _refProperty.parseRefProperty)(host, name, value) : value;
  var shouldUnstableParse = (unstableParse || _types["default"].includes(schemaType)) && !(0, _isValuePropertySelector["default"])(resolvedValue) && typeof resolvedValue === 'string';

  if (shouldUnstableParse && isValidParseType && !host.hasAttribute((0, _uiInternal.kebabCase)(name))) {
    var component = "<".concat((0, _uiInternal.toLower)(host.tagName), " />");
    (0, _log["default"])("For property ".concat(name, " on component ").concat(component, ": expected to be type ").concat(schemaType, " but received string. ").concat(_constants.PACKAGE_NAME, " will attempt to parse this value, but you should set this to the correct type."), {
      level: _uiInternal.allLogTypes.WARN,
      host: host,
      origin: 'parseProperty'
    });
  }

  return shouldUnstableParse ? (0, _uiInternal.sandbox)(_parse["default"], {
    args: [resolvedValue, schemaType, name, host.tagName],
    host: host,
    dispatch: dispatch,
    location: _uiInternal.locations.PROPERTY,
    details: {
      parse: true,
      name: name
    },
    log: _log["default"]
  }) : resolvedValue;
}
//# sourceMappingURL=property.js.map