/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseRefProperty = parseRefProperty;
exports.getUnsubscribe = getUnsubscribe;
exports.getHostUnsubs = getHostUnsubs;

var _uiInternal = require("@servicenow/ui-internal");

var refUnsubs = new Map();

function parseRefProperty(host, name, value) {
  var _unsubs, _unsubs$get;

  var isReference = (0, _uiInternal.isRef)(value);
  var unsubs = refUnsubs.get(host);
  var ref = (_unsubs = unsubs) === null || _unsubs === void 0 ? void 0 : (_unsubs$get = _unsubs.get(name)) === null || _unsubs$get === void 0 ? void 0 : _unsubs$get.ref;

  if (ref && (ref.current !== value || isReference)) {
    if (unsubs) {
      var _unsubs$get2;

      (_unsubs$get2 = unsubs.get(name)) === null || _unsubs$get2 === void 0 ? void 0 : _unsubs$get2.fn();
      unsubs["delete"](name);

      if (unsubs.size === 0) {
        refUnsubs["delete"](host);
        unsubs = null;
      }
    }
  }

  if (isReference) {
    refUnsubs.set(host, (unsubs || new Map()).set(name, {
      fn: (0, _uiInternal.subscribeToRef)(value, host, name),
      ref: value
    }));
    return value.current;
  }

  return value;
}

function getUnsubscribe(host, propName) {
  var _refUnsubs$get;

  return (_refUnsubs$get = refUnsubs.get(host)) === null || _refUnsubs$get === void 0 ? void 0 : _refUnsubs$get.get(propName);
}

function getHostUnsubs(host) {
  return refUnsubs.get(host);
}
//# sourceMappingURL=refProperty.js.map