/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = resolveComputedProperties;

var _symbols = require("../symbols");

var _filterComputedProperties = _interopRequireDefault(require("./filterComputedProperties"));

var _computeProperty = _interopRequireDefault(require("./computeProperty"));

var _quickEquals = _interopRequireDefault(require("./helpers/quickEquals"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

// TODO: account for p2.computed relying on p1.computed; use getters???
function resolveComputedProperties(state, host, hostVariables, interactionId) {
  var computed = (0, _filterComputedProperties["default"])(host);
  state.properties = state.properties || {};

  for (var k in computed) {
    var previousValue = hostVariables[_symbols.PROPS][k];
    state.properties[k] = (0, _computeProperty["default"])(k, computed[k], {
      state: state,
      host: host
    });
    hostVariables[_symbols.PROPS][k] = state.properties[k]; // eslint-disable-line no-param-reassign

    if (!(0, _quickEquals["default"])(state.properties[k], previousValue)) {
      host[_symbols.UPDATE_AFFECTED_COMPONENTS]({
        name: k,
        propertyDetails: {
          previousValue: previousValue,
          value: state.properties[k]
        },
        interactionId: interactionId,
        includeSource: false
      });
    }
  }

  return state;
}
//# sourceMappingURL=resolveComputedProperties.js.map