/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = resolveProperties;

var shouldReturnEmptyString = function shouldReturnEmptyString() {
  var schema = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return schema.type === 'string' || Array.isArray(schema.type) && schema.type.indexOf('string') !== -1 || schema.oneOf && schema.oneOf.find(function (_ref) {
    var type = _ref.type;
    return type === 'string';
  });
};
/**
 * @private
 * @param {*} value
 * @param {*} defaultValue
 * @returns resolved safe string value
 */


var fetchSafeStringValue = function fetchSafeStringValue(value, defaultValue) {
  if (value || defaultValue === undefined) return value;
  return defaultValue;
};
/**
 * @private
 * @param {*} propertyDefinition
 * @param {*} values
 * @param {*} defaultValue
 * @returns resolved value
 **/


var resolveValue = function resolveValue(propertyDefinition, values, defaultValue) {
  var valueCount = values.length;
  var hasSchema = propertyDefinition === null || propertyDefinition === void 0 ? void 0 : propertyDefinition.schema;

  for (var i = 0; i < valueCount; i++) {
    var value = values[i];

    if (typeof value === 'string') {
      if (!hasSchema) return fetchSafeStringValue(value, defaultValue);else if (value.length || propertyDefinition.schema && shouldReturnEmptyString(propertyDefinition.schema)) return value;
    } else {
      if (value !== undefined && !propertyDefinition.readOnly) return value;
    }
  }

  return defaultValue;
};

function resolveProperties() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {
    properties: {}
  };
  var propertyDefinitions = arguments.length > 1 ? arguments[1] : undefined;
  var instanceProperties = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var resolvedProperties = {};

  for (var k in propertyDefinitions) {
    resolvedProperties[k] = resolveValue(propertyDefinitions[k], [instanceProperties[k], state.properties[k]], propertyDefinitions[k]["default"]);
  }

  return resolvedProperties;
}
//# sourceMappingURL=resolveProperties.js.map