/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _ajv = _interopRequireDefault(require("../ajv"));

var _getComponentName = _interopRequireDefault(require("../getComponentName"));

var _validateSchema = _interopRequireDefault(require("../validate/validateSchema"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

/**
 * Build the validator for the action payload schema
 * @private
 * @param {*} payloadSchema The schema provided by the user in config.actions[key].schema
 * @returns {Function(value, host)} The function to validate the action's payload
 */
function buildActionPayloadValidator(payloadSchema) {
  if (process.env.NODE_ENV !== 'production') {
    var validate;

    try {
      validate = _ajv["default"].compile(payloadSchema);
    } catch (e) {
      console.error(e); // eslint-disable-line no-console

      return function () {
        return false;
      };
    }

    return function validateActionPayload(value, host) {
      var component = (0, _getComponentName["default"])(host);
      return (0, _validateSchema["default"])(validate, value, {
        pathPrefix: 'payload',
        errorPrefix: "Action payload for ".concat(component, ": ")
      });
    };
  }

  return function () {
    return true;
  };
}

var _default = buildActionPayloadValidator;
exports["default"] = _default;
//# sourceMappingURL=buildActionPayloadValidator.js.map