/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _logValidationError = _interopRequireDefault(require("../logValidationError"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function getPath(pathPrefix, dataPath) {
  if (!pathPrefix && !dataPath) return '';
  if (!pathPrefix && dataPath.startsWith('.')) return "".concat(dataPath.substr(1), " ");
  return "".concat(pathPrefix).concat(dataPath, " ");
}
/**
 * Run a validator against a value to determine its validity. If it is not valid, log an error.
 * @private
 * @param {Function} validator The validator built from one of the methods in ./build
 * @param {any} value The value to run the schema validation against
 * @param {Object} options
 * @param {string} options.pathPrefix Name of the value being validated. Optional
 * @param {string} options.errorPrefix Prefix for the error message
 * @returns {boolean} Whether the value passed validation
 */


function validateSchema(validator, value) {
  var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      _ref$pathPrefix = _ref.pathPrefix,
      pathPrefix = _ref$pathPrefix === void 0 ? '' : _ref$pathPrefix,
      _ref$errorPrefix = _ref.errorPrefix,
      errorPrefix = _ref$errorPrefix === void 0 ? '' : _ref$errorPrefix,
      _ref$shouldLog = _ref.shouldLog,
      shouldLog = _ref$shouldLog === void 0 ? true : _ref$shouldLog;

  var isValid = validator(value);

  if (!isValid) {
    // If there are errors, only report the first one
    var _validator$errors = _slicedToArray(validator.errors, 1),
        _validator$errors$ = _validator$errors[0],
        dataPath = _validator$errors$.dataPath,
        message = _validator$errors$.message,
        _validator$errors$$pa = _validator$errors$.params;

    _validator$errors$$pa = _validator$errors$$pa === void 0 ? {} : _validator$errors$$pa;
    var _validator$errors$$pa2 = _validator$errors$$pa.level,
        level = _validator$errors$$pa2 === void 0 ? 'error' : _validator$errors$$pa2;
    var path = getPath(pathPrefix, dataPath);
    var errorMessage = "".concat(path).concat(message);
    if (shouldLog) (0, _logValidationError["default"])("".concat(errorPrefix).concat(errorMessage), {
      level: level,
      origin: 'validateSchema'
    });
  }

  return isValid;
}

var _default = validateSchema;
exports["default"] = _default;
//# sourceMappingURL=validateSchema.js.map