/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _uiUtils = require("@servicenow/ui-utils");

var _log = _interopRequireDefault(require("../log"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var styleSheetsContent = {};
var PREFIX = 'SEISMIC_STYLES';

var get = function get(version, tagName) {
  var contents = styleSheetsContent[tagName];
  if (contents) return contents;

  try {
    var errorText = "<title>Error</title>";
    contents = localStorage.getItem("".concat(PREFIX, "_").concat(tagName).concat(version));
    if (contents.includes(errorText)) contents = '';
  } catch (e) {
    (0, _log["default"])("The stylesheets content for the tag, ".concat(tagName, ", could not be retrieved."));
  }

  return contents;
};

var set = function set(version, tagName, contents) {
  try {
    styleSheetsContent[tagName] = contents;
    (0, _uiUtils.requestIdleCallback)(function () {
      localStorage.setItem("".concat(PREFIX, "_").concat(tagName).concat(version), contents);
    });
  } catch (e) {
    (0, _log["default"])("The stylesheets content for the custom element, ".concat(name, ", could not be cached."));
  }
};

var _default = {
  set: set,
  get: get
};
exports["default"] = _default;
//# sourceMappingURL=styleSheetsContentCache.js.map