/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = triggerComponentUpdates;

var _triggerPropertyChangeAction = _interopRequireDefault(require("./triggerPropertyChangeAction"));

var _constants = require("../constants");

var _symbols = require("../symbols");

var _log = _interopRequireDefault(require("../log"));

var _isPropToSkipRender = require("./isPropToSkipRender");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function triggerComponentUpdates(componentRegistry, affected, propertyDetails, interactionId, label) {
  var value = propertyDetails.value;
  var renderList = [];

  for (var id in affected) {
    var _el$VARIABLES, _el$VARIABLES$PROPERT;

    var _id$split = id.split(_constants.PROPERTY_SELECTOR_SIGIL),
        _id$split2 = _slicedToArray(_id$split, 2),
        elementId = _id$split2[0],
        propName = _id$split2[1];

    var el = componentRegistry[elementId];
    renderList.push({
      elementId: elementId,
      tagName: el === null || el === void 0 ? void 0 : el.tagName,
      propName: propName,
      propDef: el === null || el === void 0 ? void 0 : (_el$VARIABLES = el[_symbols.VARIABLES]) === null || _el$VARIABLES === void 0 ? void 0 : (_el$VARIABLES$PROPERT = _el$VARIABLES[_symbols.PROPERTY_DEFS]) === null || _el$VARIABLES$PROPERT === void 0 ? void 0 : _el$VARIABLES$PROPERT[propName]
    });

    if (el && propName) {
      (0, _triggerPropertyChangeAction["default"])(el, propertyDetails, {
        propName: propName,
        path: affected[id]
      }, interactionId);
      if (propName === _constants.NOW_HOIST_PROP || propName === _constants.HOIST_ATTR) el[_symbols.HOIST_CHANGED](value);
      if (propName === _constants.HOIST_APPEND_TO_PROP || propName === _constants.NOW_HOIST_APPEND_TO_PROP || propName === _constants.NOW_HOIST_PREPEND_TO_PROP) el[_symbols.HOIST_PARENT_CHANGED](propName);
    }
  }

  for (var i = 0; i < renderList.length; i++) {
    var _renderList$i = renderList[i],
        _elementId = _renderList$i.elementId,
        tagName = _renderList$i.tagName,
        _propName = _renderList$i.propName,
        propDef = _renderList$i.propDef;

    if (componentRegistry[_elementId]) {
      if (!(0, _isPropToSkipRender.isPropToSkipRender)(tagName, _propName, propDef)) componentRegistry[_elementId][_symbols.RENDER](interactionId, label, {
        propName: _propName,
        value: value
      });
    } else {
      (0, _log["default"])("Component with id \"".concat(_elementId, "\" could not be re-rendered from a change in property \"").concat(_propName, "\" because the component is disconnected or not known."), {
        componentId: _elementId,
        origin: 'triggerComponentUpdates'
      });
    }
  }
}
//# sourceMappingURL=triggerComponentUpdates.js.map