/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = triggerPropertyChangeAction;

var _uiMetrics = require("@servicenow/ui-metrics");

var _symbols = require("../symbols");

var _constants = require("../constants");

var _uiInternal = require("@servicenow/ui-internal");

var _getHostVariables = _interopRequireDefault(require("./getHostVariables"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function triggerPropertyChangeAction(el, propertyDetail, affectedPropDetail, interactionId) {
  var name = affectedPropDetail.propName,
      path = affectedPropDetail.path;
  var previousValue = propertyDetail.previousValue,
      value = propertyDetail.value;

  if (path) {
    previousValue = (0, _uiInternal.get)(previousValue, path);
    value = (0, _uiInternal.get)(value, path);
  }

  var hostVariables = (0, _getHostVariables["default"])(el);

  hostVariables[_symbols.CONDITIONAL_DISPATCH](_constants.COMPONENT_PROPERTY_CHANGED, {
    name: name,
    previousValue: previousValue,
    value: value,
    host: el
  }, (0, _uiMetrics.setInteractionId)({
    internal: true
  }, interactionId));
}
//# sourceMappingURL=triggerPropertyChangeAction.js.map