/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validatePresentationalConfig = validatePresentationalConfig;
exports["default"] = validateConfig;

var _buildConfigValidator = _interopRequireDefault(require("./utils/schema/build/buildConfigValidator"));

var _constants = require("./constants");

var _effects = require("./effects");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) { symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); } keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var TYPE_ARRAY = {
  type: 'array'
};
var TYPE_BOOLEAN = {
  type: 'boolean'
};
var TYPE_FN = {
  fn: true
};
var TYPE_NUMBER = {
  type: 'number'
};
var TYPE_OBJECT = {
  type: 'object'
};
var TYPE_STRING = {
  type: 'string'
};

var properties = _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
  additionalProperties: _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
    properties: {
      "default": {},
      computed: TYPE_FN,
      required: TYPE_BOOLEAN,
      selectable: TYPE_BOOLEAN,
      reflect: TYPE_BOOLEAN,
      deepCompare: TYPE_BOOLEAN,
      resolveRef: TYPE_BOOLEAN,
      schema: TYPE_OBJECT,
      unstableParse: _objectSpread(_objectSpread({}, TYPE_BOOLEAN), {}, {
        deprecated: 'has been deprecated, use Type Schemas instead'
      }),
      unstablePreserveEmptyStr: _objectSpread(_objectSpread({}, TYPE_BOOLEAN), {}, {
        deprecated: 'has been deprecated, use Type Schemas instead'
      }),
      dependencies: _objectSpread(_objectSpread({}, TYPE_ARRAY), {}, {
        items: TYPE_STRING
      }),
      onChange: TYPE_FN
    },
    additionalProperties: false
  })
});

var interceptors = _objectSpread(_objectSpread({}, TYPE_ARRAY), {}, {
  items: {
    anyOf: [_objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
      properties: {
        before: TYPE_FN,
        after: TYPE_FN
      },
      additionalProperties: false
    }), TYPE_FN]
  }
});

var actions = _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
  additionalProperties: _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
    properties: {
      handlers: _objectSpread(_objectSpread({}, TYPE_ARRAY), {}, {
        deprecated: 'has been deprecated, use actions handlers instead',
        items: {
          anyOf: [_objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
            properties: {
              effect: TYPE_FN,
              fn: TYPE_FN,
              args: TYPE_ARRAY
            },
            additionalProperties: false
          }), TYPE_FN]
        }
      }),
      interceptors: _objectSpread(_objectSpread({}, interceptors), {}, {
        deprecated: 'has been deprecated, use actions handler `interceptors` instead'
      }),
      effect: _objectSpread(_objectSpread({}, TYPE_FN), {}, {
        deprecated: 'has been deprecated, use actions handlers instead'
      }),
      "private": {
        type: 'boolean',
        deprecated: 'has been deprecated, use actions handlers `stopPropagation` instead'
      },
      schema: TYPE_OBJECT
    },
    additionalProperties: false
  })
});

var actionHandlers = _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
  additionalProperties: {
    anyOf: [_objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
      properties: {
        effect: TYPE_FN,
        args: TYPE_ARRAY,
        descendantOnly: TYPE_BOOLEAN,
        stopPropagation: TYPE_BOOLEAN,
        interceptors: interceptors,
        modifier: {
          anyOf: [_objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
            properties: {
              name: _objectSpread(_objectSpread({}, TYPE_STRING), {}, {
                "enum": [_effects.modifierTypes.DEBOUNCE]
              }),
              delay: TYPE_NUMBER,
              maxWait: TYPE_NUMBER,
              collate: TYPE_BOOLEAN
            },
            additionalProperties: false
          }), _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
            properties: {
              name: _objectSpread(_objectSpread({}, TYPE_STRING), {}, {
                "enum": [_effects.modifierTypes.THROTTLE]
              }),
              limit: TYPE_NUMBER
            },
            additionalProperties: false
          }), _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
            properties: {
              name: _objectSpread(_objectSpread({}, TYPE_STRING), {}, {
                "enum": [_effects.modifierTypes.TAKE_LATEST]
              })
            },
            additionalProperties: false
          })]
        }
      },
      additionalProperties: false
    }), TYPE_FN]
  }
});

var eventHandlers = _objectSpread(_objectSpread({}, TYPE_ARRAY), {}, {
  items: _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
    properties: {
      events: _objectSpread(_objectSpread({}, TYPE_ARRAY), {}, {
        minItems: 1,
        items: TYPE_STRING
      }),
      effect: TYPE_FN,
      target: {
        anyOf: [TYPE_OBJECT, TYPE_STRING]
      },
      capture: TYPE_BOOLEAN,
      modifier: {
        anyOf: [_objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
          properties: {
            name: _objectSpread(_objectSpread({}, TYPE_STRING), {}, {
              "enum": [_effects.modifierTypes.DEBOUNCE]
            }),
            delay: TYPE_NUMBER,
            maxWait: TYPE_NUMBER,
            collate: TYPE_BOOLEAN
          },
          additionalProperties: false
        }), _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
          properties: {
            name: _objectSpread(_objectSpread({}, TYPE_STRING), {}, {
              "enum": [_effects.modifierTypes.THROTTLE]
            }),
            limit: TYPE_NUMBER
          },
          additionalProperties: false
        }), _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
          properties: {
            name: _objectSpread(_objectSpread({}, TYPE_STRING), {}, {
              "enum": [_effects.modifierTypes.TAKE_LATEST]
            })
          },
          additionalProperties: false
        })]
      }
    },
    required: ['events', 'effect'],
    additionalProperties: false
  })
});

var initialState = {};

var setInitialState = _objectSpread({}, TYPE_FN);

var onBootstrap = _objectSpread(_objectSpread({}, TYPE_FN), {}, {
  deprecated: 'life cycle has been deprecated, use `COMPONENT_BOOTSTRAPPED` action handler instead'
});

var onConnect = _objectSpread(_objectSpread({}, TYPE_FN), {}, {
  deprecated: 'life cycle has been deprecated, use `COMPONENT_CONNECTED` action handler instead'
});

var onDisconnect = _objectSpread(_objectSpread({}, TYPE_FN), {}, {
  deprecated: 'life cycle has been deprecated, use `COMPONENT_DISCONNECTED` action handler instead'
});

var onPropertiesSet = _objectSpread(_objectSpread({}, TYPE_FN), {}, {
  deprecated: true
});

var behavior = _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
  properties: {
    name: TYPE_STRING,
    properties: properties,
    actions: actions,
    actionHandlers: actionHandlers,
    eventHandlers: eventHandlers,
    initialState: initialState,
    setInitialState: setInitialState,
    onBootstrap: onBootstrap,
    onConnect: onConnect,
    onDisconnect: onDisconnect,
    onPropertiesSet: onPropertiesSet
  },
  additionalProperties: false
});

var presentationalBehavior = _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
  properties: {
    name: TYPE_STRING,
    properties: properties
  },
  additionalProperties: false
});

var validator = (0, _buildConfigValidator["default"])(_objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
  properties: {
    behaviors: _objectSpread(_objectSpread({}, TYPE_ARRAY), {}, {
      items: {
        anyOf: [behavior, _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
          properties: {
            behavior: behavior,
            options: TYPE_OBJECT
          },
          additionalProperties: false
        })]
      },
      uniqueItems: true
    }),
    actions: actions,
    actionHandlers: actionHandlers,
    eventHandlers: eventHandlers,
    initialState: initialState,
    setInitialState: setInitialState,
    properties: properties,
    renderer: _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
      properties: {
        view: TYPE_FN,
        transformState: TYPE_FN,
        transformProps: TYPE_FN,
        type: TYPE_OBJECT
      },
      additionalProperties: false
    }),
    view: TYPE_FN,
    transformState: TYPE_FN,
    transformProps: TYPE_FN,
    type: TYPE_OBJECT,
    resources: _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
      deprecated: 'has been deprecated, use effects instead'
    }),
    actionTypes: _objectSpread(_objectSpread({}, TYPE_ARRAY), {}, {
      deprecated: true
    }),
    actionTypesToProps: _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
      deprecated: true
    }),
    onBootstrap: onBootstrap,
    onConnect: onConnect,
    onDisconnect: onDisconnect,
    onPropertiesSet: onPropertiesSet,
    dispatches: TYPE_OBJECT,
    slots: TYPE_OBJECT,
    styles: TYPE_STRING,
    stylesheets: _objectSpread(_objectSpread({}, TYPE_ARRAY), {}, {
      items: TYPE_STRING
    }),
    shadowRootOptions: _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
      properties: {
        mode: _objectSpread(_objectSpread({}, TYPE_STRING), {}, {
          "enum": [_constants.SHADOW_ROOT_MODE_OPEN, _constants.SHADOW_ROOT_MODE_CLOSED]
        })
      },
      additionalProperties: false
    })
  },
  additionalProperties: false
}));
var presentationalValidator = (0, _buildConfigValidator["default"])(_objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
  properties: {
    behaviors: _objectSpread(_objectSpread({}, TYPE_ARRAY), {}, {
      items: {
        anyOf: [presentationalBehavior, _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
          properties: {
            behavior: presentationalBehavior,
            options: TYPE_OBJECT
          },
          additionalProperties: false
        })]
      },
      uniqueItems: true
    }),
    properties: properties,
    renderer: _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
      properties: {
        type: TYPE_OBJECT
      },
      additionalProperties: false
    }),
    view: TYPE_FN,
    transformProps: TYPE_FN,
    styles: TYPE_STRING,
    shadowRootOptions: _objectSpread(_objectSpread({}, TYPE_OBJECT), {}, {
      properties: {
        mode: _objectSpread(_objectSpread({}, TYPE_STRING), {}, {
          "enum": [_constants.SHADOW_ROOT_MODE_OPEN, _constants.SHADOW_ROOT_MODE_CLOSED]
        })
      },
      additionalProperties: false
    })
  },
  additionalProperties: false
}));

function validatePresentationalConfig(name, config, options) {
  return presentationalValidator(config, "Config for ".concat(name, ": "), options);
}

function validateConfig(name, config, options) {
  return validator(config, "Config for ".concat(name, ": "), options);
}
//# sourceMappingURL=validateConfig.js.map