/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.whenAllComponentsIdle = whenAllComponentsIdle;
exports["default"] = whenComponentIsIdle;

var _scheduler = require("./scheduler");

var _log = _interopRequireDefault(require("./log"));

var _uiInternal = require("@servicenow/ui-internal");

var _uiConfig = require("@servicenow/ui-config");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

var WHEN_COMPONENT_IS_IDLE_TIMEOUT = _uiConfig.configConstants.WHEN_COMPONENT_IS_IDLE_TIMEOUT,
    WHEN_COMPONENT_IS_IDLE_THROTTLE = _uiConfig.configConstants.WHEN_COMPONENT_IS_IDLE_THROTTLE,
    WHEN_COMPONENT_IS_IDLE_MIN_INTERVAL = _uiConfig.configConstants.WHEN_COMPONENT_IS_IDLE_MIN_INTERVAL;
var timeout = (0, _uiConfig.getConfigItem)(WHEN_COMPONENT_IS_IDLE_TIMEOUT);
var throttle = (0, _uiConfig.getConfigItem)(WHEN_COMPONENT_IS_IDLE_THROTTLE);
var interval = (0, _uiConfig.getConfigItem)(WHEN_COMPONENT_IS_IDLE_MIN_INTERVAL);
var WHEN_COMPONENT_IS_IDLE = _scheduler.types.WHEN_COMPONENT_IS_IDLE,
    WHEN_ALL_COMPONENTS_IDLE = _scheduler.types.WHEN_ALL_COMPONENTS_IDLE;

var check = function check(el) {
  return new Promise(function (resolve) {
    (0, _scheduler.schedule)({
      fn: function fn() {
        resolve((0, _uiInternal.getHostPendingNetworkActionCount)(el) + (0, _scheduler.countHighPriorityTasks)(el) + (0, _scheduler.countMediumPriorityTasks)(el));
      },
      type: WHEN_COMPONENT_IS_IDLE,
      meta: {
        el: el,
        interactionId: 'componentIdleCheck'
      }
    });
  });
};

var pause = function pause() {
  return new Promise(function (resolve) {
    setTimeout(function () {
      return resolve();
    }, throttle);
  });
};

var checkAll = function checkAll() {
  return new Promise(function (resolve) {
    (0, _scheduler.schedule)({
      fn: function fn() {
        resolve((0, _uiInternal.getPendingNetworkActionCount)() + (0, _scheduler.countAllMediumPriorityTasks)() + (0, _scheduler.countAllHighPriorityTasks)());
      },
      type: WHEN_ALL_COMPONENTS_IDLE,
      meta: {
        el: {},
        interactionId: 'allComponentsIdleCheck'
      }
    });
  });
};

function whenAllComponentsIdle() {
  return new Promise(function (resolve) {
    (0, _scheduler.schedule)({
      fn: function () {
        var _fn = _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee() {
          var busy, didPause, timeoutId;
          return regeneratorRuntime.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  busy = true;
                  didPause = false;
                  timeoutId = setTimeout(function () {
                    busy = false;
                    resolve(didPause);
                  }, timeout);

                case 3:
                  if (!busy) {
                    _context.next = 13;
                    break;
                  }

                  _context.next = 6;
                  return checkAll();

                case 6:
                  busy = _context.sent;

                  if (!busy) {
                    _context.next = 11;
                    break;
                  }

                  didPause = true;
                  _context.next = 11;
                  return pause();

                case 11:
                  _context.next = 3;
                  break;

                case 13:
                  clearTimeout(timeoutId);
                  resolve(didPause);

                case 15:
                case "end":
                  return _context.stop();
              }
            }
          }, _callee);
        }));

        function fn() {
          return _fn.apply(this, arguments);
        }

        return fn;
      }(),
      type: WHEN_ALL_COMPONENTS_IDLE,
      meta: {
        el: {},
        interactionId: 'whenAllComponentsIdle'
      }
    });
  });
}

function whenComponentIsIdle(el) {
  return new Promise(function (resolve) {
    (0, _scheduler.schedule)({
      fn: function () {
        var _fn2 = _asyncToGenerator(
        /*#__PURE__*/
        regeneratorRuntime.mark(function _callee2() {
          var busy, timeoutId, start, end;
          return regeneratorRuntime.wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  busy = true;
                  timeoutId = setTimeout(function () {
                    (0, _log["default"])('whenComponentIsIdle timed out.', {
                      host: el,
                      origin: 'whenComponentIsIdle'
                    });
                    busy = false;
                    resolve(el);
                  }, timeout);

                case 2:
                  if (!busy) {
                    _context2.next = 13;
                    break;
                  }

                  start = Date.now();
                  _context2.next = 6;
                  return check(el);

                case 6:
                  busy = _context2.sent;
                  end = Date.now(); // throttle in case of network activity, i.e. something is pending, but component is otherwise idle
                  // if not there will be numerous unnecessary checks while waiting for network request to complete

                  if (!(busy && end - start < interval)) {
                    _context2.next = 11;
                    break;
                  }

                  _context2.next = 11;
                  return pause();

                case 11:
                  _context2.next = 2;
                  break;

                case 13:
                  clearTimeout(timeoutId);
                  resolve(el);

                case 15:
                case "end":
                  return _context2.stop();
              }
            }
          }, _callee2);
        }));

        function fn() {
          return _fn2.apply(this, arguments);
        }

        return fn;
      }(),
      type: WHEN_COMPONENT_IS_IDLE,
      meta: {
        el: el,
        interactionId: 'whenComponentIsIdle'
      }
    });
  });
}
//# sourceMappingURL=whenComponentIsIdle.js.map