/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { applyInteractionId } from '@servicenow/ui-metrics';
import { CONFIG, DISPATCH, PRESENTATIONAL } from '../symbols';
import { emitNowUiEvent, getHostVariables } from '../utils';
import { ACTION_DISPATCHED } from '../constants';
export function createConditionalDispatch(host) {
  const hostVariables = getHostVariables(host);
  return function dispatch(type, payload, meta, error) {
    applyInteractionId(meta);
    if (!hostVariables[PRESENTATIONAL] && hostVariables[CONFIG].shouldDispatch[type]) return hostVariables[DISPATCH](type, payload, meta, error);
    emitNowUiEvent(ACTION_DISPATCHED, {
      host,
      action: {
        type,
        payload,
        meta,
        error
      }
    });
  };
}
//# sourceMappingURL=createConditionalDispatch.js.map