/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { getHostVariables } from '../../utils';
import { addActionHandler } from '../customEventActionHandler';
import { onAction } from '../onAction';
import { COMPONENT_ACTION_DISPATCHED } from '../../constants';
import { RUNTIME_ACTION_HANDLERS, TARGETED_ACTION_TYPES, ANCESTOR_ROOT_NODE, CONFIG, ACTION_HANDLERS, PRESENTATIONAL } from '../../symbols';

const shouldApplyActionHandler = (hostVariables, type) => !hostVariables[PRESENTATIONAL] && !hostVariables[RUNTIME_ACTION_HANDLERS].includes(type) && hostVariables[TARGETED_ACTION_TYPES].includes(COMPONENT_ACTION_DISPATCHED);

const createActionHandler = (host, hostVariables, type) => {
  hostVariables[RUNTIME_ACTION_HANDLERS].push(type);

  if (hostVariables[CONFIG].interceptors[type] && hostVariables[CONFIG].interceptors[type].has('host')) {
    const interceptors = [...hostVariables[CONFIG].interceptors[type].get('host'), ...hostVariables[CONFIG].interceptors[COMPONENT_ACTION_DISPATCHED].get('host')];
    hostVariables[CONFIG].interceptors[type].set('host', interceptors);
    return;
  }

  hostVariables[CONFIG].interceptors[type] = hostVariables[CONFIG].interceptors[COMPONENT_ACTION_DISPATCHED]; // eslint-disable-line no-param-reassign

  const removeActionHandler = addActionHandler(host, 'host', host, type, false, false, onAction);
  hostVariables[ACTION_HANDLERS].push(removeActionHandler);
  hostVariables[TARGETED_ACTION_TYPES] = [...hostVariables[TARGETED_ACTION_TYPES], type]; // eslint-disable-line no-param-reassign
};

export function applyRuntimeActionHandlers(host, type) {
  let parent = host;
  let parentVariables = getHostVariables(parent);

  while (parent) {
    if (shouldApplyActionHandler(parentVariables, type)) createActionHandler(parent, parentVariables, type);
    parent = parentVariables[ANCESTOR_ROOT_NODE];
    parentVariables = getHostVariables(parent);
  }
}
//# sourceMappingURL=applyRuntimeActionHandlers.js.map