/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { locations } from '@servicenow/ui-internal';
import { getHostVariables } from '../../utils';
import { createMetaData } from './createMetaData';
import { appendToPayload } from './appendToPayload';
import { COMPONENT_ID } from '../../symbols';
export function createEventActionDetails(actionType, host, event, capture, eventHandler) {
  const {
    detail = {}
  } = event;
  const {
    type = actionType,
    payload = eventHandler ? appendToPayload(host, {
      event,
      host
    }) : undefined,
    error = null,
    meta = createMetaData(host, {
      eventHandler,
      capture,
      effectType: locations.EVENT,
      event,
      host,

      get path() {
        return event.composedPath().map(el => {
          const hostVariables = getHostVariables(el);
          return hostVariables && hostVariables[COMPONENT_ID];
        }).filter(componentId => componentId);
      }

    }, getHostVariables(host))
  } = detail;
  return {
    type,
    payload,
    error,
    meta
  };
}
//# sourceMappingURL=createEventActionDetails.js.map