/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { getInteractionId, setInteractionId } from '@servicenow/ui-metrics';
import { COMPONENT_DISCONNECTED, SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED } from '../../constants';
import { HELPERS_SCHEDULED, HELPERS_IMMEDIATE } from '../../symbols';
import { normalizeActionMeta, getHostVariables } from '../../utils'; // this is done so that we don't break anyone dispatching from a disconnect
// function or action handler; people are relying on this to unsubscribe from
// AMB and other types of clean up. the AMB effect should be changed into a
// behavior that has a COMPONENT_DISCONNECTED handler, so that people don't
// have to manually clean up or rely on this broken functionality.

export function wrapDispatch(host, action) {
  const hostVariables = getHostVariables(host);
  const interactionId = getInteractionId(action.meta);
  const helpers = action.meta[SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED] ? hostVariables[HELPERS_IMMEDIATE] : hostVariables[HELPERS_SCHEDULED];
  const {
    updateState: originalUpdateState,
    updateProperties: originalUpdateProperties,
    updateContext: originalUpdateContext,
    dispatch: originalDispatch
  } = helpers;

  const updateState = (...args) => originalUpdateState(...args, interactionId);

  const updateProperties = (...args) => originalUpdateProperties(...args, interactionId);

  const updateContext = (...args) => originalUpdateContext(...args, interactionId);

  let dispatch;

  if (action.type !== COMPONENT_DISCONNECTED && !action.meta[SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED]) {
    dispatch = (type, payload, meta = {}, error) => {
      // eslint-disable-next-line no-param-reassign
      meta = normalizeActionMeta(meta, {
        host
      });
      setInteractionId(meta, interactionId);
      return originalDispatch(type, payload, meta, error);
    };
  } else {
    dispatch = (type, payload, meta = {}, error) => {
      // eslint-disable-next-line no-param-reassign
      meta = normalizeActionMeta(meta, {
        host
      });
      setInteractionId(meta, interactionId);
      meta[SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED] = true; // eslint-disable-line no-param-reassign

      return originalDispatch(type, payload, meta, error);
    };
  }

  dispatch.updateState = updateState;
  dispatch.updateProperties = updateProperties;
  dispatch.updateContext = updateContext;
  return {
    updateState,
    updateProperties,
    updateContext,
    dispatch
  };
}
//# sourceMappingURL=wrapDispatch.js.map