/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
const consumerCache = {};
export const addConsumer = (key, host) => {
  if (!consumerCache[key]) {
    consumerCache[key] = new Set();
    return consumerCache[key].add(host);
  } else if (consumerCache[key].has(host)) {
    return;
  }

  consumerCache[key].add(host);
};
export const removeConsumer = host => {
  for (const k in consumerCache) {
    if (consumerCache[k].has(host)) {
      consumerCache[k].delete(host);
    }
  }
};
export const getConsumers = key => {
  return consumerCache[key] && consumerCache[key].values() || new Set().values();
};
export const removeConsumersForContextKey = key => {
  if (consumerCache[key]) consumerCache[key].clear();
};
export const removeAllConsumers = () => {
  for (const k in consumerCache) delete consumerCache[k];
};
//# sourceMappingURL=consumers.js.map