/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { memoize } from '@servicenow/ui-internal';
import log from '../log';
import { addConsumer } from './consumers';
import { prefix } from './createContext';
import { getContext } from './contexts';
const shouldNotResolveContext = memoize(key => !key.toString().includes(prefix), {
  profileName: 'shouldNotResolveContext'
});
export default function createContextProxy(host) {
  return new Proxy({}, {
    get(target, prop) {
      if (shouldNotResolveContext(prop)) {
        if (process.env.NODE_ENV !== 'production') {
          log(`The context ${prop.toString()} could not be accessed. Context must be defined using the "createContext" function. "createContext" can imported from @servicenow/ui-core.`, {
            host: this,
            origin: 'createContextProxy'
          });
        }

        return target[prop];
      }

      addConsumer(prop, host);
      return getContext(prop);
    },

    set(target, prop) {
      if (process.env.NODE_ENV !== 'production') {
        log(`Cannot set context value ${prop.toString()}. updateContext must be used.`, {
          host: this,
          origin: 'createContextProxy'
        });
      }
    }

  });
}
//# sourceMappingURL=createContextProxy.js.map