/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { debounce, takeLatest, throttle, getTargetElement, hostMap, modifierTypes } from './modifiers';
import runEffect from './runEffect';
export default function effectRunner(effect, ctx, ...args) {
  const {
    effect: effectFn,
    modifier = {}
  } = effect;
  const coeffects = args[args.length - 1];
  if (!coeffects) return runEffect(effectFn, args, ctx);
  const targetElement = getTargetElement(coeffects);

  switch (modifier.name) {
    case modifierTypes.DEBOUNCE:
      return debounce(targetElement, coeffects.action.type, effectFn, modifier, ctx, runEffect)(...args);

    case modifierTypes.TAKE_LATEST:
      return runEffect(takeLatest(targetElement, coeffects.action.type, effectFn), args, ctx).then(() => {
        delete hostMap.get(targetElement).get(effectFn)[coeffects.action.type];
      });

    case modifierTypes.THROTTLE:
      return throttle(targetElement, coeffects.action.type, effectFn, modifier, ctx, runEffect)(...args);

    default:
      return runEffect(effectFn, args, ctx);
  }
}
//# sourceMappingURL=effectRunner.js.map