/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import hostMap from './hostMap';

function createCancelable(fn) {
  return function cancelableEffect(...args) {
    return fn(...args);
  };
}
/**
 * Take the latest effect and cancel any previously started effect fns
 *
 * Currently only supports generator fns
 *
 * @param host
 * @param actionType
 * @param effectFn
 * @returns {R|*}
 */


export default function takeLatest(host, actionType, effectFn) {
  if (!hostMap.has(host)) hostMap.set(host, new WeakMap());
  const effectMap = hostMap.get(host);
  if (!effectMap.has(effectFn)) effectMap.set(effectFn, {});
  const actionMappings = effectMap.get(effectFn);
  if (actionMappings[actionType]) actionMappings[actionType].cancelled = true;
  actionMappings[actionType] = createCancelable(effectFn);
  return actionMappings[actionType];
}
//# sourceMappingURL=takeLatest.js.map