/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { mark, types as metrics } from '@servicenow/ui-metrics';
/**
 * Runs effect function (includes async and generator functions)
 *
 * @param fn
 * @param args
 * @returns {Promise}
 */

export default function runEffect(fn, args, {
  host,
  interactionId,
  type
}) {
  mark(host, interactionId, metrics.EFFECT_START, {
    type
  });
  const generator = fn(...args);

  if (!generator || !generator.next) {
    mark(host, interactionId, metrics.EFFECT_END, {
      type
    });
    return Promise.resolve(generator);
  }

  function runner(verb, arg) {
    if (fn.cancelled) {
      mark(host, interactionId, metrics.EFFECT_END, {
        type
      });
      return Promise.resolve(generator.return());
    }

    let result;

    try {
      result = generator[verb](arg);
    } catch (err) {
      mark(host, interactionId, metrics.EFFECT_END, {
        type
      });
      return Promise.reject(err);
    }

    if (result.done) {
      mark(host, interactionId, metrics.EFFECT_END, {
        type
      });
      return Promise.resolve(result.value);
    }

    mark(host, interactionId, metrics.EFFECT_END, {
      type
    });
    return Promise.resolve(result.value).then(onResolved, onRejected);
  }

  const onResolved = runner.bind(runner, 'next');
  const onRejected = runner.bind(runner, 'throw');
  return onResolved();
}
//# sourceMappingURL=runEffect.js.map