/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { defineRegistry, get as getStats } from './stats';
import whenComponentIsIdle from './whenComponentIsIdle';
import { whenAllComponentsIdle } from './whenComponentIsIdle';
import createCustomElementFactory from './createCustomElementFactory';
import { actionTypes as componentActionTypes, locations, allLogTypes as logTypes } from '@servicenow/ui-internal';
import { COMPONENT_MODE_SUSPEND as SUSPEND, COMPONENT_MODE_ACTIVE as ACTIVE, COMPONENT_MODE_DETACH as DETACH, SHADOW_ROOT_MODE_OPEN as OPEN, SHADOW_ROOT_MODE_CLOSED as CLOSED, PRIORITY_LEVEL_LOW as LOW, PRIORITY_LEVEL_MEDIUM as MEDIUM, ARIA_LIVE_REGION_UPDATE_REQUESTED } from './constants';
import { createUnstableResolvePropertySelectorValue, unstableAddRenderOnPropertySelectorValueChange, unstableRemoveRenderOnPropertySelectorValueChange } from './property-selectors';
import { modifierTypes } from './effects';
import { declarativeOperations, getAncestorNode } from './utils';
import createContext from './context/createContext';
import { politeId, assertiveId } from './accessibility/ariaLiveRegion';
import { CONTEXT_VIEW_KEY } from './symbols';
const componentModes = {
  SUSPEND,
  ACTIVE,
  DETACH
};
const shadowRootModes = {
  OPEN,
  CLOSED
};
const nowPriorityLevels = {
  LOW,
  MEDIUM
};
const componentRegistry = {};
const unstableResolvePropertySelectorValue = createUnstableResolvePropertySelectorValue(componentRegistry);
const createCustomElement = createCustomElementFactory(componentRegistry);
const createPresentationalCustomElement = createCustomElementFactory(componentRegistry, {
  presentational: true
});
const actionTypes = { ...componentActionTypes,
  ARIA_LIVE_REGION_UPDATE_REQUESTED
};
const ariaLiveRegionIds = {
  polite: politeId,
  assertive: assertiveId
};
const contextViewKey = CONTEXT_VIEW_KEY;

const getComponentById = id => componentRegistry[id];

const getComponentInstances = () => ({ ...componentRegistry
});

defineRegistry(componentRegistry);
window.nowUiFramework = window.nowUiFramework || {};
window.nowUiFramework.core = {
  getStats
};
export { createCustomElement, createPresentationalCustomElement, actionTypes, logTypes, modifierTypes, shadowRootModes, nowPriorityLevels, declarativeOperations, locations as errorLocations, componentModes, ariaLiveRegionIds, createContext, whenComponentIsIdle, whenAllComponentsIdle, contextViewKey, getAncestorNode, getComponentById, getComponentInstances, getStats, // Unstable APIs - Subject to change
unstableResolvePropertySelectorValue, unstableAddRenderOnPropertySelectorValueChange, unstableRemoveRenderOnPropertySelectorValueChange };
//# sourceMappingURL=index.js.map