/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { PROPERTY_SELECTOR_SIGIL } from '../constants';
import isSelectableProperty from './isSelectableProperty';
/**
 * Create a function for getting components affected by a property name and source component id
 * @function createGetAffected
 * @private
 *
 * @param {Object} mappings - A object that will contain mappings
 * @returns {function} - A function for getting components affected by a property name and source component id
 */

export default (mappings => function getAffected(propertyName, providerId, componentRegistry, includeSource = true) {
  const sourceAffected = includeSource ? {
    [`${providerId}${PROPERTY_SELECTOR_SIGIL}${propertyName}`]: ''
  } : {};
  let componentsAffected = { ...mappings[`${providerId}${PROPERTY_SELECTOR_SIGIL}${propertyName}`]
  };
  if (!Object.keys(componentsAffected).length) return sourceAffected;
  if (!isSelectableProperty(componentRegistry, providerId, propertyName)) return sourceAffected;

  (function getAffectedSelectorChain(componentMap) {
    for (const selectorMap in componentMap) {
      if (mappings[selectorMap]) {
        componentsAffected = { ...componentsAffected,
          ...mappings[selectorMap]
        };
        getAffectedSelectorChain(mappings[selectorMap]);
      }
    }
  })(componentsAffected);

  return { ...sourceAffected,
    ...componentsAffected
  };
});
//# sourceMappingURL=createGetAffected.js.map