/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { PROPERTY_SELECTOR_SIGIL } from '../constants';
/**
 * Create a function removing mapped components by component id and optionally property name
 * @function createRemoveMappings
 * @private
 *
 * @param {Object} mappings - A object that will contain mappings
 * @returns {function} - A function for removing mapped components by component id and optionally property name
 */

export default (mappings => function removeMappings(elementId, propertyName) {
  if (elementId && propertyName) {
    for (const key in mappings) {
      const selectorKey = mappings[key][`${elementId}${PROPERTY_SELECTOR_SIGIL}${propertyName}`];
      if (selectorKey === '' || selectorKey) delete mappings[key][`${elementId}${PROPERTY_SELECTOR_SIGIL}${propertyName}`]; // eslint-disable-line no-param-reassign

      if (Object.keys(mappings[key]).length === 0) delete mappings[key]; // eslint-disable-line no-param-reassign
    }
  }

  if (elementId && !propertyName) {
    for (const key in mappings) {
      for (const subKey in mappings[key]) {
        const componentId = subKey.split(PROPERTY_SELECTOR_SIGIL)[0];
        if (componentId === elementId) delete mappings[key][subKey]; // eslint-disable-line no-param-reassign
      }

      if (Object.keys(mappings[key]).length === 0) delete mappings[key]; // eslint-disable-line no-param-reassign
    }
  }
});
//# sourceMappingURL=createRemoveMappings.js.map