/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { memoize } from '@servicenow/ui-internal';
const getParts = memoize(value => {
  const slashIndex = value.indexOf('/');
  if (slashIndex === -1) return [];
  const [delimeter, pathSplitIndex] = getDelimeterDetail(value);
  return [value.substring(1, slashIndex), value.substring(slashIndex + 1, pathSplitIndex === -1 ? value.length : delimeter === '.' ? pathSplitIndex : pathSplitIndex + 1), pathSplitIndex !== -1 ? value.substring(pathSplitIndex + 1) : ''];
}, {
  profileName: 'getParts'
});

const getDelimeterDetail = selector => {
  const {
    length: selectorLength
  } = selector;

  for (let i = 0; i < selectorLength; i++) {
    const value = selector.charAt(i);
    if (value === '.') return [value, i];
    if (value === '[') return [value, i - 1];
  }

  return ['', -1];
};
/**
 * Returns property selector parts, id and property name
 * @function getSelectorParts
 * @private
 *
 * @param {Object} mappings - A object that will contain mappings
 * @returns {Array} - Property selector parts, id, property name and path
 */


export default function getSelectorParts(value) {
  if (typeof value !== 'string') return [];
  if (value[0] !== '@') return [];
  return getParts(value);
}
//# sourceMappingURL=getSelectorParts.js.map