/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { getSelectorParts, isSelectableProperty, isSelector } from './';

function resolveSelectorChain(id, name, path, propertyResolver, componentRegistry) {
  const property = propertyResolver(componentRegistry, id, name, path);

  if (property && property[0] === '@') {
    const [id, name, path] = getSelectorParts(property);
    return resolveSelectorChain(id, name, path, propertyResolver, componentRegistry);
  }

  return property;
}
/**
 * Returns property selector parts for a single property
 * @function resolveSelectedProperty
 * @private
 *
 * @param {Object} componentRegistry - Component registry
 * @param {function} propertyResolver - Returns the value for a property
 * @param {Object} property - property to be resolved
 * @param {Object} propertyDef - property definition from the config
 * @returns {Object} - The  resolved property
 */


export default function resolveSelectedProperty(componentRegistry, propertyResolver, property, propertyDef) {
  const [id, name, path] = getSelectorParts(property);
  if (!id || !name) return property;
  const defaultValue = propertyDef && propertyDef.default;
  if (!isSelectableProperty(componentRegistry, id, name)) return !isSelector(defaultValue) ? defaultValue : undefined;
  const resolvedValue = resolveSelectorChain(id, name, path, propertyResolver, componentRegistry);
  if (resolvedValue !== undefined) return resolvedValue;
  return !isSelector(defaultValue) ? defaultValue : undefined;
}
//# sourceMappingURL=resolveSelectedProperty.js.map