/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { version } from '../package.json';
let componentRegistry = {};
const stats = {
  constructedByTagName: {},
  definedTagNames: [],
  deferredByTagName: {}
};
export const defineRegistry = registry => {
  componentRegistry = registry;
  stats.constructedByTagName = {};
  stats.definedTagNames = [];
  stats.deferredByTagName = {};
};
export const get = () => {
  const countByTagName = {};

  for (const k in componentRegistry) {
    const tagName = componentRegistry[k].tagName.toLowerCase();
    if (!countByTagName[tagName]) countByTagName[tagName] = 0;
    countByTagName[tagName]++; // eslint-disable-line no-plusplus
  }

  let totalConstructed = 0;

  for (const k in stats.constructedByTagName) {
    totalConstructed += stats.constructedByTagName[k];
  }

  const unusedTagNames = [];

  for (let i = 0; i < stats.definedTagNames.length; i++) {
    const tagName = stats.definedTagNames[i];

    if (!stats.constructedByTagName[tagName]) {
      unusedTagNames.push(tagName);
    }
  }

  return {
    version,
    definitions: {
      total: stats.definedTagNames.length,
      unusedTagNames,
      tagNames: [...stats.definedTagNames]
    },
    constructed: {
      total: totalConstructed,
      countByTagName: stats.constructedByTagName
    },
    connected: {
      total: Object.keys(componentRegistry).length,
      countByTagName,
      deferredByTagName: stats.deferredByTagName,
      nodes: { ...componentRegistry
      }
    }
  };
};
export const set = (statName, tagName) => {
  switch (statName) {
    case 'constructed':
      stats.constructedByTagName[tagName] = stats.constructedByTagName[tagName] || 0;
      stats.constructedByTagName[tagName]++; // eslint-disable-line no-plusplus

      break;

    case 'defined':
      stats.definedTagNames.push(tagName);
      break;

    case 'defer':
      stats.deferredByTagName[tagName] = stats.deferredByTagName[tagName] || 0;
      stats.deferredByTagName[tagName]++; // eslint-disable-line no-plusplus

      break;

    case 'undefer':
      stats.deferredByTagName[tagName] = stats.deferredByTagName[tagName] || 0;
      if (stats.deferredByTagName[tagName]) stats.deferredByTagName[tagName]--; // eslint-disable-line no-plusplus

      break;
  }
};
//# sourceMappingURL=stats.js.map