/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import getHostVariables from './getHostVariables';
import { ROOT_NODE } from '../symbols';
export default function applyIntersectStyles(el, config, options) {
  const {
    width,
    height
  } = options;
  if (!width && !height) return;
  const hostVariables = getHostVariables(el);
  const rootNode = hostVariables[ROOT_NODE];
  const styles = `:host {${width ? `width: ${width};` : ''}${height ? `height: ${height};` : ''}}`;
  return applyWithTag(rootNode, styles);
}

function applyWithTag(rootNode, styles) {
  const styleElement = window.document.createElement('style');
  styleElement.type = 'text/css';
  styleElement.innerHTML = styles;
  rootNode.appendChild(styleElement);
  return () => rootNode.removeChild(styleElement);
}
//# sourceMappingURL=applyIntersectStyles.js.map