/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { createInteractionId, applyInteractionId } from '@servicenow/ui-metrics';
import { normalizeActionMeta } from './normalizeActionMeta';

function applyMetricsToDispatch(dispatch) {
  return (type, payload, meta = {}, error) => {
    // eslint-disable-next-line no-param-reassign
    meta = normalizeActionMeta(meta, {});
    applyInteractionId(meta);
    return dispatch(type, payload, meta, error);
  };
}

export default function applyMetrics(helpers, interactionId = createInteractionId()) {
  const withMetrics = typeof helpers === 'function' ? applyMetricsToDispatch(helpers) : {};
  if (helpers.updateState) withMetrics.updateState = (...args) => helpers.updateState(...args, interactionId);
  if (helpers.updateContext) withMetrics.updateContext = (...args) => helpers.updateContext(...args, interactionId);
  if (helpers.updateProperties) withMetrics.updateProperties = (...args) => helpers.updateProperties(...args, interactionId);
  if (helpers.dispatch) withMetrics.dispatch = applyMetricsToDispatch(helpers.dispatch);
  return withMetrics;
}
//# sourceMappingURL=applyMetrics.js.map