/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import log from '../log';
import { allLogTypes as logTypes } from '@servicenow/ui-internal';

function isAlphaNumeric(str) {
  return /^[a-z]+[a-z0-9]*$/i.test(str);
}

function assertValidPropertyNames(properties) {
  if (process.env.NODE_ENV === 'development') {
    const {
      length: propertiesCount
    } = properties;

    for (let i = 0; i < propertiesCount; i++) {
      const property = properties[i];

      if (isAlphaNumeric(property) === false) {
        log(`Invalid property name ${property}`, {
          level: logTypes.ERROR,
          origin: 'assertValidPropertyNames'
        });
      }
    }
  }

  return true;
}

export default assertValidPropertyNames;
//# sourceMappingURL=assertValidPropertyNames.js.map