/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { normalizeActionHandler } from './normalizeActionHandler'; // TODO: this format should be deprecated in paris

export function mergeActions(actionHandlers, actions, tagName) {
  for (const k in actions) {
    const {
      handlers = [],
      effect: actionEffect,
      args: actionArgs,
      interceptors: actionInterceptors,
      private: isPrivate
    } = actions[k];
    const {
      length: handlersCount
    } = handlers;
    actionHandlers[k] = actionHandlers[k] || []; // eslint-disable-line no-param-reassign

    for (let i = 0; i < handlersCount; i++) {
      const handler = handlers[i];
      const {
        effect
      } = handler;
      const {
        fn,
        args = []
      } = effect || handler;

      if (typeof fn === 'function') {
        const {
          interceptors
        } = handler;
        actionHandlers[k].push(normalizeActionHandler({
          effect: fn,
          args,
          interceptors,
          private: isPrivate
        }, tagName));
      } else {
        actionHandlers[k].push(normalizeActionHandler({ ...handler,
          private: isPrivate
        }, tagName));
      }
    }

    if (handlersCount === 0 && isPrivate || actionEffect || actionInterceptors) {
      actionHandlers[k].push(normalizeActionHandler({
        effect: actionEffect || (() => ({})),
        interceptors: actionInterceptors,
        args: actionArgs,
        private: isPrivate
      }, tagName));
    }
  }

  return actionHandlers;
}
//# sourceMappingURL=mergeActions.js.map