/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import log from '../../log';
export function mergeBehaviorActions(actions, behaviors, tagName) {
  const mergedActions = {};
  const {
    length: behaviorsCount
  } = behaviors;
  const behaviorNames = {};

  for (let i = 0; i < behaviorsCount; i++) {
    const {
      name,
      behavior: {
        actions: behaviorActions = {}
      }
    } = behaviors[i];

    for (const k in behaviorActions) {
      if (mergedActions[k]) {
        log(`Behavior ${name} overwrote action ${k} for ${tagName}.`, {
          tagName,
          origin: 'mergeBehaviorActions'
        });
      }

      mergedActions[k] = behaviorActions[k];
      behaviorNames[k] = name;
    }
  }

  for (const k in actions) {
    if (mergedActions[k]) {
      log(`The component definition for ${tagName} overwrote action ${k} defined by the ${behaviorNames[k]} behavior.`, {
        tagName,
        origin: 'mergeBehaviorActions'
      });
    }

    mergedActions[k] = actions[k];
  }

  return mergedActions;
}
//# sourceMappingURL=mergeBehaviorActions.js.map