/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { normalizeActionHandler } from './normalizeActionHandler';
export function mergeEventActionHandlers(actionHandlers, eventHandlers, tagName) {
  const {
    length: eventHandlerCount
  } = eventHandlers;

  for (let i = 0; i < eventHandlerCount; i++) {
    const {
      events = [],
      effect,
      modifiers = [],
      modifier = {},
      interceptors = [],
      target,
      capture,
      behaviorOptions
    } = eventHandlers[i];
    const {
      length: eventsCount
    } = events;

    for (let j = 0; j < eventsCount; j++) {
      const eventName = events[j];
      actionHandlers[eventName] = actionHandlers[eventName] || []; // eslint-disable-line no-param-reassign

      const normalizedActionHandler = { ...normalizeActionHandler({
          effect,
          modifiers,
          modifier,
          interceptors,
          target,
          capture,
          eventHandler: true
        }, tagName),
        ...(behaviorOptions && {
          behaviorOptions
        })
      };
      actionHandlers[eventName].push(normalizedActionHandler);
    }
  }

  return actionHandlers;
}
//# sourceMappingURL=mergeEventActionHandlers.js.map