/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import log from '../log';
import { CONTEXT_VIEW_KEY } from '../symbols';
export default function createOnStateChangeArgs(useCoeffectsLikeObjectInView, componentId, state, context, helpers, host) {
  if (useCoeffectsLikeObjectInView) {
    const {
      properties
    } = state;
    const stateSansProperties = { ...state
    };
    delete stateSansProperties.properties;
    return [{
      state: stateSansProperties,
      properties,
      context,
      ...helpers,
      host
    }, undefined];
  }

  let contextKey = 'context';

  if (state.context) {
    contextKey = CONTEXT_VIEW_KEY;
    const message = 'A "context" property key already exists in the view derived state object. The context value was assigned to a symbol.' + ' The symbol constant, "contextViewKey", can imported from @servicenow/ui-core.';
    log(message, {
      host,
      origin: 'createOnStateChangeArgs'
    });
  }

  return [{
    componentId,
    ...state,
    [contextKey]: context
  }, helpers];
}
//# sourceMappingURL=createOnStateChangeArgs.js.map