/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { toLower } from '@servicenow/ui-internal';
import { SEISMIC_HOIST_TAG_NAME } from '../constants';
const doc = window.document;
const styleFragments = {};
/**
 * @function
 * @param {String} [style]
 * @param {Array.<String>} [stylesheetsContent]
 * @param {String} [tagName]
 * @returns {HTMLElement}
 */

export default function createStyleFragment(style, stylesheetsContent, tagName) {
  const cachedStyleFragment = styleFragments[tagName];
  if (cachedStyleFragment) return cachedStyleFragment.content.cloneNode(true);
  let styleFragment;
  const externalStylesFragment = document.head.querySelector(`#now-${toLower(tagName)}-styles`);

  if (externalStylesFragment) {
    styleFragment = mergeStylesheets(externalStylesFragment, stylesheetsContent);
  } else {
    let styles = '';
    styleFragment = doc.createElement('template');
    if (style) styles += style;
    if (stylesheetsContent) styles += stylesheetsContent;
    styleFragment.innerHTML = `<style type="text/css">${styles}</style>`;
  }

  if (tagName !== SEISMIC_HOIST_TAG_NAME) styleFragments[tagName] = styleFragment;
  return styleFragment.content.cloneNode(true);
}

function mergeStylesheets(externalStylesFragment, stylesheetsContent) {
  if (!stylesheetsContent) return externalStylesFragment; // this case should be rare, if ever; ideally we should deprecate support for config.stylesheets

  const externalStylesEl = externalStylesFragment.content.cloneNode(true).children[0];
  externalStylesEl.textContent = `${stylesheetsContent}\n${externalStylesEl.textContent}`;
  const mergedStyleFragment = doc.createElement('template');
  mergedStyleFragment.content.appendChild(externalStylesEl);
  return mergedStyleFragment;
}
//# sourceMappingURL=createStyleFragment.js.map