/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { toString, get } from '@servicenow/ui-internal';
import toAttributeName from './toAttributeName';
import { isSelector } from '../property-selectors';
export default function getInitialPropertyValue(el, key, propertyDefs = {}) {
  if (propertyDefs[key] && propertyDefs[key].readOnly) return propertyDefs[key].default;
  const property = el[key];
  const attr = toAttributeName(key);
  const unstablePreserveEmptyStr = propertyDefs[key] && propertyDefs[key].unstablePreserveEmptyStr || false;

  if (key !== 'style' && el.hasAttribute(attr) && !isSelector(property)) {
    const value = el.getAttribute(attr);
    const preserve = unstablePreserveEmptyStr || get(propertyDefs, `[${key}].schema.type`) === 'string';
    return value || (preserve ? value : true);
  } // property.toString() will return [object CSSStyleDeclaration] if the style property is a true style property
  // vs. a style object (key, value) that is passed via JSX in the view layer


  return key === 'style' ? toString(property) === '[object Object]' ? property : undefined : property;
}
//# sourceMappingURL=getInitialPropertyValue.js.map