/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { applyInteractionId, mark, getInteractionId, types as metrics } from '@servicenow/ui-metrics';
import getStatePropsContext from './getStatePropsContext';
import deferHelperExecution from './deferHelperExecution';
import { SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED, HELPERS_SCHEDULED, HELPERS_DEFER } from '../../constants';
import { normalizeActionMeta } from '../normalizeActionMeta';

const wrapDispatch = (host, shouldWrap) => {
  if (shouldWrap === HELPERS_SCHEDULED) return host.dispatch;
  return function dispatch(type, payload, meta = {}, error) {
    meta[SCHEDULE_IMMEDIATELY_DO_NOT_USE_OR_YOU_WILL_BE_IMMEDIATELY_FIRED] = true; // eslint-disable-line no-param-reassign

    return host.dispatch(type, payload, meta, error);
  };
};

export default function createDispatch(host, executionTiming) {
  return function dispatch(typeOrFn, payload, meta = {}, error) {
    // eslint-disable-next-line no-param-reassign
    meta = normalizeActionMeta(meta, {
      host
    });
    const interactionId = getInteractionId(meta);
    if (executionTiming === HELPERS_DEFER) mark(host, interactionId, metrics.DEFER, {
      deferred: metrics.DISPATCH,
      type: typeOrFn
    });
    deferHelperExecution(executionTiming, () => {
      const wrappedDispatch = wrapDispatch(host, executionTiming);

      if (typeof typeOrFn === 'function') {
        const statePropsContext = getStatePropsContext(host, interactionId);
        const {
          type,
          payload,
          meta = {},
          error,
          shouldDispatch = true
        } = typeOrFn(statePropsContext);

        if (shouldDispatch) {
          applyInteractionId(meta);
          wrappedDispatch(type, payload, meta, error);
        }
      } else {
        applyInteractionId(meta);
        wrappedDispatch(typeOrFn, payload, meta, error);
      }
    }, { ...meta,
      el: host,
      interactionId
    });
  };
}
//# sourceMappingURL=createDispatch.js.map