/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import log from '../../log';
import createUpdateState from './createUpdateState';
import createUpdateContext from './createUpdateContext';
import createUpdateProperties from './createUpdateProperties';
import createDispatch from './createDispatch';
export default function createHelpers(host, propertyDefs, executionTiming = null, options = {}) {
  const wrappedDispatch = createDispatch(host, executionTiming);

  const dispatch = (...args) => {
    log(`Please update the view for ${host.tagName.toLowerCase()} from using dispatch as the second argument to an object with dispatch, updateState, and updateProperties`, {
      host,
      origin: 'createHelpers'
    });
    wrappedDispatch(...args);
  };

  if (options.presentational) return {
    dispatch: wrappedDispatch
  };
  dispatch.dispatch = wrappedDispatch;
  dispatch.updateState = createUpdateState(host, executionTiming);
  dispatch.updateProperties = createUpdateProperties(host, propertyDefs, executionTiming);
  dispatch.updateContext = createUpdateContext(host, executionTiming);
  return dispatch;
}
//# sourceMappingURL=index.js.map