/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { getConfigItem, configConstants } from '@servicenow/ui-config';
import { INTERSECTION_TRIGGERED } from '../symbols';
const {
  INTERSECT_ROOT_MARGIN
} = configConstants;

function intersection(entries) {
  entries.forEach(entry => {
    if (entry.isIntersecting) {
      entry.target[INTERSECTION_TRIGGERED]();
      observer.unobserve(entry.target);
    }
  });
} // The distance away from the viewport when a component should render


const VIEWPORT_INTERSECTION_MARGIN = getConfigItem(INTERSECT_ROOT_MARGIN);
const options = {
  rootMargin: VIEWPORT_INTERSECTION_MARGIN
};
let observer;
if (!window.IntersectionObserver) observer = {
  observe() {},

  unobserve() {}

};else observer = new IntersectionObserver(intersection, options);
export default observer;
//# sourceMappingURL=intersectionObserver.js.map