/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { get as _get, set as _set } from '@servicenow/ui-internal';
import { assign as _assign } from './assign';
import { merge as _merge } from './merge';

function operateAtPath(operationFn, obj, path, value) {
  if (!path) return operationFn(obj, value);

  const location = _get(obj, path);

  if (location === undefined || typeof location !== 'object') return _set(obj, path, value);
  return operationFn(location, value);
} // Object


export const set = (obj, {
  path,
  value
}) => _set(obj, path, value);
export const assign = (obj, {
  path,
  value
}) => operateAtPath(_assign, obj, path, value);
export const merge = (obj, {
  path,
  value
}) => operateAtPath(_merge, obj, path, value); // Array

export const pop = (obj, {
  path
}) => _get(obj, path).pop();
export const push = (obj, {
  path,
  value
}) => _get(obj, path).push(value);
export const shift = (obj, {
  path
}) => _get(obj, path).shift();
export const unshift = (obj, {
  path,
  value
}) => _get(obj, path).unshift(value);
export const splice = (obj, {
  path,
  start: _start = 0,
  deleteCount: _deleteCount = 0,
  addItems: _addItems = []
}) => _get(obj, path).splice(_start, _deleteCount, ..._addItems);
export const concat = (obj, {
  path,
  value
}) => {
  const arr = _get(obj, path);

  return arr.push.apply(arr, value); // eslint-disable-line prefer-spread
};
export const declarativeOperations = {
  SET: 'set',
  ASSIGN: 'assign',
  MERGE: 'merge',
  POP: 'pop',
  PUSH: 'push',
  SHIFT: 'shift',
  UNSHIFT: 'unshift',
  SPLICE: 'splice',
  CONCAT: 'concat'
}; // TODO: Map / Set / Date
//# sourceMappingURL=operations.js.map