/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { toLower } from '@servicenow/ui-internal';
/**
 * Parse the property value
 * @private
 *
 * @param {*} value The value of the property
 * @param {*} schemaType The type of the property defined in its schema
 */

export default function parse(value, schemaType, name, tagName) {
  if (schemaType === 'boolean' && value === '') return true;

  try {
    return JSON.parse(value);
  } catch (e) {
    // throw to take advantage of sandbox logging and dispatching
    throw new Error(`Parsing failed for property ${name} on component ${toLower(tagName)}: expected to be type ${schemaType} but received string.`);
  }
}
//# sourceMappingURL=parse.js.map