/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { PARSE_PROP_CONF_KEY, PACKAGE_NAME } from '../../constants';
import isValuePropertySelector from './isValuePropertySelector';
import { sandbox, locations, toLower, kebabCase, allLogTypes as logTypes } from '@servicenow/ui-internal';
import log from '../../log';
import { parseRefProperty } from './refProperty';
import { default as validParseTypes } from './types';
import parse from './parse';
export default function parseProperty(name, value, def = {}, host, dispatch) {
  const unstableParse = def[PARSE_PROP_CONF_KEY];
  const schemaType = def.schema && def.schema.type;
  const isValidParseType = validParseTypes.includes(schemaType);
  const resolvedValue = def.resolveRef ? parseRefProperty(host, name, value) : value;
  const shouldUnstableParse = (unstableParse || validParseTypes.includes(schemaType)) && !isValuePropertySelector(resolvedValue) && typeof resolvedValue === 'string';

  if (shouldUnstableParse && isValidParseType && !host.hasAttribute(kebabCase(name))) {
    const component = `<${toLower(host.tagName)} />`;
    log(`For property ${name} on component ${component}: expected to be type ${schemaType} but received string. ${PACKAGE_NAME} will attempt to parse this value, but you should set this to the correct type.`, {
      level: logTypes.WARN,
      host,
      origin: 'parseProperty'
    });
  }

  return shouldUnstableParse ? sandbox(parse, {
    args: [resolvedValue, schemaType, name, host.tagName],
    host,
    dispatch,
    location: locations.PROPERTY,
    details: {
      parse: true,
      name
    },
    log
  }) : resolvedValue;
}
//# sourceMappingURL=property.js.map