/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
// TODO: account for p2.computed relying on p1.computed; use getters???
import { UPDATE_AFFECTED_COMPONENTS, PROPS } from '../symbols';
import filterComputedProperties from './filterComputedProperties';
import computeProperty from './computeProperty';
import quickEquals from './helpers/quickEquals';
export default function resolveComputedProperties(state, host, hostVariables, interactionId) {
  const computed = filterComputedProperties(host);
  state.properties = state.properties || {};

  for (const k in computed) {
    const previousValue = hostVariables[PROPS][k];
    state.properties[k] = computeProperty(k, computed[k], {
      state,
      host
    });
    hostVariables[PROPS][k] = state.properties[k]; // eslint-disable-line no-param-reassign

    if (!quickEquals(state.properties[k], previousValue)) {
      host[UPDATE_AFFECTED_COMPONENTS]({
        name: k,
        propertyDetails: {
          previousValue,
          value: state.properties[k]
        },
        interactionId,
        includeSource: false
      });
    }
  }

  return state;
}
//# sourceMappingURL=resolveComputedProperties.js.map