/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

const shouldReturnEmptyString = (schema = {}) => schema.type === 'string' || Array.isArray(schema.type) && schema.type.indexOf('string') !== -1 || schema.oneOf && schema.oneOf.find(({
  type
}) => type === 'string');
/**
 * @private
 * @param {*} value
 * @param {*} defaultValue
 * @returns resolved safe string value
 */


const fetchSafeStringValue = (value, defaultValue) => {
  if (value || defaultValue === undefined) return value;
  return defaultValue;
};
/**
 * @private
 * @param {*} propertyDefinition
 * @param {*} values
 * @param {*} defaultValue
 * @returns resolved value
 **/


const resolveValue = (propertyDefinition, values, defaultValue) => {
  const {
    length: valueCount
  } = values;
  var hasSchema = propertyDefinition === null || propertyDefinition === void 0 ? void 0 : propertyDefinition.schema;

  for (let i = 0; i < valueCount; i++) {
    const value = values[i];

    if (typeof value === 'string') {
      if (!hasSchema) return fetchSafeStringValue(value, defaultValue);else if (value.length || propertyDefinition.schema && shouldReturnEmptyString(propertyDefinition.schema)) return value;
    } else {
      if (value !== undefined && !propertyDefinition.readOnly) return value;
    }
  }

  return defaultValue;
};

export default function resolveProperties(state = {
  properties: {}
}, propertyDefinitions, instanceProperties = {}) {
  const resolvedProperties = {};

  for (const k in propertyDefinitions) {
    resolvedProperties[k] = resolveValue(propertyDefinitions[k], [instanceProperties[k], state.properties[k]], propertyDefinitions[k].default);
  }

  return resolvedProperties;
}
//# sourceMappingURL=resolveProperties.js.map