/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { getConfigItem, configConstants } from '@servicenow/ui-config';
import Ajv from 'ajv';
const {
  VALIDATE_SCHEMAS
} = configConstants;
const validateSchemas = getConfigItem(VALIDATE_SCHEMAS);
let ajv;

if (validateSchemas) {
  ajv = new Ajv();
  ajv.addKeyword('fn', {
    validate: function validateFn(schema, data) {
      if (data instanceof Function) return true;
      if (data === undefined || data === null) return true;
      validateFn.errors = [{
        message: 'should be a function',
        params: {
          level: 'error'
        }
      }];
      return false;
    }
  });
  ajv.addKeyword('deprecated', {
    validate: function validateDeprecation(schema) {
      if (!schema) return true;
      const message = typeof schema === 'boolean' ? 'has been deprecated' : schema;
      validateDeprecation.errors = [{
        message,
        params: {
          level: 'warn'
        }
      }];
      return false;
    }
  });
} else {
  ajv = {
    compile: () => () => true
  };
}

export default ajv;
//# sourceMappingURL=ajv.js.map