/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import ajv from '../ajv';
import getComponentName from '../getComponentName';
import validateSchema from '../validate/validateSchema';
/**
 * Build the validator for the action payload schema
 * @private
 * @param {*} payloadSchema The schema provided by the user in config.actions[key].schema
 * @returns {Function(value, host)} The function to validate the action's payload
 */

function buildActionPayloadValidator(payloadSchema) {
  if (process.env.NODE_ENV !== 'production') {
    let validate;

    try {
      validate = ajv.compile(payloadSchema);
    } catch (e) {
      console.error(e); // eslint-disable-line no-console

      return () => false;
    }

    return function validateActionPayload(value, host) {
      const component = getComponentName(host);
      return validateSchema(validate, value, {
        pathPrefix: 'payload',
        errorPrefix: `Action payload for ${component}: `
      });
    };
  }

  return () => true;
}

export default buildActionPayloadValidator;
//# sourceMappingURL=buildActionPayloadValidator.js.map