/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { getConfigItem, configConstants } from '@servicenow/ui-config';
import ajv from '../ajv';
import validateSchema from '../validate/validateSchema';
const {
  VALIDATE_SCHEMAS
} = configConstants;
const validateSchemas = getConfigItem(VALIDATE_SCHEMAS);
/**
 * Build the validator for the config of a component
 * @private
 * @param {*} schema The schema of the config to validate against
 * @returns {Function(config, errorPrefix, options)} The function to validate the component's properties
 */

function buildConfigValidator(schema) {
  if (validateSchemas) {
    let validate;

    try {
      validate = ajv.compile(schema);
    } catch (e) {
      console.error(e); // eslint-disable-line no-console

      return () => false;
    }

    return function validator(config, errorPrefix, options = {}) {
      return validateSchema(validate, config, {
        errorPrefix,
        ...options
      });
    };
  }

  return () => true;
}

export default buildConfigValidator;
//# sourceMappingURL=buildConfigValidator.js.map