/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import { requestIdleCallback } from '@servicenow/ui-utils';
import log from '../log';
const styleSheetsContent = {};
const PREFIX = 'SEISMIC_STYLES';

const get = (version, tagName) => {
  let contents = styleSheetsContent[tagName];
  if (contents) return contents;

  try {
    const errorText = `<title>Error</title>`;
    contents = localStorage.getItem(`${PREFIX}_${tagName}${version}`);
    if (contents.includes(errorText)) contents = '';
  } catch (e) {
    log(`The stylesheets content for the tag, ${tagName}, could not be retrieved.`);
  }

  return contents;
};

const set = (version, tagName, contents) => {
  try {
    styleSheetsContent[tagName] = contents;
    requestIdleCallback(() => {
      localStorage.setItem(`${PREFIX}_${tagName}${version}`, contents);
    });
  } catch (e) {
    log(`The stylesheets content for the custom element, ${name}, could not be cached.`);
  }
};

export default {
  set,
  get
};
//# sourceMappingURL=styleSheetsContentCache.js.map