/**
 * Copyright (c) 2020 ServiceNow, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *  
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *  
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
import triggerPropertyChangeAction from './triggerPropertyChangeAction';
import { HOIST_APPEND_TO_PROP, HOIST_ATTR, NOW_HOIST_APPEND_TO_PROP, NOW_HOIST_PREPEND_TO_PROP, NOW_HOIST_PROP, PROPERTY_SELECTOR_SIGIL } from '../constants';
import { HOIST_CHANGED, HOIST_PARENT_CHANGED, PROPERTY_DEFS, RENDER, VARIABLES } from '../symbols';
import log from '../log';
import { isPropToSkipRender } from './isPropToSkipRender';
export default function triggerComponentUpdates(componentRegistry, affected, propertyDetails, interactionId, label) {
  const {
    value
  } = propertyDetails;
  const renderList = [];

  for (const id in affected) {
    var _el$VARIABLES, _el$VARIABLES$PROPERT;

    const [elementId, propName] = id.split(PROPERTY_SELECTOR_SIGIL);
    const el = componentRegistry[elementId];
    renderList.push({
      elementId,
      tagName: el === null || el === void 0 ? void 0 : el.tagName,
      propName,
      propDef: el === null || el === void 0 ? void 0 : (_el$VARIABLES = el[VARIABLES]) === null || _el$VARIABLES === void 0 ? void 0 : (_el$VARIABLES$PROPERT = _el$VARIABLES[PROPERTY_DEFS]) === null || _el$VARIABLES$PROPERT === void 0 ? void 0 : _el$VARIABLES$PROPERT[propName]
    });

    if (el && propName) {
      triggerPropertyChangeAction(el, propertyDetails, {
        propName,
        path: affected[id]
      }, interactionId);
      if (propName === NOW_HOIST_PROP || propName === HOIST_ATTR) el[HOIST_CHANGED](value);
      if (propName === HOIST_APPEND_TO_PROP || propName === NOW_HOIST_APPEND_TO_PROP || propName === NOW_HOIST_PREPEND_TO_PROP) el[HOIST_PARENT_CHANGED](propName);
    }
  }

  for (let i = 0; i < renderList.length; i++) {
    const {
      elementId,
      tagName,
      propName,
      propDef
    } = renderList[i];

    if (componentRegistry[elementId]) {
      if (!isPropToSkipRender(tagName, propName, propDef)) componentRegistry[elementId][RENDER](interactionId, label, {
        propName,
        value
      });
    } else {
      log(`Component with id "${elementId}" could not be re-rendered from a change in property "${propName}" because the component is disconnected or not known.`, {
        componentId: elementId,
        origin: 'triggerComponentUpdates'
      });
    }
  }
}
//# sourceMappingURL=triggerComponentUpdates.js.map